        subroutine adapt_4000_sub(SN,TEMSC,NBANDS,IEL,IEH,IBIN,
     *          IPART,ISOFT,ISWCX,IMASK,INFILE,IFILL)
C
C  SAS Version 12.0
C  Version 4.2  21 February 2012
C
C  Makes adaptively smoothed images
C
C  adapt-900 created adaptively smothed exposure corrected and
C  background subtracted images for the XMM-ESAS package.
C
C  INPUT:
C    SN: Number of counts for smoothing
C    temsc: Threshold for masking
C    nbands: Number of bands to merge
C    iel: List of lower energy limits for the energy bands in eV
C    ieh: List of upper energy limits for the energy bands in eV
C    ibin: 1: No binning, 2: Bin by 2, 4: Bin by 4
C    ipart: Particle background control - 1: include, 2: exclude
C    isoft: Soft proton background control - 1: include, 2: exclude
C    imask: Mask control - 0: No additional masking
C          1: Read in the mask file name
C    infile: Mask file name (only if the mask control is 1)
C    ifill: Number of passes to fill in empty pixels
C
C    Examples: adapt_4000 temsc=0.02 nbands=1 iel=4000 ieh=7200 
C                   ibin=2 ipart=1 isoft=0 imask=0 ifill=0
C              adapt_4000 temsc=0.02 nbands=2 iel="400 750" 
C                   ieh="750 1250" ipart=1 isoft=1 imask=1 
C                   mask=mask.fits ifill=2
C    Number of passes to fill in empty pixels
C
C  OUPUT:
C    Adaptively smoothed image and radial profile QDP plot
C
        IMPLICIT NONE
C
        INTEGER*2 INI2(4000,4000)
        INTEGER CLOBBER, INI4(4000,4000), ISIZE, I, I2, IB, IBIN,
     +      IEL(10), IELMIN, IEH(10), IEHMAX, IFILL, IFLAG, II, 
     +      IMASK, IOS, IPART, IPEEL, IR, ISOFT, ISWCX, 
     +      J, J1, J2, JJ, JJ1, JJ2, K, LUN, 
     +      NBANDS, NN, NOFF, STATUS
C
        REAL AREA, B, BACK(4000,4000), BLAST,
     +      BR(2000), BZERO, BSCALE, C, CLAST, COUNTS(4000,4000),
     +      CR(2000), CTS, CX, E, ELAST, ER(2000), EXPOSE(4000,4000),
     +      MASK(4000,4000), NUM, NUMLAST, OUTPUT(4000,4000),
     +      OUT200(2000,2000), OUT400(1000,1000), PROT(4000,4000),
     +      QPB, R, RAD, RN(2000), SCA, SIZE(4000,4000), SN, SP, 
     +      SR(2000), SWCX(4000,4000), T, TEMP, TLIM, TN, TRAD,
     +      TEMSC, XH(2000), XR(2000), XMAX, YMAX, YH(2000), YR(2000)
C
        DOUBLE PRECISION CDELT(2), CRPIX(2), CRVAL(2)
C
        CHARACTER*8  CTYPE(2), UNITS
        CHARACTER*60 COMMENT
        CHARACTER*132 INFILE, INFILES
C
        EQUIVALENCE (OUTPUT,OUT200,OUT400)
        EQUIVALENCE (EXPOSE,INI4)
C
        DATA CLOBBER /1/
        DATA STATUS /0/
C
        DO I=1,2000
            XH(I) = -0.1 + I/15.0
            YH(I) = 0.0
        ENDDO
C
C    Example: adapt_4000 100 0.02 2 350 800 800 1250 4 1 0 0 1 mask.fits 0
C
C
C  Set the scale factor for the maps, this is the ratio of
C  one square degree to the solid angle of the original map pixel
C
        SCA = (60.0*60.0)*(60.0*60.0)/(2.5*2.5)
C
C  Enter number of counts for smoothing.  The program smoothes using
C  a circular region surrounding the pixel whose radius is increased
C  until the selected number of counts are enclosed.
C
        PRINT *, 'Number of counts for smoothing: ', SN
C
C  Enter the fractional limit for template, the program will exclude
C  all pixels in the field where the mask value is less than the
C  fractional limit times the average non-zero value of the mask
C
        PRINT *, 'Fractional limit for template: ', TEMSC
C
C  Enter the number of bands to add
C
        NN = 3
        DO IB=1,NBANDS
            NOFF = (IB - 1)*2
            NN = 3 + NOFF + 1
            NN = 3 + NOFF + 2
            PRINT *, 'Band: ', IB,'  Energy Limits: ',
     +                  IEL(IB), IEH(IB)
        ENDDO
        IELMIN = 10000
        IEHMAX = 0
        DO IB=1,NBANDS
            IF(IEL(IB) .LT. IELMIN) IELMIN = IEL(IB)
            IF(IEH(IB) .GT. IEHMAX) IEHMAX = IEH(IB)
        ENDDO
C
C  Enter the binning control
C
        NN = NN + 1
C
C  Create the count array file names and read in the data
C
 1000   FORMAT(A80)
C
        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3100) IEL(IB), IEH(IB)
 3100           FORMAT('obj-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3110) IEL(IB), IEH(IB)
 3110               FORMAT('obj-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3120) IEL(IB), IEH(IB)
 3120               FORMAT('obj-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3130) IEL(IB), IEH(IB)
 3130               FORMAT('obj-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3140) IEL(IB), IEH(IB)
 3140               FORMAT('obj-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3150) IEL(IB), IEH(IB)
 3150               FORMAT('obj-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF
C
            IF(IB .EQ. 1) THEN
                CALL RETFIT (INFILE,COUNTS,INI2,INI4,1,4000,
     +                  4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                  4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,4000
                    DO II=1,4000
                        COUNTS(I,II) = COUNTS(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 1'
                PRINT *, 'Event image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ENDDO
        CTS = 0.0
        DO I=1,4000
            DO II=1,4000
                CTS = CTS + COUNTS(I,II)
            ENDDO
        ENDDO
C
C  Enter the exposure array file names and read in the data
C
        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3200) IEL(IB), IEH(IB)
 3200           FORMAT('exp-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3210) IEL(IB), IEH(IB)
 3210               FORMAT('exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3220) IEL(IB), IEH(IB)
 3220               FORMAT('exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3230) IEL(IB), IEH(IB)
 3230               FORMAT('exp-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3240) IEL(IB), IEH(IB)
 3240               FORMAT('exp-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3250) IEL(IB), IEH(IB)
 3250               FORMAT('exp-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF
C
            IF(IB .EQ. 1) THEN
                CALL RETFIT (INFILE,EXPOSE,INI2,INI4,1,4000,
     +                  4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                  4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,4000
                    DO II=1,4000
                        EXPOSE(I,II) = EXPOSE(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 2'
                PRINT *, 'Exposure image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ENDDO
        IF(NBANDS .GT. 1) THEN
            DO I=1,4000
                DO II=1,4000
                    EXPOSE(I,II) = EXPOSE(I,II)/NBANDS
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the background count array file names and read in the data
C
        IF(IPART .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3300) IEL(IB), IEH(IB)
 3300               FORMAT('back-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3310) IEL(IB), IEH(IB)
 3310                   FORMAT('back-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3320) IEL(IB), IEH(IB)
 3320                   FORMAT('back-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3330) IEL(IB), IEH(IB)
 3330                   FORMAT('back-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3340) IEL(IB), IEH(IB)
 3340                   FORMAT('back-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3350) IEL(IB), IEH(IB)
 3350                   FORMAT('back-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,BACK,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,4000
                        DO II=1,4000
                            BACK(I,II) = BACK(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'ADAPT_4000 Position 3'
                    PRINT *, 'Particle background image file ',
     +                      'STATUS: ', STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,4000
                DO II=1,4000
                    BACK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
        QPB = 0.0
        DO I=1,4000
            DO II=1,4000
                QPB = QPB + BACK(I,II)
            ENDDO
        ENDDO
C
C  Enter the soft proton count array file names and read in the data
C
        IF(ISOFT .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3400) IEL(IB), IEH(IB)
 3400               FORMAT('prot-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3410) IEL(IB), IEH(IB)
 3410                   FORMAT('prot-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3420) IEL(IB), IEH(IB)
 3420                   FORMAT('prot-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEL(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3430) IEL(IB), IEH(IB)
 3430                   FORMAT('prot-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3440) IEL(IB), IEH(IB)
 3440                   FORMAT('prot-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3450) IEL(IB), IEH(IB)
 3450                   FORMAT('prot-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,PROT,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,4000
                        DO II=1,4000
                            PROT(I,II) = PROT(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'ADAPT_4000 Position 4'
                    PRINT *, 'Soft proton background image file ', 
     +                      'STATUS: ', STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,4000
                DO II=1,4000
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
        SP = 0.0
        DO I=1,4000
            DO II=1,4000
                SP = SP + PROT(I,II)
            ENDDO
        ENDDO
C
C  Enter the solar wind charge exchange count array file names and 
C  read in the data
C
        IF(ISWCX .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3500) IEL(IB), IEH(IB)
 3500               FORMAT('swcx-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3510) IEL(IB), IEH(IB)
 3510                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3520) IEL(IB), IEH(IB)
 3520                   FORMAT('swcx-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEL(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3530) IEL(IB), IEH(IB)
 3530                   FORMAT('swcx-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3540) IEL(IB), IEH(IB)
 3540                   FORMAT('swcx-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3550) IEL(IB), IEH(IB)
 3550                   FORMAT('swcx-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN
                    CALL RETFIT (INFILE,SWCX,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                      4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,4000
                        DO II=1,4000
                            SWCX(I,II) = SWCX(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'ADAPT_4000 Position 4'
                    PRINT *, 'Solar wind charge exchange background ',
     +                      'image file ', 'STATUS: ', STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,4000
                DO II=1,4000
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
        CX = 0.0
        DO I=1,4000
            DO II=1,4000
                CX = CX + SWCX(I,II)
            ENDDO
        ENDDO
C
C  Add the background components
C
        DO I=1,4000
            DO II=1,4000
                BACK(I,II) =  BACK(I,II) + PROT(I,II) + SWCX(I,II)
            ENDDO
        ENDDO
        PRINT *, 'Observed Counts: ', CTS
        PRINT *, 'QPB Counts: ', QPB
        PRINT *, 'SP Counts: ', SP
        PRINT *, 'SWCX Counts: ', CX
C
C  Read in the template
C
        IF(IMASK .EQ. 1) THEN
C
C  Return the mask
C
            CALL RETFIT (INFILE,MASK,INI2,INI4,1,4000,
     +                  4000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 5'
                PRINT *, 'MASK image file ', 
     +                  'STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
            DO I=1,4000
                DO II=1,4000
                    IF(EXPOSE(I,II) .LE. 0.0) MASK(I,II) = 0.0
                ENDDO
            ENDDO
        ELSE
C
C  Use the exposure map as the template
C
            DO I=1,4000
                DO II=1,4000
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ENDIF
C
C  Set the template limit
C
        J = 0
        TLIM = 0.
        DO I=1,4000
            DO II=1,4000
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J
C
C  Read in the filling control
C
        NN = NN + 1
C
C  Make a radial profile
C
        DO I=1,2000
            BR(I) = 0.0
            CR(I) = 0.0
            ER(I) = 0.0
            RN(I) = 0.0
        ENDDO
        DO I=1,4000
            DO II=1,4000
                IF(MASK(I,II) .GT. TLIM) THEN
                    R = SQRT((I-2000.5)**2 + (II-2000.5)**2)
                    IR = 60.0*R*CDELT(2) + 1.0
c                    IR = 2.0*60.0*R*CDELT(2)
                    IF(IR .GT. 2000) IR = 2000
                    BR(IR) = BR(IR) + BACK(I,II)
                    CR(IR) = CR(IR) + COUNTS(I,II)
                    ER(IR) = ER(IR) + EXPOSE(I,II)
                    RN(IR) = RN(IR) + 1.0
                ENDIF
            ENDDO
        ENDDO
        XMAX = 0.0
        YMAX = 0.0
        DO I=1,2000
            IF(ER(I) .GT. 0.0) THEN
                YR(I) = (CR(I) - BR(I))/ER(I)
                SR(I) = SQRT(CR(I))/ER(I)
                XR(I) = (I - 0.5)
                IF((YR(I)) .GT. YMAX) YMAX = YR(I)
                IF(XR(I) .GT. XMAX) XMAX = XR(I)
            ELSE
                YR(I) = 0.0
                SR(I) = 0.0
                XR(I) = (I - 0.5)/2.0
            ENDIF
        ENDDO
C
C  Write out the new FITS adaptively smoothed image file
C
        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3600) IELMIN, IEHMAX
 3600       FORMAT('radial-',I5,'-',I5,'.qdp')
            WRITE(INFILES,3605) IELMIN, IEHMAX
 3605       FORMAT('size-',I5,'-',I5,'.qdp')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3610) IELMIN, IEHMAX
 3610           FORMAT('radial-',I4,'-',I5,'.qdp')
                WRITE(INFILES,3615) IELMIN, IEHMAX
 3615           FORMAT('size-',I4,'-',I5,'.qdp')
            ELSE
                WRITE(INFILE,3620) IELMIN, IEHMAX
 3620           FORMAT('radial-',I4,'-',I4,'.qdp')
                WRITE(INFILES,3625) IELMIN, IEHMAX
 3625           FORMAT('size-',I4,'-',I4,'.qdp')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3630) IELMIN, IEHMAX
 3630           FORMAT('radial-',I3,'-',I5,'.qdp')
                WRITE(INFILES,3635) IELMIN, IEHMAX
 3635           FORMAT('size-',I3,'-',I5,'.qdp')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3640) IELMIN, IEHMAX
 3640           FORMAT('radial-',I3,'-',I4,'.qdp')
                WRITE(INFILES,3645) IELMIN, IEHMAX
 3645           FORMAT('size-',I3,'-',I4,'.qdp')
            ELSE
                WRITE(INFILE,3650) IELMIN, IEHMAX
 3650           FORMAT('radial-',I3,'-',I3,'.qdp')
                WRITE(INFILES,3655) IELMIN, IEHMAX
 3655           FORMAT('size-',I3,'-',I3,'.qdp')
            ENDIF
        ENDIF
C
C  Get an unused lun
C
        call ftgiou(lun,status)
        if(status .ne. 0) then
            print *, 'STATUS FTGIOU =', status
            print *, 'ADAPT_4000 POSITION 1'
            stop
        endif
C
        IF(CLOBBER .EQ. 1) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED',
     +          FILE=INFILE)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILE,
     +          IOSTAT=IOS)
            IF(IOS .EQ. 0) THEN
                PRINT *, 'Output file already exists and no clobber'
                STOP
            ENDIF
        ENDIF
C
        YMAX = 1.05*YMAX
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CS 1.5'
        WRITE(LUN,*) 'READ SERR 2'
        WRITE(LUN,*) 'LA Y Count Rate (counts/s)'
        WRITE(LUN,*) 'LA X Radius'
        WRITE(LUN,*) 'LINE OFF 2'
        WRITE(LUN,*) 'R X 0 ', XMAX
        WRITE(LUN,*) 'R Y 0 ', YMAX
        WRITE(LUN,*) 'LW 2'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA T Radial Profile'
        DO I=1,2000
            WRITE(LUN,*) XR(I), YR(I), SR(I)
        ENDDO
        CLOSE(LUN)
C
C  Set the binning limits
C
        IF(IBIN .EQ. 1) THEN
            I2 = 4000
        ELSEIF(IBIN .EQ. 2) THEN
            I2 = 2000
        ELSEIF(IBIN .EQ. 4) THEN
            I2 = 1000
        ENDIF
C
C  Bin the data
C
        PRINT *, 'START THE BINNING'
        DO I=1,I2
            J2 = I*IBIN
            J1 = J2 - IBIN + 1
            DO II=1,I2
                JJ2 = II*IBIN
                JJ1 = JJ2 - IBIN + 1
                B = 0.
                C = 0.
                E = 0.
                DO J=J1,J2
                    DO JJ=JJ1,JJ2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO
                BACK(I,II) = B
                COUNTS(I,II) = C
                EXPOSE(I,II) = E
            ENDDO
        ENDDO
C
C  Now smooth the data
C
        PRINT *, 'START THE SMOOTHING'
        DO I=1,I2
            IF(MOD(I,25) .EQ. 0) PRINT *, I
            DO II=1,I2
                IF(EXPOSE(I,II) .GT. 0.) THEN
                    B = 0.
                    C = 0.
                    E = 0.
                    NUM = 0
                    K = -1
                    DO WHILE (C .LT. SN)
                        BLAST = B
                        CLAST = C
                        ELAST = E
                        NUMLAST = NUM
                        K = K + 1
                        IF(K .EQ. 1) THEN
                            RAD = 1.5*1.5
                        ELSEIF(K .EQ. 2) THEN
                            RAD = 2.5*2.5
                        ELSEIF(K .EQ. 3) THEN
                            RAD = 3.2*3.2
                        ELSE
                            RAD = K*K*1.05*1.05
                        ENDIF
                        J1 = I - K
                        IF(J1 .LT. 1) J1 = 1
                        J2 = I + K
                        IF(J2 .GT. I2) J2 = I2
                        JJ1 = II - K
                        IF(JJ1 .LT. 1) JJ1 = 1
                        JJ2 = II + K
                        IF(JJ2 .GT. I2) JJ2 = I2
                        B = 0.
                        C = 0.
                        E = 0.
                        NUM = 0
                        DO J=J1,J2
                            DO JJ=JJ1,JJ2
                                TRAD = (J-I)**2 + (JJ-II)**2
c                                TRAD = SQRT(TRAD)
                                IF((EXPOSE(J,JJ) .GE. 0.) .AND.
     +                                  (TRAD .LE. RAD)) THEN
                                    IF(RAD .EQ. 0.0) THEN
                                        B = B + BACK(J,JJ)
                                        C = C + COUNTS(J,JJ)
                                        E = E + EXPOSE(J,JJ)
                                    ELSE
c                                        T = EXP(-((TRAD/RAD)**2))
c                                        B = B + BACK(J,JJ)*T
c                                        C = C + COUNTS(J,JJ)*T
c                                        E = E + EXPOSE(J,JJ)*T
c
c                                        B = B + BACK(J,JJ)
c                                        C = C + COUNTS(J,JJ)
c                                        E = E + EXPOSE(J,JJ)
c
                                        B = B + BACK(J,JJ)*
     +                                      (RAD - TRAD/1.0)/RAD
                                        C = C + COUNTS(J,JJ)*
     +                                      (RAD - TRAD/1.0)/RAD
                                        E = E + EXPOSE(J,JJ)*
     +                                      (RAD - TRAD/1.0)/RAD
                                    ENDIF
                                    NUM = NUM + 1
                                ENDIF
                            ENDDO
                        ENDDO
                    ENDDO
                    IF((K .GT. 1) .AND. (NUMLAST .GT. 0.)) THEN
                        TEMP = SCA*(CLAST - BLAST)/ELAST
                        IF(IBIN .EQ. 1) THEN
                            OUTPUT(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 2) THEN
                            OUT200(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 4) THEN
                            OUT400(I,II) = TEMP
                        ENDIF
                    ELSEIF(NUM .GT. 0.) THEN
                        TEMP = SCA*(C - B)/E
                        IF(IBIN .EQ. 1) THEN
                            OUTPUT(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 2) THEN
                            OUT200(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 4) THEN
                            OUT400(I,II) = TEMP
                        ENDIF
                    ELSE
                        IF(IBIN .EQ. 1) THEN
                            OUTPUT(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 2) THEN
                            OUT200(I,II) = TEMP
                        ELSEIF(IBIN .EQ. 4) THEN
                            OUT400(I,II) = TEMP
                        ENDIF
                    ENDIF
                    RAD = 60.0*CDELT(2)*SQRT(RAD)
                    SIZE(I,II) = RAD
                    IR = 15*RAD + 1
                    YH(IR) = YH(IR) + 1
                ENDIF
            ENDDO
        ENDDO
C
C  Get an unused lun
C
        call ftgiou(lun,status)
        if(status .ne. 0) then
            print *, 'STATUS FTGIOU =', status
            print *, 'ADAPT_4000 POSITION 1'
            stop
        endif
C
        IF(CLOBBER .EQ. 1) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED',
     +          FILE=INFILES)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILES,
     +          IOSTAT=IOS)
            IF(IOS .EQ. 0) THEN
                PRINT *, 'Output file already exists and no clobber'
                STOP
            ENDIF
        ENDIF
C
        YMAX = 0.0
        DO I=1,100
            IF(YH(I) .GT. YMAX) YMAX = YH(I)
        ENDDO
        YMAX = 1.05*YMAX
C
        WRITE(LUN,*) 'VIEWPORT 0.15 0.15 0.95 0.85'
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CS 1.8'
        WRITE(LUN,*) 'LA X Conical Sampling FWHM (arcmin)'
        WRITE(LUN,*) 'LA Y Number'
        WRITE(LUN,*) 'LA F ', INFILES
        WRITE(LUN,*) 'COL 1 ON 2'
        WRITE(LUN,*) 'R X 0 5'
        WRITE(LUN,*) 'R Y 0 ', YMAX
        WRITE(LUN,*) 'LW 2'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA T Smoothing FWHM Histogram'
        WRITE(LUN,*) 'LINE STEP'
        DO I=1,2000
            WRITE(LUN,*) XH(I), YH(I)
        ENDDO
        CLOSE(LUN)
C
C  Write out the new FITS adaptively smoothed image file
C
        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3700) IELMIN, IEHMAX
 3700       FORMAT('adapt-',I5,'-',I5,'.fits')
            WRITE(INFILES,3705) IELMIN, IEHMAX
 3705       FORMAT('size-',I5,'-',I5,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3710) IELMIN, IEHMAX
 3710           FORMAT('adapt-',I4,'-',I5,'.fits')
                WRITE(INFILES,3715) IELMIN, IEHMAX
 3715           FORMAT('size-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3720) IELMIN, IEHMAX
 3720           FORMAT('adapt-',I4,'-',I4,'.fits')
                WRITE(INFILES,3725) IELMIN, IEHMAX
 3725           FORMAT('size-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3730) IELMIN, IEHMAX
 3730           FORMAT('adapt-',I3,'-',I5,'.fits')
                WRITE(INFILES,3735) IELMIN, IEHMAX
 3735           FORMAT('size-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3740) IELMIN, IEHMAX
 3740           FORMAT('adapt-',I3,'-',I4,'.fits')
                WRITE(INFILES,3745) IELMIN, IEHMAX
 3745           FORMAT('size-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3750) IELMIN, IEHMAX
 3750           FORMAT('adapt-',I3,'-',I3,'.fits')
                WRITE(INFILES,3755) IELMIN, IEHMAX
 3755           FORMAT('size-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
        COMMENT = 'Adaptive-filter smoothed count-rate image for XMM'
C
        IFLAG = 1
        ISIZE = 4000
        IF(IBIN .EQ. 2) THEN
            CDELT(1) = 2.0*CDELT(1)
            CDELT(2) = 2.0*CDELT(2)
            CRPIX(1) = 1000.5
            CRPIX(2) = 1000.5
            ISIZE = 2000
        ELSEIF(IBIN .EQ. 4) THEN
            CDELT(1) = 4.0*CDELT(1)
            CDELT(2) = 4.0*CDELT(2)
            CRPIX(1) = 500.5
            CRPIX(2) = 500.5
            ISIZE = 1000
        ENDIF
        UNITS = 'c/s/deg2'
        BZERO = 0.0
        BSCALE = 1.0
        AREA = 60.0*CDELT(2)
C
        IPEEL = IFILL - 1
        IF(IFILL .GT. 0) THEN
            PRINT *, 'Starting infill process'
        ENDIF
        IF(IBIN .EQ. 1) THEN
            IF(IFILL .GT. 0) THEN
                DO IB=1,IFILL
                    PRINT *, 'Starting infill number', IB
                    DO I=2,ISIZE-1
                        IF(MOD(I,100) .EQ. 0) PRINT *, I
                        DO II=2,ISIZE-1
                            IF(OUTPUT(I,II) .EQ. 0.0) THEN
                                T = 0.0
                                TN = 0.0
                                DO J=I-1,I+1
                                    DO JJ=II-1,II+1
                                        IF(OUTPUT(J,JJ) .NE. 0.0) THEN
                                            T = T + OUTPUT(J,JJ)
                                            TN = TN + 1
                                        ENDIF
                                    ENDDO
                                ENDDO
                                IF(TN .GE. 3.0) THEN
                                    COUNTS(I,II) = T/TN
                                ELSE
                                    COUNTS(I,II) = 0.0
                                ENDIF
                            ELSE
                                COUNTS(I,II) = OUTPUT(I,II)
                            ENDIF
                        ENDDO
                    ENDDO
                    DO I=2,ISIZE-1
                        DO II=2,ISIZE-1
                            OUTPUT(I,II) = COUNTS(I,II)
                        ENDDO
                    ENDDO
                    PRINT 2100, IB
 2100               FORMAT(1X,'Infill pass',I3,' complete')
                ENDDO
 2200               FORMAT(1X,'Peal pass',I3,' complete')
            ENDIF
            CALL PUTFIT(INFILE,OUTPUT,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 6'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ELSEIF(IBIN .EQ. 2) THEN
            IF(IFILL .GT. 0) THEN
C
C  Fill in the holes and around the outside
C
                DO IB=1,IFILL
                    PRINT *, 'Starting infill number', IB
                    DO I=2,ISIZE-1
                        IF(MOD(I,100) .EQ. 0) PRINT *, I
                        DO II=2,ISIZE-1
                            IF(OUT200(I,II) .EQ. 0.0) THEN
                                T = 0.0
                                TN = 0.0
                                DO J=I-1,I+1
                                    DO JJ=II-1,II+1
                                        IF(OUT200(J,JJ) .NE. 0.0) THEN
                                            T = T + OUT200(J,JJ)
                                            TN = TN + 1
                                        ENDIF
                                    ENDDO
                                ENDDO
                                IF(TN .GE. 3.0) THEN
                                    COUNTS(I,II) = T/TN
                                ELSE
                                    COUNTS(I,II) = 0.0
                                ENDIF
                            ELSE
                                COUNTS(I,II) = OUT200(I,II)
                            ENDIF
                        ENDDO
                    ENDDO
                    DO I=2,ISIZE-1
                        DO II=2,ISIZE-1
                            OUT200(I,II) = COUNTS(I,II)
                        ENDDO
                    ENDDO
                    PRINT 2100, IB
                ENDDO
C
C  Now peel back the outside
C
                DO IB=1,IPEEL
                    PRINT *, 'Starting peel number', IB
                    DO I=2,ISIZE-1
                        IF(MOD(I,100) .EQ. 0) PRINT *, I
                        DO II=2,ISIZE-1
                            IF(OUT200(I,II) .GT. 0.0) THEN
                                TN = 0.0
                                DO J=I-1,I+1
                                    DO JJ=II-1,II+1
                                        IF(OUT200(J,JJ) .NE. 0.0) THEN
                                            TN = TN + 1
                                        ENDIF
                                    ENDDO
                                ENDDO
                                IF(TN .LT. 7.0) THEN
                                    COUNTS(I,II) = 0.0
                                ELSE
                                    COUNTS(I,II) = OUT200(I,II)
                                ENDIF
                            ELSE
                                COUNTS(I,II) = OUT200(I,II)
                            ENDIF
                        ENDDO
                    ENDDO
                    DO I=2,ISIZE-1
                        DO II=2,ISIZE-1
                            OUT200(I,II) = COUNTS(I,II)
                        ENDDO
                    ENDDO
                PRINT 2200, IB
                ENDDO
            ENDIF
            CALL PUTFIT(INFILE,OUT200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 7'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ELSEIF(IBIN .EQ. 4) THEN
            IF(IFILL .GT. 0) THEN
                DO IB=1,IFILL
                    PRINT *, 'Starting infill number', IB
                    DO I=2,ISIZE-1
                        IF(MOD(I,100) .EQ. 0) PRINT *, I
                        DO II=2,ISIZE-1
                            IF(OUT400(I,II) .EQ. 0.0) THEN
                                T = 0.0
                                TN = 0.0
                                DO J=I-1,I+1
                                    DO JJ=II-1,II+1
                                        IF(OUT400(J,JJ) .NE. 0.0) THEN
                                            T = T + OUT400(J,JJ)
                                            TN = TN + 1
                                        ENDIF
                                    ENDDO
                                ENDDO
                                IF(TN .GE. 3.0) THEN
                                    COUNTS(I,II) = T/TN
                                ELSE
                                    COUNTS(I,II) = 0.0
                                ENDIF
                            ELSE
                                COUNTS(I,II) = OUT400(I,II)
                            ENDIF
                        ENDDO
                    ENDDO
                    DO I=2,ISIZE-1
                        DO II=2,ISIZE-1
                            OUT400(I,II) = COUNTS(I,II)
                        ENDDO
                    ENDDO
                    PRINT 2100, IB
                ENDDO
            ENDIF
            CALL PUTFIT(INFILE,OUT400,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 8'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ENDIF
C
        COMMENT = 'Adaptive-filter smoothing kernal'
C
        IFLAG = 1
        UNITS = 'arcmin'
        BZERO = 0.0
        BSCALE = 1.0
C
        IF(IBIN .EQ. 1) THEN
C            DO I=1,ISIZE
C                DO II=1,ISIZE
C                    SIZE(I,II) = AREA*SIZE(I,II)
C                ENDDO
C            ENDDO
            CALL PUTFIT(INFILES,SIZE,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 8'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILEs
                STOP
            ENDIF
        ELSEIF(IBIN .EQ. 2) THEN
            DO I=1,ISIZE
                DO II=1,ISIZE
                    OUT200(I,II) = SIZE(I,II)
                ENDDO
            ENDDO
            CALL PUTFIT(INFILES,OUT200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 8'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILEs
                STOP
            ENDIF
        ELSEIF(IBIN .EQ. 4) THEN
            DO I=1,ISIZE
                DO II=1,ISIZE
                    OUT400(I,II) = SIZE(I,II)
                ENDDO
            ENDDO
            CALL PUTFIT(INFILES,OUT400,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'ADAPT_4000 Position 8'
                PRINT *, 'MASK image file STATUS: ', STATUS
                PRINT *, INFILEs
                STOP
            ENDIF
        ENDIF        
C
        IF (STATUS .LE. 0) THEN
            PRINT *,'*** Program completed successfully ***'
        ELSE
            PRINT *, STATUS
            PRINT *,'*** ERROR - program did not run successfully ***'
        ENDIF
C
        RETURN
        END SUBROUTINE adapt_4000_sub
