        subroutine bin_image_merge_sub(temsc,nbands,iel,ieh,ibin,
     +          ipart,isoft,iswcx,maskfile)
C
C  Version 1.0  11 March 2010
C
C  Bins merged images from the ESAS processed data
C
C  INPUT:
C    temsc: Threshold for masking
C    nbands: Number of bands to merge
C    iel: List of lower energy limits for the energy bands in eV
C    ieh: List of upper energy limits for the energy bands in eV
C    ibin: 1: No binning, 2: Bin by 2, 4: Bin by 4, 8: Bin by 8, 
C          16: Bin by 16, 32: Bin by 32
C    ipart: Particle background control - 1: include, 2: exclude
C    isoft: Soft proton background control - 1: include, 2: exclude
C    maskfile: Mask file name 
C
C    Examples: bin_image_merge thresholdmasking=0.02 nbands=1 elow=2000 ehigh=7200 
C                   ibin=2 withpartcontrol=1 withsoftcontrol=0 
C              bin_image_merge thresholdmasking=0.02 nbands=2 elow="400 750" ehigh="750 1250" 
C                   withpartcontrol=1 withsoftcontrol=1 mask=maskfile.fits
C
C  OUPUT:
C    Binned count rate image and count rate uncertainty image
C
        IMPLICIT NONE
C
        INTEGER*2 INI2(2000,2000)
        INTEGER CLOBBER, INI4(2000,2000), I, I2, 
     +      IB, IBIN, IEL(10), IELMIN, 
     +      IEH(10), IEHMAX, IFLAG, II, IPART, ISIZE, ISOFT, ISWCX, 
     +      J, J1, J2, JJ, JJ1, JJ2, NBANDS, STATUS
C
C        REAL*4 B, BACK(2000,2000), 
        REAL B, BACK(2000,2000), 
     +      BZERO, BSCALE, C, COUNTS(2000,2000), 
     +      E, EXPOSE(2000,2000), 
     +      OUTPUT(2000,2000), OUTSIG(2000,2000), 
     +      OUT200(1000,1000), OUT2SIG(1000,1000), 
     +      OUT400(500,500), OUT4SIG(500,500), 
     +      OUT800(250,250), OUT8SIG(250,250), 
     +      OUT160(125,125), OUT16SIG(125,125), 
     +      OUT320(63,63), OUT32SIG(63,63), PROT(2000,2000), 
     +      R, S, SCA, MASK(2000,2000), SWCX(2000,2000), TLIM, 
     +      TEMSC
C
c        REAL*8 CDELT(2), CRPIX(2), CRVAL(2)
        DOUBLE PRECISION CDELT(2), CRPIX(2), CRVAL(2)
C
        CHARACTER*8  CTYPE(2), UNITS
        CHARACTER*60 COMMENT
        CHARACTER*110 MASKFILE
        CHARACTER*132 INFILE
C
        EQUIVALENCE (OUTPUT,OUT200,OUT400,OUT800,OUT160,OUT160)
        EQUIVALENCE (OUTSIG,OUT2SIG,OUT4SIG,OUT8SIG,OUT32SIG,OUT32SIG)
        EQUIVALENCE (EXPOSE,INI4)
C
        DATA CLOBBER /1/
        DATA STATUS /0/
c
        do i=1,2000
            do ii=1,2000
                output(i,ii) = 0.0
                outsig(i,ii) = 0.0
            enddo
        enddo
C
C  Set the scale factor for the maps, this is the ratio of
C  one square degree to the solid angle of the original map pixel
C
        SCA = (60.0*60.0)*(60.0*60.0)/(2.5*2.5)
C
        PRINT *, 'Fractional Limit for Template: ', TEMSC
        PRINT *, 'Number of Energy Bands to Bin: ', NBANDS
        DO IB=1,NBANDS
            PRINT *, 'Band: ', IB,'  Energy Limits: ', 
     +                  IEL(IB), IEH(IB)
        ENDDO
        IELMIN = 10000
        IEHMAX = 0
        DO IB=1,NBANDS
            IF(IEL(IB) .LT. IELMIN) IELMIN = IEL(IB)
            IF(IEH(IB) .GT. IEHMAX) IEHMAX = IEH(IB)
        ENDDO
        PRINT *, 'Binning Control: ', IBIN
C
C  Create the count array file names and read in the data
C
 1000   FORMAT(A80)
C
        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3100) IEL(IB), IEH(IB)
 3100           FORMAT('obj-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3110) IEL(IB), IEH(IB)
 3110               FORMAT('obj-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3120) IEL(IB), IEH(IB)
 3120               FORMAT('obj-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3130) IEL(IB), IEH(IB)
 3130               FORMAT('obj-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3140) IEL(IB), IEH(IB)
 3140               FORMAT('obj-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3150) IEL(IB), IEH(IB)
 3150               FORMAT('obj-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF
C
            IF(IB .EQ. 1) THEN   
                CALL RETFIT (INFILE,COUNTS,INI2,INI4,1,2000,
     +                  2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI2,INI4,1,2000,
     +                  2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,2000
                    DO II=1,2000
                        COUNTS(I,II) = COUNTS(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            IF(STATUS .NE. 0) THEN
                PRINT *, 'BIN_IMAGE_MERGE Position 1'
                PRINT *, 'Event image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ENDDO
C
C  Enter the exposure array file names and read in the data
C
        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3200) IEL(IB), IEH(IB)
 3200           FORMAT('exp-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3210) IEL(IB), IEH(IB)
 3210               FORMAT('exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3220) IEL(IB), IEH(IB)
 3220               FORMAT('exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3230) IEL(IB), IEH(IB)
 3230               FORMAT('exp-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3240) IEL(IB), IEH(IB)
 3240               FORMAT('exp-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3250) IEL(IB), IEH(IB)
 3250               FORMAT('exp-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF
C
            IF(IB .EQ. 1) THEN   
                CALL RETFIT (INFILE,EXPOSE,INI2,INI4,1,2000,
     +                  2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            ELSE
                CALL RETFIT (INFILE,MASK,INI2,INI4,1,2000,
     +                  2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                DO I=1,2000
                    DO II=1,2000
                        EXPOSE(I,II) = EXPOSE(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
            IF(STATUS .NE. 0) THEN
                PRINT *, 'BIN_IMAGE_MERGE Position 2'
                PRINT *, 'Exposure image file STATUS: ', STATUS
                PRINT *, INFILE
                STOP
            ENDIF
        ENDDO
        IF(NBANDS .GT. 1) THEN
            DO I=1,2000
                DO II=1,2000
                    EXPOSE(I,II) = EXPOSE(I,II)/NBANDS
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the background count array file names and read in the data
C
        IF(IPART .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3300) IEL(IB), IEH(IB)
 3300               FORMAT('back-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3310) IEL(IB), IEH(IB)
 3310                   FORMAT('back-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3320) IEL(IB), IEH(IB)
 3320                   FORMAT('back-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3330) IEL(IB), IEH(IB)
 3330                   FORMAT('back-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3340) IEL(IB), IEH(IB)
 3340                   FORMAT('back-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3350) IEL(IB), IEH(IB)
 3350                   FORMAT('back-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,BACK,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,2000
                        DO II=1,2000
                            BACK(I,II) = BACK(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'BIN_IMAGE_MERGE Position 3'
                    PRINT *, 'Particle background image file STATUS: ', 
     +                      STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,2000
                DO II=1,2000
                    BACK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the proton count array file names and read in the data
C
        IF(ISOFT .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3400) IEL(IB), IEH(IB)
 3400               FORMAT('prot-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3410) IEL(IB), IEH(IB)
 3410                   FORMAT('prot-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3420) IEL(IB), IEH(IB)
 3420                   FORMAT('prot-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3430) IEL(IB), IEH(IB)
 3430                   FORMAT('prot-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3440) IEL(IB), IEH(IB)
 3440                   FORMAT('prot-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3450) IEL(IB), IEH(IB)
 3450                   FORMAT('prot-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,PROT,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,2000
                        DO II=1,2000
                            PROT(I,II) = PROT(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'BIN_IMAGE_MERGE Position 4'
                    PRINT *, 'Soft proton background image file ', 
     +                      'STATUS: ', STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,2000
                DO II=1,2000
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the solar wind charge exchange count array file names and 
C  read in the data
C
        IF(ISWCX .EQ. 1) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3500) IEL(IB), IEH(IB)
 3500               FORMAT('swcx-im-',I4,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3510) IEL(IB), IEH(IB)
 3510                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3520) IEL(IB), IEH(IB)
 3520                   FORMAT('swcx-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3530) IEL(IB), IEH(IB)
 3530                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3540) IEL(IB), IEH(IB)
 3540                   FORMAT('swcx-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3550) IEL(IB), IEH(IB)
 3550                   FORMAT('swcx-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF
C
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,SWCX,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                ELSE
                    CALL RETFIT (INFILE,MASK,INI2,INI4,1,2000,
     +                      2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    DO I=1,2000
                        DO II=1,2000
                            SWCX(I,II) = SWCX(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
                IF(STATUS .NE. 0) THEN
                    PRINT *, 'BIN_IMAGE_MERGE Position 4'
                    PRINT *, 'Solar wind charge exchange background ',
     +                      'image file ', 'STATUS: ', STATUS
                    PRINT *, INFILE
                    STOP
                ENDIF
            ENDDO
        ELSE
            DO I=1,2000
                DO II=1,2000
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Add the background components
C
        DO I=1,2000
            DO II=1,2000
                BACK(I,II) =  BACK(I,II) + PROT(I,II) + SWCX(I,II)
            ENDDO
        ENDDO
C
C  Prepare the template
C
        IF(MASKFILE .EQ. '      ') THEN
C
C  Use the exposure map as the template
C
            DO I=1,2000
                DO II=1,2000
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ELSE
C
C  Read in the mask
C
            CALL RETFIT (MASKFILE,MASK,INI2,INI4,1,2000,
     +                  2000,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'BIN_IMAGE_MERGE Position 5'
                PRINT *, 'MASK image file ', 
     +                  'STATUS: ', STATUS
                PRINT *, MASKFILE
                STOP
            ENDIF
C
C  Make sure that the mask is zero where there is no exposure
C
            DO I=1,2000
                DO II=1,2000
                    IF(EXPOSE(I,II) .LE. 0.0) MASK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Set the template limit
C
        J = 0
        TLIM = 0.
        DO I=1,2000
            DO II=1,2000
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J
C
C  Set the binning limits
C
        IF(IBIN .EQ. 1) THEN
            I2 = 2000
        ELSEIF(IBIN .EQ. 2) THEN
            I2 = 1000
        ELSEIF(IBIN .EQ. 4) THEN
            I2 = 500
        ELSEIF(IBIN .EQ. 8) THEN
            I2 = 250
        ELSEIF(IBIN .EQ. 16) THEN
            I2 = 125
        ELSEIF(IBIN .EQ. 32) THEN
            I2 = 63
        ENDIF
C
C  Bin the data
C
        PRINT *, 'START THE BINNING'
        DO I=1,I2
            J2 = I*IBIN
            J1 = J2 - IBIN + 1
            IF(J2 .GT. 2000) J2 = 2000
            DO II=1,I2
                JJ2 = II*IBIN
                JJ1 = JJ2 - IBIN + 1
                IF(JJ2 .GT. 2000) JJ2 = 2000
                B = 0.
                C = 0.
                E = 0.
                DO J=J1,J2
                    DO JJ=JJ1,JJ2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO
                IF((E .GT. 0.0) .AND. (C .GT. 0.0)) THEN
                    R = SCA*(C - B)/E
                    S = SCA*SQRT(C)/E
                ELSE
                    R = 0.0
                    S = 0.0
                ENDIF
c                IF(R/S .LT. 3.0) THEN
c                    R = 0.0
c                    S = 0.0
c                ENDIF
                IF(IBIN .EQ. 1) THEN
                    OUTPUT(I,II) = R
                    OUTSIG(I,II) = S
                ELSEIF(IBIN .EQ. 2) THEN
                    OUT200(I,II) = R
                    OUT2SIG(I,II) = S
                ELSEIF(IBIN .EQ. 4) THEN
                    OUT400(I,II) = R
                    OUT4SIG(I,II) = S
                ELSEIF(IBIN .EQ. 8) THEN
                    OUT800(I,II) = R
                    OUT8SIG(I,II) = S
                ELSEIF(IBIN .EQ. 16) THEN
                    OUT160(I,II) = R
                    OUT16SIG(I,II) = S
                ELSEIF(IBIN .EQ. 32) THEN
                    OUT320(I,II) = R
                    OUT32SIG(I,II) = S
                ENDIF
            ENDDO
        ENDDO
C
C  Write out the new FITS binned image file
C
        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3600) IELMIN, IEHMAX
 3600       FORMAT('bin-',I4,'-',I4,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3610) IELMIN, IEHMAX
 3610           FORMAT('bin-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3620) IELMIN, IEHMAX
 3620           FORMAT('bin-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3630) IELMIN, IEHMAX
 3630           FORMAT('bin-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3640) IELMIN, IEHMAX
 3640           FORMAT('bin-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3650) IELMIN, IEHMAX
 3650           FORMAT('bin-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
        COMMENT = 'Binned count-rate image for XMM'
C
        UNITS = 'c/s/am2'
        BZERO = 0.0
        BSCALE = 1.0
        IFLAG = 1
        IF(IBIN .EQ. 1) THEN
            ISIZE = 2000
            CALL PUTFIT(INFILE,OUTPUT,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 2) THEN
            CDELT(1) = 2.0*CDELT(1)
            CDELT(2) = 2.0*CDELT(2)
            CRPIX(1) = 500.5
            CRPIX(2) = 500.5
            ISIZE = 1000
            CALL PUTFIT(INFILE,OUT200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 4) THEN
            CDELT(1) = 4.0*CDELT(1)
            CDELT(2) = 4.0*CDELT(2)
            CRPIX(1) = 250.5
            CRPIX(2) = 250.5
            ISIZE = 500
            CALL PUTFIT(INFILE,OUT200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 8) THEN
            CDELT(1) = 8.0*CDELT(1)
            CDELT(2) = 8.0*CDELT(2)
            CRPIX(1) = 125.5
            CRPIX(2) = 125.5
            ISIZE = 250
            CALL PUTFIT(INFILE,OUT800,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 16) THEN
            CDELT(1) = 16.0*CDELT(1)
            CDELT(2) = 16.0*CDELT(2)
            CRPIX(1) = 63.0
            CRPIX(2) = 63.0
            ISIZE = 125
            CALL PUTFIT(INFILE,OUT160,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 32) THEN
            CDELT(1) = 32.0*CDELT(1)
            CDELT(2) = 32.0*CDELT(2)
            CRPIX(1) = 31.25
            CRPIX(2) = 31.25
            ISIZE = 63
            CALL PUTFIT(INFILE,OUT320,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ENDIF
        IF(STATUS .NE. 0) THEN
            PRINT *, 'BIN_IMAGE_MERGE Position 6'
            PRINT *, 'MASK image file STATUS: ', STATUS
            PRINT *, INFILE
            STOP
        ENDIF
C
C  Write out the new FITS binned image file
C
        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3605) IELMIN, IEHMAX
 3605       FORMAT('sig-',I4,'-',I4,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3615) IELMIN, IEHMAX
 3615           FORMAT('sig-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3625) IELMIN, IEHMAX
 3625           FORMAT('sig-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3635) IELMIN, IEHMAX
 3635           FORMAT('sig-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3645) IELMIN, IEHMAX
 3645           FORMAT('sig-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3655) IELMIN, IEHMAX
 3655           FORMAT('sig-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
        COMMENT = 'Binned count-rate map for XMM'
C
        IFLAG = 1
        IF(IBIN .EQ. 1) THEN
            CALL PUTFIT(INFILE,OUTSIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 2) THEN
            CALL PUTFIT(INFILE,OUT2SIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 4) THEN
            ISIZE = 500
            CALL PUTFIT(INFILE,OUT4SIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 8) THEN
            CALL PUTFIT(INFILE,OUT8SIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 16) THEN
            CALL PUTFIT(INFILE,OUT16SIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 32) THEN
            CALL PUTFIT(INFILE,OUT32SIG,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ENDIF
        IF(STATUS .NE. 0) THEN
            PRINT *, 'BIN_IMAGE_MERGE Position 7'
            PRINT *, 'MASK image file STATUS: ', STATUS
            PRINT *, INFILE
            STOP
        ENDIF
C
        IF (STATUS .LE. 0) THEN
            PRINT *,'*** Program completed successfully ***'
        ELSE
            PRINT *, STATUS
            PRINT *,'*** ERROR - program did not run successfully ***'
        ENDIF
C
        RETURN
        END SUBROUTINE bin_image_merge_sub
