       subroutine bin_image_sub(TEMSC,IDET,PREFIX,PREF,IBIN,IEL,IEH,
     +         IPART,IPROT,ISWCX,MASKFILE)
c
c  SAS Version 12.0
c  Version 1.1  19 January 2012
c
c  bin_image bins images in the XMM-ESAS suite of software.  It operates
c  at the individual image level.
c
c  Input
c      temsc: Threshold for masking
c      idet: Detector selection
c          0: Use the output of comb
c          1: Use a specific exposure
c      prefix: Exposure ID if the detector selection is 1
c      pref: Same as prefix
c      ibin: Binning control
c          1: No binning
c          2: Bin by 2
c          3: Bin by 3
c          4: Bin by 4
c          6: Bin by 6
c          9: Bin by 9
c      iel: Lower energy limit for the energy band in eV
c      ieh: Upper energy limit for the energy band in eV
c      ipart: Particle background control'
c          1: include
c          0: do not include
c      iprot: Soft proton control
c          1: include
c          0: do not include
c      iswcx: Solar wind charge exchange control
c          1: include
c          0: do not include
c      imask: Masking control
c          1: include
c          0: do not include
c      maskfile: Mask file name (if absent defaults to using the 
c          exposure map for masking)'
c 
c   Examples
c      bin_image thresholdmasking=0.02 detector=0 binning=2 elow=350 
c              ehigh=1250 withpartcontrol=1 withsoftcontrol=1
c      bin_image thresholdmasking=0.02 detector=1 prefix=1S001 binning=9 
c              elow=350 ehigh=1250 withpartcontrol=1 withsoftcontrol=1
c      bin_image thresholdmasking=0.02 detector=0 binning=4 elow=350 
c              ehigh=1250 withpartcontrol=1 withsoftcontrol=0 
c              maskfile=mask.fits
c
c  OUPUT:
c      binned image
c
        IMPLICIT NONE
C
        INTEGER*2 INI2(900,900)
        INTEGER BLOCK, CLOBBER, DETECT, INI4(900,900), ISIZE, 
     +      I, I2, IBIN, IDET, IEL, IEH, IFLAG, II, IPART, IPROT,
     +      ISWCX, J, J1, J2, JJ, JJ1, JJ2, LUN, STATUS
C
        REAL B, BACK(900,900),
     +      BZERO, BSCALE, C, COUNTS(900,900), COUNTSO(900,900),
     +      E, EX, EXPOSE(900,900), OOTSCA, RNP, SCA,
     +      PROT(900,900), R, REX, RNB, RNC, RNS,
     +      MASK(900,900), S, SWCX(900,900), TLIM, TEMSC, XCEN, YCEN,
     +      OUTPUT(900,900), SIGMA(900,900),
     +      OUT200(450,450), SIG200(450,450),
     +      OUT300(300,300), SIG300(300,300),
     +      OUT400(225,225), SIG400(225,225),
     +      OUT600(150,150), SIG600(150,150),
     +      OUT900(100,100), SIG900(100,100)
C
        DOUBLE PRECISION CDELT(2), CRPIX(2), CRVAL(2)
C
        CHARACTER*1  PRE(4), PREF(5)
        CHARACTER*4  PREFI
        CHARACTER*5  PREFIX
        CHARACTER*8  CTYPE(2), UNITS
        CHARACTER*25 MODE
        CHARACTER*60 COMMENT
        CHARACTER*80 COM
        CHARACTER*132 INFILE, INFILEO, MASKFILE, RATEFILE, SIGFILE
C
        EQUIVALENCE (OUTPUT,OUT200,OUT300,OUT400,OUT600,OUT900)
        EQUIVALENCE (SIGMA,SIG200,SIG300,SIG400,SIG600,SIG900)
        EQUIVALENCE (EXPOSE,INI4)
        EQUIVALENCE (PREFI,PRE)
C
        DATA CLOBBER /1/
        DATA LUN /90/
        STATUS = 0
C
        PRINT *, 'Fractional limit for template: ', TEMSC
C
C  Process the detector control
C
        PRINT *, 'Detector control: ', IDET
        IF(IDET .GT. 0) THEN
            PRINT *, 'Prefix: ', PREFIX
            IF((PREF(1) .EQ. '1') .OR. (PREF(1) .EQ. '2')) THEN
                DETECT = 1
            ELSE
                DETECT = 2
                DO I=1,4
                    PRE(I) = PREF(I)
                ENDDO
            ENDIF
        ELSE
            DETECT = 0
        ENDIF
        PRINT *, 'Detector selection: ', DETECT
        PRINT *, 'Binning factor: ', IBIN
        PRINT *, 'Band energies in eV: ', IEL, IEH
        PRINT *, 'Particle Background Control: ', IPART
        PRINT *, 'Soft Proton background Control: ', IPROT
        PRINT *, 'Solar Wind Charge Exchange background ',
     +          'Control: ', ISWCX
C
C  Enter the count array file name
C
 1000   FORMAT(A110)
C
        IF(IDET .GT. 0) THEN
            IF(DETECT .EQ. 1) THEN
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3001) PREFIX, IEL, IEH
 3001                   FORMAT('mos',A5,'-obj-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3002) PREFIX, IEL, IEH
 3002                   FORMAT('mos',A5,'-obj-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3003) PREFIX, IEL, IEH
 3003               FORMAT('mos',A5,'-obj-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3004) PREFIX, IEL, IEH
 3004               FORMAT('mos',A5,'-obj-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3005) PREFIX, IEL, IEH
 3005               FORMAT('mos',A5,'-obj-im-',I3,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3006) PREFI, IEL, IEH
 3006                   FORMAT('pn',A4,'-obj-im-',I4,'-',I5,'.fits')
                        WRITE(INFILEO,2906) PREFI, IEL, IEH
 2906                   FORMAT('pn',A4,'-obj-im-',I4,'-',I5,'-oot.fits')
                    ELSE
                        WRITE(INFILE,3007) PREFI, IEL, IEH
 3007                   FORMAT('pn',A4,'-obj-im-',I4,'-',I4,'.fits')
                        WRITE(INFILEO,2907) PREFI, IEL, IEH
 2907                   FORMAT('pn',A4,'-obj-im-',I4,'-',I4,'-oot.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3008) PREFI, IEL, IEH
 3008               FORMAT('pn',A4,'-obj-im-',I3,'-',I3,'.fits')
                    WRITE(INFILEO,2908) PREFI, IEL, IEH
 2908               FORMAT('pn',A4,'-obj-im-',I3,'-',I3,'-oot.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3009) PREFI, IEL, IEH
 3009               FORMAT('pn',A4,'-obj-im-',I3,'-',I5,'.fits')
                    WRITE(INFILEO,2909) PREFI, IEL, IEH
 2909               FORMAT('pn',A4,'-obj-im-',I3,'-',I5,'-oot.fits')
                ELSE
                    WRITE(INFILE,3010) PREFI, IEL, IEH
 3010               FORMAT('pn',A4,'-obj-im-',I3,'-',I4,'.fits')
                    WRITE(INFILEO,2910) PREFI, IEL, IEH
 2910               FORMAT('pn',A4,'-obj-im-',I3,'-',I4,'-oot.fits')
                ENDIF
            ENDIF
        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3011) IEL, IEH
 3011               FORMAT('comb-obj-im-',I4,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3012) IEL, IEH
 3012               FORMAT('comb-obj-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3013) IEL, IEH
 3013           FORMAT('comb-obj-im-',I3,'-',I3,'.fits')
            ELSEIF(IEH .GE. 10000) THEN
                WRITE(INFILE,3014) IEL, IEH
 3014           FORMAT('comb-obj-im-',I3,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3015) IEL, IEH
 3015           FORMAT('comb-obj-im-',I3,'-',I4,'.fits')
            ENDIF
        ENDIF
C
        CALL RETFIT (INFILE,COUNTS,INI2,INI4,1,900,
     +          900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
        PRINT *, 'Event image file STATUS: ', STATUS
        PRINT *, INFILE
        SCA = -1.0/(CDELT(1)*CDELT(2))
        IF(DETECT .EQ. 2) THEN
            CALL RETFIT (INFILEO,COUNTSO,INI2,INI4,1,900,
     +              900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            PRINT *, 'Event image file STATUS: ', STATUS
            PRINT *, INFILEO
c
c  This is a bit painful, but we need to know the SUBMODE
c  keyword for the proper scaling of the OOT events
c
c  Get an unused lun
c
            call ftgiou(lun,status)
            if(status .ne. 0) then
                print *, 'STATUS FTGIOU =', status
                print *, 'ADAPT-900 POSITION 21'
            endif
c
c  Open the FITS file
c
            call ftopen (lun,infileo,0,block,status)
            if(status .ne. 0) then
                print *, 'STATUS FTOPEN =', status
                print *, 'ADAPT-900 POSITION 22'
                print *, infileo
            endif
c
c  Read in the mode information if a PN observation
c
            call ftgkys (lun,'SUBMODE',mode,com,status)
            if(status .ne. 0) then
                print *, 'Missing SUBMODE keyword'
                status = 0
                ootsca = 0.0
            else
                if(mode .eq. 'PrimeFullWindow') ootsca = 0.063
                if(mode .eq. 'PrimeFullWindowExtended') ootsca = 0.023
                if(mode .eq. 'PrimeFullWindowExtended5') ootsca = 0.013
            endif
            print *, 'OOT scale factor: ', ootsca
c
c  Close the FITS file
c
            call ftclos(lun,status)
            if(status .ne. 0) then
                print *, 'STATUS FTCLOS =', status
                print *, 'POSITION 7'
                print *, infile
            endif
            DO I=1,900
                DO II=1,900
                    COUNTS(I,II) = COUNTS(I,II) - OOTSCA*COUNTSO(I,II)
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the exposure array file name
C
        IF(IDET .GT. 0) THEN
            IF(DETECT .EQ. 1) THEN
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3101) PREFIX, IEL, IEH
 3101                   FORMAT('mos',A5,'-exp-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3102) PREFIX, IEL, IEH
 3102                   FORMAT('mos',A5,'-exp-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3103) PREFIX, IEL, IEH
 3103               FORMAT('mos',A5,'-exp-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3104) PREFIX, IEL, IEH
 3104               FORMAT('mos',A5,'-exp-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3105) PREFIX, IEL, IEH
 3105               FORMAT('mos',A5,'-exp-im-',I3,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3106) PREFI, IEL, IEH
 3106                   FORMAT('pn',A4,'-exp-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3107) PREFI, IEL, IEH
 3107                   FORMAT('pn',A4,'-exp-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3108) PREFI, IEL, IEH
 3108               FORMAT('pn',A4,'-exp-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3109) PREFI, IEL, IEH
 3109               FORMAT('pn',A4,'-exp-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3110) PREFI, IEL, IEH
 3110               FORMAT('pn',A4,'-exp-im-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF

        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3111) IEL, IEH
 3111               FORMAT('comb-exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3112) IEL, IEH
 3112               FORMAT('comb-exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3113) IEL, IEH
 3113           FORMAT('comb-exp-im-',I3,'-',I3,'.fits')
            ELSEIF(IEH .GE. 10000) THEN
                WRITE(INFILE,3114) IEL, IEH
 3114           FORMAT('comb-exp-im-',I3,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3115) IEL, IEH
 3115           FORMAT('comb-exp-im-',I3,'-',I4,'.fits')
            ENDIF
        ENDIF
C
        CALL RETFIT (INFILE,EXPOSE,INI2,INI4,1,900,
     +          900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
        PRINT *, 'Exposure map file STATUS: ', STATUS
        PRINT *, INFILE
C
C  Enter the background count array file name
C
        IF(IPART .EQ. 1) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3201) PREFIX, IEL, IEH
 3201                       FORMAT('mos',A5,'-back-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3202) PREFIX, IEL, IEH
 3202                       FORMAT('mos',A5,'-back-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3203) PREFIX, IEL, IEH
 3203                   FORMAT('mos',A5,'-back-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3204) PREFIX, IEL, IEH
 3204                   FORMAT('mos',A5,'-back-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3205) PREFIX, IEL, IEH
 3205                   FORMAT('mos',A5,'-back-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3206) PREFI, IEL, IEH
 3206                       FORMAT('pn',A4,'-back-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3207) PREFI, IEL, IEH
 3207                       FORMAT('pn',A4,'-back-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3208) PREFI, IEL, IEH
 3208                   FORMAT('pn',A4,'-back-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3209) PREFI, IEL, IEH
 3209                   FORMAT('pn',A4,'-back-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3210) PREFI, IEL, IEH
 3210                   FORMAT('pn',A4,'-back-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3211) IEL, IEH
 3211                   FORMAT('comb-back-im-sky-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3212) IEL, IEH
 3212                   FORMAT('comb-back-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3213) IEL, IEH
 3213               FORMAT('comb-back-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3214) IEL, IEH
 3214               FORMAT('comb-back-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3215) IEL, IEH
 3215               FORMAT('comb-back-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,BACK,INI2,INI4,1,900,
     +                  900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            PRINT *, 'Background model image file STATUS: ', STATUS
            PRINT *, INFILE
        ELSE
            DO I=1,900
                DO II=1,900
                    BACK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the proton count array file name
C
        IF(IPROT .EQ. 1) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3301) PREFIX, IEL, IEH
 3301                       FORMAT('mos',A5,'-prot-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3302) PREFIX, IEL, IEH
 3302                       FORMAT('mos',A5,'-prot-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3303) PREFIX, IEL, IEH
 3303                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3304) PREFIX, IEL, IEH
 3304                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3305) PREFIX, IEL, IEH
 3305                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3306) PREFI, IEL, IEH
 3306                       FORMAT('pn',A4,'-prot-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3307) PREFI, IEL, IEH
 3307                       FORMAT('pn',A4,'-prot-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3308) PREFI, IEL, IEH
 3308                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3309) PREFI, IEL, IEH
 3309                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3310) PREFI, IEL, IEH
 3310                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3311) IEL, IEH
 3311                   FORMAT('comb-prot-im-sky-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3312) IEL, IEH
 3312                   FORMAT('comb-prot-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3313) IEL, IEH
 3313               FORMAT('comb-prot-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3314) IEL, IEH
 3314               FORMAT('comb-prot-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3315) IEL, IEH
 3315               FORMAT('comb-prot-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,PROT,INI2,INI4,1,900,
     +                  900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            PRINT *, 'Soft proton model image file STATUS: ', STATUS
            PRINT *, INFILE
        ELSE
            DO I=1,900
                DO II=1,900
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Enter the solar wind charge exchange count array file name
C
        IF(ISWCX .EQ. 1) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3401) PREFIX, IEL, IEH
 3401                       FORMAT('mos',A5,'-swcx-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3402) PREFIX, IEL, IEH
 3402                       FORMAT('mos',A5,'-swcx-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3403) PREFIX, IEL, IEH
 3403                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3404) PREFIX, IEL, IEH
 3404                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3405) PREFIX, IEL, IEH
 3405                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3406) PREFI, IEL, IEH
 3406                       FORMAT('pn',A4,'-swcx-im-sky-',I4,'-',
     +                              I5,'.fits')
                        ELSE
                            WRITE(INFILE,3407) PREFI, IEL, IEH
 3407                       FORMAT('pn',A4,'-swcx-im-sky-',I4,'-',
     +                              I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3408) PREFI, IEL, IEH
 3408                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-',
     +                          I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3409) PREFI, IEL, IEH
 3409                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-',
     +                          I5,'.fits')
                    ELSE
                        WRITE(INFILE,3410) PREFI, IEL, IEH
 3410                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-',
     +                          I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3411) IEL, IEH
 3411                   FORMAT('comb-swcx-im-sky-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3412) IEL, IEH
 3412                   FORMAT('comb-swcx-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3413) IEL, IEH
 3413               FORMAT('comb-swcx-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3414) IEL, IEH
 3414               FORMAT('comb-swcx-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3415) IEL, IEH
 3415               FORMAT('comb-swcx-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,SWCX,INI2,INI4,1,900,
     +                  900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            PRINT *, 'Solar wind charge exchange model image ',
     +              'file STATUS: ', STATUS
            PRINT *, INFILE
        ELSE
            DO I=1,900
                DO II=1,900
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
C
C  Find out how many counts we have to play with and the exposure center
C
        RNC = 0.0
        RNB = 0.0
        RNP = 0.0
        RNS = 0.0
        REX = 0.0
        EX = 0.0
        DO I=1,900
            DO II=1,900
                RNC = RNC + COUNTS(I,II)
                RNB = RNB + BACK(I,II)
                RNP = RNP + PROT(I,II)
                RNS = RNS + SWCX(I,II)
                REX = REX + EXPOSE(I,II)
                BACK(I,II) = BACK(I,II) + PROT(I,II) + SWCX(I,II)
                IF(EXPOSE(I,II) .GT. EX) THEN
                    EX = EXPOSE(I,II)
                    XCEN = I
                    YCEN = II
                ENDIF
            ENDDO
        ENDDO
        PRINT *, 'Total Counts: ', RNC
        PRINT *, 'Back Exposure: ', REX
        PRINT *, 'Back Counts: ', RNB
        PRINT *, 'Proton Counts: ', RNP
        PRINT *, 'SWCX Counts: ', RNS
C
C  Read in the template
C
        IF(MASKFILE .EQ. '      ') THEN
C
C  Use the exposure maps as the template
C
            DO I=1,900
                DO II=1,900
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ELSE
C
C  Return the mask
C
            CALL RETFIT (MASKFILE,MASK,INI2,INI4,1,900,
     +                  900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            IF(STATUS .NE. 0) THEN
                PRINT *, 'Bad return from RETFIT'
                PRINT *, 'BIN_IMAGE Position 10'
                PRINT *, 'STATUS = ', STATUS
            ENDIF
        ENDIF
C
C  Set the template limit
C
        J = 0
        TLIM = 0.
        DO I=1,900
            DO II=1,900
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J
C
C  Set the binning limits
C
        I2 = 900/IBIN
C
C  Bin the data
C
        PRINT *, 'START THE BINNING'
        DO I=1,I2
            J2 = I*IBIN
            J1 = J2 - IBIN + 1
            DO II=1,I2
                JJ2 = II*IBIN
                JJ1 = JJ2 - IBIN + 1
                B = 0.
                C = 0.
                E = 0.
                DO J=J1,J2
                    DO JJ=JJ1,JJ2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO
                IF(E .GT. 0.0) THEN
                    R = SCA*(C - B)/E
                    IF(C .GT. 0.0) THEN
                        S = SCA*SQRT(C)/E
                    ELSE
                        S = 0.0
                    ENDIF
                ELSE
                    R = 0.0
                    S = 0.0
                ENDIF
                IF(IBIN .EQ. 1) THEN
                    OUTPUT(I,II) = R
                    SIGMA(I,II) = S
                ELSEIF(IBIN .EQ. 2) THEN
                    OUT200(I,II) = R
                    SIG200(I,II) = S
                ELSEIF(IBIN .EQ. 3) THEN
                    OUT300(I,II) = R
                    SIG300(I,II) = S
                ELSEIF(IBIN .EQ. 4) THEN
                    OUT400(I,II) = R
                    SIG400(I,II) = S
                ELSEIF(IBIN .EQ. 6) THEN
                    OUT600(I,II) = R
                    SIG600(I,II) = S
                ELSEIF(IBIN .EQ. 9) THEN
                    OUT900(I,II) = R
                    SIG900(I,II) = S
                ENDIF
            ENDDO
        ENDDO
C
C  Open the new FITS file
C
        IF(IEL .GE. 1000) THEN
            IF(IEH .GE. 10000) THEN
                WRITE(RATEFILE,3600) IEL, IEH
 3600           FORMAT('rate-',I4,'-',I5,'.fits')
                WRITE(SIGFILE,3601) IEL, IEH
 3601           FORMAT('sigma-',I4,'-',I5,'.fits')
            ELSE
                WRITE(RATEFILE,3605) IEL, IEH
 3605           FORMAT('rate-',I4,'-',I4,'.fits')
                WRITE(SIGFILE,3606) IEL, IEH
 3606           FORMAT('sigma-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEH .GE. 10000) THEN
                WRITE(RATEFILE,3610) IEL, IEH
 3610           FORMAT('rate-',I3,'-',I5,'.fits')
                WRITE(SIGFILE,3611) IEL, IEH
 3611           FORMAT('sigma-',I3,'-',I5,'.fits')
            ELSEIF(IEH .GE. 1000) THEN
                WRITE(RATEFILE,3615) IEL, IEH
 3615           FORMAT('rate-',I3,'-',I4,'.fits')
                WRITE(SIGFILE,3616) IEL, IEH
 3616           FORMAT('sigma-',I3,'-',I4,'.fits')
           ELSE
                WRITE(RATEFILE,3620) IEL, IEH
 3620           FORMAT('rate-',I3,'-',I3,'.fits')
                WRITE(SIGFILE,3621) IEL, IEH
 3621           FORMAT('sigma-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
C
C  Check to see if there is an existing file
C
        CALL FTOPEN (LUN,RATEFILE,0,BLOCK,STATUS)
        IF(STATUS .EQ. 0) THEN
            PRINT *, 'Rate file exists with output file name'
            IF(CLOBBER .EQ. 1) THEN
                PRINT *, 'Clobber the file and continue'
                PRINT *, RATEFILE
                CALL FTDELT (LUN,STATUS)
            ELSE
                PRINT *, 'Will not clobber file'
                PRINT *, RATEFILE
                STOP
            ENDIF
        ENDIF
        STATUS = 0
C
        COMMENT = 'Binned count-rate image'
C
        IFLAG = 1
        ISIZE = 900
        IF(IBIN .EQ. 2) THEN
            CDELT(1) = 2.0*CDELT(1)
            CDELT(2) = 2.0*CDELT(2)
            CRPIX(1) = 225.5
            CRPIX(2) = 225.5
            ISIZE = 450
        ELSEIF(IBIN .EQ. 3) THEN
            CDELT(1) = 3.0*CDELT(1)
            CDELT(2) = 3.0*CDELT(2)
            CRPIX(1) = 150.5
            CRPIX(2) = 150.5
            ISIZE = 300
        ELSEIF(IBIN .EQ. 4) THEN
            CDELT(1) = 4.0*CDELT(1)
            CDELT(2) = 4.0*CDELT(2)
            CRPIX(1) = 113.0
            CRPIX(2) = 113.0
            ISIZE = 225
        ELSEIF(IBIN .EQ. 6) THEN
            CDELT(1) = 6.0*CDELT(1)
            CDELT(2) = 6.0*CDELT(2)
            CRPIX(1) = 75.5
            CRPIX(2) = 75.5
            ISIZE = 150
        ELSEIF(IBIN .EQ. 9) THEN
            CDELT(1) = 9.0*CDELT(1)
            CDELT(2) = 9.0*CDELT(2)
            CRPIX(1) = 50.5
            CRPIX(2) = 50.5
            ISIZE = 100
        ENDIF
        UNITS = 'c/s/deg2'
        BZERO = 0.0
        BSCALE = 1.0
C
        IF(IBIN .EQ. 1) THEN
            CALL PUTFIT(RATEFILE,OUTPUT,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 2) THEN
            CALL PUTFIT(RATEFILE,OUT200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 3) THEN
            CALL PUTFIT(RATEFILE,OUT300,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 4) THEN
            CALL PUTFIT(RATEFILE,OUT400,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 6) THEN
            CALL PUTFIT(RATEFILE,OUT600,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 9) THEN
            CALL PUTFIT(RATEFILE,OUT900,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ENDIF
C
C  Check to see if there is an existing file
C
        COMMENT = 'Binned count-rate uncertainty image'
C
        CALL FTOPEN (LUN,SIGFILE,0,BLOCK,STATUS)
        IF(STATUS .EQ. 0) THEN
            PRINT *, 'Rate file exists with output file name'
            IF(CLOBBER .EQ. 1) THEN
                PRINT *, 'Clobber the file and continue'
                PRINT *, SIGFILE
                CALL FTDELT (LUN,STATUS)
            ELSE
                PRINT *, 'Will not clobber file'
                PRINT *, SIGFILE
                STOP
            ENDIF
        ENDIF
        STATUS = 0

        IF(IBIN .EQ. 1) THEN
            CALL PUTFIT(SIGFILE,SIGMA,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 2) THEN
            CALL PUTFIT(SIGFILE,SIG200,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 3) THEN
            CALL PUTFIT(SIGFILE,SIG300,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 4) THEN
            CALL PUTFIT(SIGFILE,SIG400,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 6) THEN
            CALL PUTFIT(SIGFILE,SIG600,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ELSEIF(IBIN .EQ. 9) THEN
            CALL PUTFIT(SIGFILE,SIG900,INI2,INI4,IFLAG,UNITS,
     +          COMMENT,ISIZE,ISIZE,BZERO,BSCALE,CTYPE,CRVAL,
     +          CDELT,CRPIX,CLOBBER,STATUS)
        ENDIF
C
        IF (STATUS .LE. 0) THEN
            PRINT *,'*** Program completed successfully ***'
        ELSE
            PRINT *, STATUS
            PRINT *,'*** ERROR - program did not run successfully ***'
        ENDIF
C
        RETURN
        end subroutine bin_image_sub
