!+
! Module: comb_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 7 May 2009
!
! Comments:
!
! $Id: comb_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
module comb_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public comb

contains

  !---------------------------------------------------------------------
  subroutine comb
  !---------------------------------------------------------------------
    integer, dimension(:), allocatable :: iel, ieh
    character (len=5), dimension(:), allocatable :: prefix
    character (len=1024) :: xmmdir
    integer :: i, nbands, ndata, imask
    real :: alpha
    logical :: ipart, iprot, iswcx

    xmmdir = stringParameter("caldb")
    ipart = booleanParameter("withpartcontrol")
    iprot = booleanParameter("withsoftcontrol")
    iswcx = booleanParameter("withswcxcontrol")
    nbands = intParameter("nbands")
    alpha = realParameter("alpha")

    allocate(iel(nbands),ieh(nbands))
    do i=1, parameterCount("elowlist")
      iel(i) = intParameter("elowlist",i-1)
      ieh(i) = intParameter("ehighlist",i-1)
    end do

    imask = intParameter("mask")
    ndata = intParameter("ndata")
    allocate(prefix(ndata))
    do i=1, parameterCount("prefixlist")
      prefix(i) = stringParameter("prefixlist",i-1)
    end do

    call comb_sub(xmmdir,ipart,iprot,iswcx,nbands,iel,ieh,imask,ndata,prefix,alpha)
    deallocate(iel,ieh,prefix)

  end subroutine comb

end module comb_mod
