        subroutine fill_list_sub(srclistin,srclistout)
C
C  Version 1.0  08 March 2011
C
C  Fills in entries in the emllist.fits source list
C
C
C  INPUT:
C    Input file with a list of directories
C
C    Example: fill_list srclistin=emllist.fits srclistout=emllistout.fits
C
C  OUPUT:
C    Merged ascii source list
C
        IMPLICIT NONE
C
        real flux, fluxerr, ft, nft, sft
        INTEGER
     +      blockl, err, hdutype, i, ib, iband, ibt, icol, idec, 
     +      idist, iflux, ifluxerr, iml, ira, iradecerr, irate, 
     +      irateerr, is, iscts, isel, isour, ist, lunin, lunout, 
     +      nchan,  ncols, status, tfields, varidat
        character*8  tform(300), ttype(300), tunit(300)
        character*10 colnam(300)
        character*20 extname
        character*132 srclistin, srclistout

        LOGICAL ANYF
c
        data err /0/
        data status /0/
c
c  Get an unused lun
c
        call ftgiou(lunin,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MERGE_SOURCE_LIST POSITION 01'
        endif
c
c  Get a second unused lun
c
        call ftgiou(lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MERGE_SOURCE_LIST POSITION 01a'
        endif
c
c  Open the input source list
c
        call ftopen(lunin,srclistin,0,blockl,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02'
            print *, srclistin
        endif
c
c  Initialize the output source list
c
        call ftinit(lunout,srclistout,blockl,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistout
        endif
c
c  Copy the primary header from the input to the output
c
        call ftcopy(lunin,lunout,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistin, srclistout
        endif
c
c  Move to the source list extension of the input file
c
        call ftmrhd (lunin,1,hdutype,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTMRHD =', status
            print *, 'MERGE_SOURCE_LIST POSITION 03'
            print *, 'Could not find the source list extension'
            print *, srclistin
        endif
c
c  Create a new empty extension in the output file and move to it
c
        call ftcrhd(lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCRHD =', status
            print *, 'MERGE_SOURCE_LIST POSITION 03'
            print *, 'Could not find the source list extension'
            print *, srclistout
        endif
c
c  Copy the source list from the input to the output file
c
        call ftcopy(lunin,lunout,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCOPY =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistin, srclistout
        endif
c
c  Read the FITS file header information
c
        call ftghbn (lunout,300,nchan,tfields,ttype,tform,
     +          tunit,extname,varidat,status)
         if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGHBN =', status
            print *, 'MERGE_SOURCE_LIST POSITION 04'
            print *, srclistout
        endif
        if(extname .ne. 'SRCLIST') then
            print *, extname
            print *, 'This was supposed to be a source list ext'
            stop
        endif
c
c  Read in the column names
c
        call ftgkns (lunout,'TTYPE',1,300,colnam,ncols,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGKNS =', status
            print *, 'MERGE_SOURCE_LIST POSITION 05'
            print *, srclistout
        endif
c
c  Extract the column numbers for the desired columns
c
        isour = 0
        iband = 0
        iml = 0
        iflux = 0
        ifluxerr = 0
        irate = 0
        irateerr = 0
        ira = 0
        idec = 0
        iradecerr = 0
        idist = 0
        do icol=1,ncols
            if(colnam(icol) .eq. 'ML_ID_SRC') isour = icol
            if(colnam(icol) .eq. 'ID_BAND') iband = icol
            if(colnam(icol) .eq. 'SCTS') iscts = icol
            if(colnam(icol) .eq. 'DET_ML') iml = icol
            if(colnam(icol) .eq. 'FLUX') iflux = icol
            if(colnam(icol) .eq. 'FLUX_ERR') ifluxerr = icol
            if(colnam(icol) .eq. 'RATE') irate = icol
            if(colnam(icol) .eq. 'RATE_ERR') irateerr = icol
            if(colnam(icol) .eq. 'RA') ira = icol
            if(colnam(icol) .eq. 'DEC') idec = icol
            if(colnam(icol) .eq. 'RADEC_ERR') iradecerr = icol
            if(colnam(icol) .eq. 'DIST_NN') idist = icol
        enddo
c
        i = 1
c
c  Loop through all the data
c
        do while (i .lt. nchan)
c
c  Read the band number
c
            call ftgcvj(lunout,iband,i,1,1,0,ib,anyf,status)
            if(ib .eq. 0) then
c
c  Read in the flux
c
                call ftgcve(lunout,iflux,i,1,1,-999,flux,anyf,status)
                if(anyf) then
c
c  Read the source number
c
                    call ftgcvj(lunout,isour,i,1,1,0,is,anyf,status)
                    ft = 0.0
                    sft = 0.0
                    nft = 0.0
                    ist = is
                    isel = i
                    do while ((ist .eq. is) .and. (i .lt. nchan))
c
c  Read the source number and band
c
                        i = i + 1
                        call ftgcvj(lunout,isour,i,1,1,0,ist,
     +                          anyf,status)
                        call ftgcvj(lunout,iband,i,1,1,0,ibt,
     +                          anyf,status)
                        if((ist .eq. is) .and. (ibt .eq. 1) .and. 
     +                          (i .le. nchan)) then
c
c  Read in the flux
c
                            call ftgcve(lunout,iflux,i,1,1,-999,
     +                              flux,anyf,status)
                            if(.not. anyf) then    
c
c  Read in the flux uncertainty
c
                                call ftgcve(lunout,ifluxerr,i,1,1,0,
     +                                  fluxerr,anyf,status)
                                ft = ft + flux
                                sft = sft + fluxerr*fluxerr
                                nft = nft + 1.0
                            endif
                        endif
                    enddo
c
                    if(nft .gt. 0.0) then
                        ft = ft/nft
                        sft = sqrt(sft)/nft
                        call ftpcle(lunout,iflux,isel,1,1,ft,status)
                        call ftpcle(lunout,ifluxerr,isel,1,1,
     +                          sft,status)
                    endif
c
                else
                    i = i + 1
                endif
            else
                i = i + 1
            endif
        enddo
c
        call ftclos (lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCLOS =', status
            print *, 'MERGE_SOURCE_LIST POSITION 08'
        endif
c
        return
        end
