        subroutine swcx_sub(prefixin,xmmdir,xmmd,ccd1,ccd2,ccd3,ccd4,
     +          ccd5,ccd6,ccd7,elow,ehigh,nlines,enlinelist,gnormlist,
     +          obj_rmf,obj_arf,obj_spec)
c
c  SAS Version 12.0
c  Version 1.1  15 March 2012
c
c  Casts a map of the SWCX background using the fitted spectral parameters 
c  for Gaussians at the SWCX energies.  Exposure maps for the energies of
c  the SWCX lines are used for the detector maps.
c
c  Usage:
c     prefixin - exposure ID prefix
c     xmmdir - ESAS CCF directory 
c     xmmd - ESAS CCF directory 
c     ccd1-ccd7 - MOS ccd#1 - ccd#7 or PN quad#1 - quad #4 inclusion control
c     elow - lower energy band limit in eV
c     ehigh - upper energy band limit in eV
c     nlines - number of lines
c     enlinelist - energies of the lines to be included 
c     gnormlist - Gaussian normalizations from Xspec
c     obj_rmf - input RMF
c     obj_arf - input ARF
c     obj_spec - input spectrum
c
c     Examples:
c         swcx prefix=1S001 caldb=/CalDB elow=400 ehigh=1250 enlines='0.56 0.65', ccd1=1 
c               ccd2=1 ccd3=1 ccd1=4 ccd5=1 ccd6=0 ccd1=7 gnorm='0.1 0.05'
c
        implicit none

        integer ccd(7), ccd1, ccd2, ccd3, ccd4, ccd5, ccd6, ccd7
        integer bitpix, bitpixn, block, chanrmf(4096), channel(4096), 
     +      clobber, ehigh, 
     +      elow, err, gcount, gcountn, group, hdutype, i, i1, i2, 
     +      ichan, idet, ien, iext, ii, ii1, ii2, iii, iline, irmf, 
     +      k, kk, lun, luna, lunb, naxes(2), naxesn(2), naxis, 
     +      naxisn, nchan, nchana, nchanrin, nchanrout, nchans, 
     +      nlines, obsid, pcount, pcountn, quality(4096), revl, status
c
        real area, areascal, arf(4096), backscal, counts(4096), 
     +      detm(4096,4096), emax(4096), emaxin(4096), emaxout(4096), 
     +      enerrmf(4096), emin(4096), eminin(4096), eminout(4096), 
     +      enim(10), enlinelist(20), exposure, frac, gnormlist(20), 
     +      inarr(780,780), rn, rnorm, 
     +      stat(4096), swcxmap(780,780), test
c
        double precision exposen
c
        character*1 db_fi(80), detnam, swcx_fi(19), xmmd(80)
        character*4 prefi
        character*5 prefix, prefixin
        character*8 extname
        character*19 swcx_file
        character*25 mode
        character*60 com
        character*80 db_file, obj_arf, objfile, obj_rmf, 
     +      obj_spec, outfile, xmmdir
c
        logical anyf, extend, extendn, simple, simplen
c
        equivalence (db_file,db_fi)
        equivalence (swcx_file,swcx_fi)
        equivalence (prefix,prefi)
c
        data clobber /1/
        data enim /0.56, 0.65, 0.81, 0.91, 1.34, 5*0.0/ 
        data err /0/
        data status /0/
c
        prefix = prefixin
c
c  Make sure that the line energies are in keV
c
        do i=1,nlines
            if(enlinelist(i) .gt. 20.0) then
                 enlinelist(i) = enlinelist(i)/1000.0
            endif
        enddo
c
        do i=1,780
            do ii=1,780
                swcxmap(i,ii) = 0.0
            enddo
        enddo
c
c  Get the spectrum
c
        call read_pha (obj_spec,4096,nchans,channel,counts,stat,
     +          quality,group,exposure,backscal,areascal,revl,obsid,
     +          mode,err)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 01'
            print *, 'Error on return from READ_PHA'
            stop
        endif
        backscal = backscal/1440000.0
c
c  Get the arf information
c
        call read_arf (obj_arf,4096,nchana,arf,emin,emax,err)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 02'
            print *, 'Error on return from READ_ARF'
            stop
        endif
c
c  Get the rmf information
c
        call read_rmf (obj_rmf,4096,detm,chanrmf,eminin,emaxin,
     +              nchanrin,eminout,emaxout,nchanrout)
        if(err .ne. 0) then
            print *, 'SWCX POSITION 03'
            print *, 'Error on return from READ_RMF'
            stop
        endif
c
c  Get the input parameters
c
c  Get the prefix
c
        read(prefix,2998) detnam
 2998   format(a1)
        if((detnam .eq. '1') .or. (detnam .eq. '2')) then
            read(prefix,2999) idet
 2999       format(i1)
        else
            idet = 3
            detnam = 'p'
        endif
        swcx_file = '/swcx-im.fits.gz'
c
        ccd(1) = ccd1
        ccd(2) = ccd2
        ccd(3) = ccd3
        ccd(4) = ccd4
        ccd(5) = ccd5
        ccd(6) = ccd6
        ccd(7) = ccd7
c
        print *, 'xmmdir: ', xmmdir
        print *, 'prefix: ', prefix
        if(idet .lt. 3) then
            print *, 'CCD selection', ccd(1), ccd(2), ccd(3), ccd(4),
     +              ccd(5), ccd(6), ccd(7)
        else
            print *, 'Quad selection', ccd(1), ccd(2), ccd(3), ccd(4)
        endif
c
c  Check to see whether MOS1 CCD#6 is missing
c
        if((idet .eq. 1) .and. (revl .gt. 961)) then
            print *, 'Observation took place after MOS1 CCD6 ',
     +                  'was lost'
            if(ccd(6) .eq. 1) then
                print *, 'Set ccd(6) equal to 0'
                ccd(6) = 0
            endif
            print *, 'Revised CCD selection: ', ccd(1), ccd(2), 
     +              ccd(3), ccd(4), ccd(5), ccd(6), ccd(7)
            if(revl .eq. 961) then
                print *, 'Observation took place during the ',
     +              'observation where MOS1 CCD6 was lost.  '
                print *, 'Exposure U002 has CCD6 data, U003 does not'
            endif
        endif
c
        print *, 'elow, ehigh: ', elow, ehigh
        print *, 'idet: ', idet
        do i=1,nlines
             print *, 'Line and Norm: ', enlinelist(i),
     +               '  ', gnormlist(i)
        enddo
c
c  Create the SWCX map file name
c
        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,18
            db_fi(ii+iii) = swcx_fi(iii+1)
        enddo
        do iii=ii+19,80
            db_fi(iii) = ' '
        enddo
        print *, db_file
c
c  Get an unsed LUN
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 04'
            print *, 'Status FTGIOU =', status
        endif
c
c  Open the FITS file for the SWCX image
c
        call ftopen (lun,db_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 05'
            print *, 'Status FTOPEN =', status
            print *, db_file
        endif
c
c  Loop through the selected SP bands
c
        do iline=1,nlines
c
c  Co-add the data from all lines
c
            test = 10000.0
            do i=1,5
                if(abs(enlinelist(iline) - enim(i)) .lt. test) then
                    test = abs(enlinelist(iline) - enim(i))
                    ien = i
                endif
            enddo
            iext = (idet - 1)*5 + ien + 1
c
c  Move to the correct extension
c
            call ftmahd (lun,iext,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 06'
                print *, 'Status FTGHPR =', status
                print *, db_file
                stop
            endif
c
c  Read the extension header information
c
            call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +          pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 07'
                print *, 'Status FTGHPR =', status
                print *, db_file
                stop
            endif
c
c  Read and print out the extension name
c
            call ftgkys (lun,'EXTNAME',extname,com,status)
            if(status .ne. 0) then
                print *, 'SWCX POSITION 08'
                print *, 'Missing EXTNAME keyword'
                print *, 'Status FTGKYS =', status
                print *, db_file
                stop
            endif
            print *, extname
c
c  Return the exposure image
c
            call ftg2de (lun,0,0,naxes(1),naxes(1),naxes(2),
     +          inarr,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 09'
                print *, 'Status FTG2DE =', status
                print *, db_file
            endif
c
c  Mask chips if necessary
c
            if(ccd(1) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 259
                    i2 = 522
                    ii1 = 259
                    ii2 = 522
                elseif(idet .eq. 2) then
                    i1 = 259
                    i2 = 522
                    ii1 = 259
                    ii2 = 522
                else
                    i1 = 23
                    i2 = 346
                    ii1 = 368
                    ii2 = 684
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(2) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 392
                    i2 = 652
                    ii1 = 1
                    ii2 = 251
                elseif(idet .eq. 2) then
                    i1 = 392
                    i2 = 652
                    ii1 = 1
                    ii2 = 251
                else
                    i1 = 346
                    i2 = 669
                    ii1 = 368
                    ii2 = 684
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(3) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 524
                    i2 = 780
                    ii1 = 253
                    ii2 = 516
                elseif(idet .eq. 2) then
                    i1 = 524
                    i2 = 780
                    ii1 = 253
                    ii2 = 516
                else
                    i1 = 346
                    i2 = 670
                    ii1 = 53
                    ii2 = 368
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(ccd(4) .eq. 0) then
                if(idet .eq. 1) then
                    i1 = 394
                    i2 = 654
                    ii1 = 521
                    ii2 = 780
                elseif(idet .eq. 2) then
                    i1 = 394
                    i2 = 654
                    ii1 = 521
                    ii2 = 780
                else
                    i1 = 23
                    i2 = 347
                    ii1 = 54
                    ii2 = 368
                endif
                do i=i1,i2
                    do ii=ii1,ii2
                        inarr(i,ii) = 0.0
                    enddo
                enddo
            endif
            if(idet .lt. 3) then
                if(ccd(5) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 130
                        i2 = 393
                        ii1 = 521
                        ii2 = 780
                    else
                        i1 = 128
                        i2 = 390
                        ii1 = 523
                        ii2 = 780
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
                if(ccd(6) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 1
                        i2 = 258
                        ii1 = 257
                        ii2 = 520
                    else
                        i1 = 1
                        i2 = 257
                        ii1 = 258
                        ii2 = 521
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
                if(ccd(7) .eq. 0) then
                    if(idet .eq. 1) then
                        i1 = 129
                        i2 = 391
                        ii1 = 1
                        ii2 = 253
                    else
                        i1 = 125
                        i2 = 388
                        ii1 = 1
                        ii2 = 254
                    endif
                    do i=i1,i2
                        do ii=ii1,ii2
                            inarr(i,ii) = 0.0
                        enddo
                    enddo
                endif
            endif
c
c  Find the total exposure of the exposure map
c
            rn = 0.0
            do k=1,780
                do kk=1,780
                    rn = rn + inarr(k,kk)
                enddo
            enddo
c
c  Find the number of counts
c
            do i=1,nchana
                 if((enlinelist(iline) .le. emax(i)) .and.
     +                   (enlinelist(iline) .gt. emin(i))) area = arf(i)
            enddo
            rnorm = gnormlist(iline)*area*exposure*backscal
            print 2110, enlinelist(iline), rnorm
 2110       format(1x,'Total SWCX counts from the line at ',
     +          f6.3,' keV:',f8.1) 
c
c  Now find the number of counts in the band
c
            do i=1,nchanrin
                 if((enlinelist(iline) .le. emaxin(i)) .and.
     +                   (enlinelist(iline) .gt. eminin(i))) irmf = i
            enddo
            frac = 0.0
            do i=1,nchanrout
                if((1000.0*(eminout(i)+emaxout(i))/2.0 .le. ehigh) 
     +                  .and. (1000.0*(eminout(i)+emaxout(i))/2.0 
     +                  .gt. elow)) then
                    frac = frac + detm(irmf,i)
                    write(56,*) i, irmf, 
     +                  eminout(i), emaxout(i), detm(irmf,i)
                endif
            enddo
            print 2120, enlinelist(iline), frac*rnorm
 2120       format(1x,'Number of SWCX counts in the chosen band from ', 
     +          'the line at ',f6.3,' keV:',f8.1)
            rnorm = frac*rnorm/rn
c
c  Add data to the array
c
            do k=1,780
                do kk=1,780
                    swcxmap(k,kk) = swcxmap(k,kk) + rnorm*inarr(k,kk)
                enddo
            enddo
        enddo
        rn = 0.0
        do k=1,780
            do kk=1,780
                rn = rn + swcxmap(k,kk)
            enddo
        enddo
        print 2140, rn
 2140   format(1x,'Total SWCX counts in band:',f8.1)
c
c  Close the file
c
        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 10'
            print *, 'Status FTCLOS =', status
            print *, db_file
        endif
c
c  Open up a template event image
c
c  First get an unused lun
c
        call ftgiou(luna,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 11'
            print *, 'STATUS FTGIOU =', status
        endif
c
c  Create the file name
c
        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3000) prefix, elow, ehigh
 3000               format('mos',a5,'-obj-im-det-',i4,'-',i5,'.fits')
                else
                    write(objfile,3002) prefi, elow, ehigh
 3002               format('pn',a4,'-obj-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3004) prefix, elow, ehigh
 3004               format('mos',a5,'-obj-im-det-',i4,'-',i4,'.fits')
                else
                    write(objfile,3006) prefi, elow, ehigh
 3006               format('pn',a4,'-obj-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(objfile,3010) prefix, elow, ehigh
 3010           format('mos',a5,'-obj-im-det-',i3,'-',i3,'.fits')
            else
                write(objfile,3012) prefi, elow, ehigh
 3012           format('pn',a4,'-obj-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(objfile,3020) prefix, elow, ehigh
 3020               format('mos',a5,'-obj-im-det-',i3,'-',i5,'.fits')
                else
                    write(objfile,3022) prefi, elow, ehigh
 3022               format('pn',a4,'-obj-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(objfile,3024) prefix, elow, ehigh
 3024               format('mos',a5,'-obj-im-det-',i3,'-',i4,'.fits')
                else
                    write(objfile,3026) prefi, elow, ehigh
 3026               format('pn',a4,'-obj-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Template image'
        print *, objfile
c
c  Open the FITS file
c
        call ftopen (luna,objfile,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 12'
            print *, 'STATUS FTOPEN =', status
            print *, objfile
        endif
c
c  Read the FITS file header information
c
        call ftghpr (luna,10,simplen,bitpixn,naxisn,naxesn,
     +      pcountn,gcountn,extendn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 13'
            print *, 'Status FTGHPR =', status
            print *, objfile
        endif
c
c  Get the exposure keyword
c
        call ftgkyd (luna,'EXPOSURE',exposen,com,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 14'
            print *, 'Status FTGKY =', status
            print *, objfile
        endif
c
c  Write out the SWCX count detector map
c
c  First get an unused lun
c
        call ftgiou(lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION15'
            print *, 'STATUS FTGIOU =', status
        endif
c
c  Create the file name
c
        if(elow .ge. 1000) then
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3100) prefix, elow, ehigh
 3100               format('mos',a5,'-swcx-im-det-',i4,'-',i5,'.fits')
                else
                    write(outfile,3102) prefi, elow, ehigh
 3102               format('pn',a4,'-swcx-im-det-',i4,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3104) prefix, elow, ehigh
 3104               format('mos',a5,'-swcx-im-det-',i4,'-',i4,'.fits')
                else
                    write(outfile,3106) prefi, elow, ehigh
 3106               format('pn',a4,'-swcx-im-det-',i4,'-',i4,'.fits')
                endif
            endif
        elseif(ehigh .lt. 1000) then
            if(idet .lt. 3) then
                write(outfile,3110) prefix, elow, ehigh
 3110           format('mos',a5,'-swcx-im-det-',i3,'-',i3,'.fits')
            else
                write(outfile,3112) prefi, elow, ehigh
 3112           format('pn',a4,'-swcx-im-det-',i3,'-',i3,'.fits')
            endif
        else
            if(ehigh .ge. 10000) then
                if(idet .lt. 3) then
                    write(outfile,3120) prefix, elow, ehigh
 3120               format('mos',a5,'-swcx-im-det-',i3,'-',i5,'.fits')
                else
                    write(outfile,3122) prefi, elow, ehigh
 3122               format('pn',a4,'-swcx-im-det-',i3,'-',i5,'.fits')
                endif
            else
                if(idet .lt. 3) then
                    write(outfile,3124) prefix, elow, ehigh
 3124               format('mos',a5,'-swcx-im-det-',i3,'-',i4,'.fits')
                else
                    write(outfile,3126) prefi, elow, ehigh
 3126               format('pn',a4,'-swcx-im-det-',i3,'-',i4,'.fits')
                endif
            endif
        endif
        print *, 'Output file name'
        print *, outfile
c
c  Check to see if there is an existing file
c
        call ftopen (lunb,outfile,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber .eq.  1) then
                print *, 'Clobber the file and continue'
                call ftdelt(lunb,status)
            else
                print *, 'SWCX POSITION 16'
                print *, 'Will not clobber file'
                stop
            endif
        endif
        status = 0
c
c  Create the output FITS file using the original file as a template
c
        call ftinit (lunb,outfile,2880,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 17'
            print *, 'STATUS FTINIT =', status
            print *, outfile
        endif
c
c  Copy the header information from the old to the new file
c
        call ftcphd (luna,lunb,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 18'
            print *, 'STATUS FTCPHD =', status
        endif
c
        call ftpdef (lunb,bitpixn,naxisn,naxesn,pcountn,
     +              gcountn,.FALSE.,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX POSITION 19'
            print *, 'STATUS FTPDEF =', status
        endif
        call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'SWCX Position 20'
            print *, 'STATUS FTMKYJ =', status
        endif
        if(err .eq. 0) then
c
c  Write the output image
c
            group = 1
            call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2),
     +                  swcxmap,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 21'
                print *, 'STATUS FTP2DE =', status
                print *, 'Could not write the background image'
            endif
c
c  Close the output file
c
            call ftclos(lunb,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'SWCX POSITION 22'
                print *, 'STATUS FTCLOS =', status
                print *, 'Could not close the output file'
            endif
        endif
c
        RETURN
        END SUBROUTINE swcx_sub
