c
c  SAS Version 12.0
c  Version 4.3  14 March 2012
c
c
c  Version 4.0  11 March 2007
c
        subroutine ANGSEP(DLONG1,DLAT1,DLONG2,DLAT2,DANG,IERR)
C
CC    Calculates the angular separation between two celestial bodies
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:48
C
CA  author : KOD        date: 06-SEP-1989 16:40
CU  update : KOD        date: 20-MAR-1990 15:45
CU  update : KOD        date: 24-OCT-1991 13:57  (sin --> dsin, ...)
C
CT  status: tested
C
C   general description
CG
CG  Subroutine to calculate the angular separation between two
CG  celestial bodies. The 'latitude' is defined as to have
CG  0 degrees at the equator and +90 degrees at the north pole.
CG  Special care is taken for small angles, according to
CG    R.W.Sinnott, 'Sky and Telescope', August 1984, p.159.
C
C   call_var.          type I/O description
CP
CP  DLONG1              R8  I   'longitude 1' (e.g. R.A.; radians)
CP  DLAT1               R8  I   'latitude  1' (e.g. Dec.; radians)
CP
CP  DLONG2              R8  I   'longitude 2' (e.g. R.A.; radians)
CP  DLAT2               R8  I   'latitude  2' (e.g. Dec.; radians)
CP
CP  DANG                R8    O angular separation       (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
        INCLUDE 'cgenl.cmn'
        INCLUDE 'igenl.inc'
C
        DOUBLE PRECISION DLONG1, DLAT1, DLONG2, DLAT2, DANG, D1, D2, DR
        INTEGER IERR
        CHARACTER*8   RTNAME
        CHARACTER*80  LOG
C
        DATA RTNAME /'ANGSEP'/
C
        IERR = 0
C
        if (max(abs(DLONG1),abs(DLONG2)).gt.D2PI) then
            IERR = max(IERR,1)
            LOG = 'Unusual value for the longitude'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (max(abs(DLAT1), abs(DLAT2)).gt.DPID2) then
            IERR = max(IERR,2)
            LOG = 'Error in input parameters'
            call WRERR(RTNAME,IERR,LOG)
        endif
C
        if (IERR.lt.2) then
C
            D1 = dsin ( 0.5D0 * ( DLAT1 -  DLAT2 ) )
            D2 = dsin ( 0.5D0 * (DLONG1 - DLONG2 ) )
            DR = D1 * D1 + D2 * D2 * dcos(DLAT1) * dcos(DLAT2)
            DANG = 2.D0 * datan2 ( dsqrt(DR), dsqrt(1.D0-DR) )
C
        endif
C
        return
        end
c
c
c
        subroutine aug_get_exp (db_file,c_val,c_dst,c_tim,
     +      emin,emax,p_flg,aug,aug_exp,idet,ichip,mchan,
     +      obsid,prefix)
c
        implicit none
c
        integer block, err, hdutype,
     +      i, ichip, icol, idate, idet, iexpo,
     +      ifilt, ihard, ii, ikeep, iname, iobid,
     +      iobjt, iodd, iplot, irate, irevl, ispec, itd,
     +      itime, iurate, iuhard, lun, lunp, mchan, n,
     +      nchan, ncols, nobs, obsid, ord(7500),
     +      p_flg, revl(7500), status, tfields, varidat
c
        real aug(2400), aug_exp, ave_rate, c_dst(3), cum_exp,
     +      c_tim, c_val(3), d1, d2, ddist(7500), dist(7500),
     +      emax(mchan), emin(mchan), energ(2400), expo(7500),
     +      hard(7500), hardmax, hardmin, hdist(7500), muhard, murate,
     +      rate(7500), ratemax, ratemin, rdist(7500), spec(2400,7500),
     +      td, tdist(7500), temp, temphard, time(7500), tr, 
     +      tspec(2400), uhard(7500), urate(7500)
c
        double precision date(7500)
c
        character*5 prefix
        character*6 filt(7500)
        character*8 colnam(23), tform(23), ttype(23), tunit(23)
        character*11 test
        character*13 obid(7500)
        character*15 objt(7500)
        character*17 name(7500)
        character*20 extname
        character*24 title
        character*80 augfile, db_file
        logical anyf
c
        save iplot, lunp
c
        data iplot /0/
        data status /0/

c        print *, db_file,c_val,c_dst,c_tim,p_flg,
c     +      aug_exp,idet,ichip,mchan,obsid,prefix
c
c  if(n_params(0) LT 7)then begin
c    print,"get_aug,db_file,r_lim,h_lim,d_lim,c_val,c_dst,c_tim,p_flg,aug,aug_exp"
c    print,"Purpose:to create a mean corner spectrum"
c    print,"  Given a rate, hardness, and date,"
c    print,"  finds the closest c_tim seconds of exposure"
c    print,"  in that 3-D space"
c    print,"Inputs:"
c    print,"  db_file:database file of corner spectra"
c    print,"  c_val:central [rate,hardness,date]"
c    print,"  c_dst:distance scaling in those directions"
c    print,"    as factor of mean uncertainty in those directions"
c    print,"    suggest:[0.00323,0.358,1.0]"
c    print,"  c_tim:how many seconds of exposure desired for spectrum"
c    print,"  p_flg:1 if informative plot to be made"
c    print,"Outputs:"
c    print,"  aug:the augmented spectrum"
c    print,"  aug_exp:the amount of exposure time in the augmented spectrum"
c    return
c  endif
c
        err = 0
c
c  If selected, make a plot, first set up the plot
c
        if((p_flg .eq. 1) .and. (iplot .eq. 0)) then
            iplot = 1
c
c  Get an unused lun
c
            status = 0
            call ftgiou(lunp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_EXP POSITION 01'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,2999) prefix
 2999       format('mos',a5,'-aug.qdp')
            open(lunp,form='formatted',status='unknown',
     +              file=augfile)
            write(lunp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
 3000       format('"',i10,' -- MOS',a5,'"')
            write(lunp,*) ' la t ', title
            write(lunp,*) ' read serr 1'
            write(lunp,*) ' read serr 2'
            write(lunp,*) ' read serr 3'
            write(lunp,*) ' read serr 4'
            write(lunp,*) ' read serr 5'
            write(lunp,*) ' read serr 6'
            write(lunp,*) ' read serr 7'
            write(lunp,*) ' read serr 8'
            write(lunp,*) ' read serr 9'
            write(lunp,*) ' read serr 10'
            write(lunp,*) ' read serr 11'
            write(lunp,*) ' read serr 12'
            write(lunp,*) ' read serr 13'
            write(lunp,*) ' read serr 14'
            write(lunp,*) ' read serr 15'
            write(lunp,*) ' read serr 16'
            write(lunp,*) ' read serr 17'
            write(lunp,*) ' read serr 18'
            write(lunp,*) ' read serr 19'
            write(lunp,*) ' font roman'
            write(lunp,*) ' cs 1.5'
            write(lunp,*) ' la f'
            write(lunp,*) ' col 2 on 2'
            write(lunp,*) ' col 1 on 3'
            write(lunp,*) ' col 3 on 4'
            write(lunp,*) ' col 2 on 5'
            write(lunp,*) ' col 1 on 6'
            write(lunp,*) ' col 3 on 7'
            write(lunp,*) ' col 2 on 8'
            write(lunp,*) ' col 1 on 9'
            write(lunp,*) ' col 3 on 10'
            write(lunp,*) ' col 2 on 11'
            write(lunp,*) ' col 1 on 12'
            write(lunp,*) ' col 3 on 13'
            write(lunp,*) ' col 2 on 14'
            write(lunp,*) ' col 1 on 15'
            write(lunp,*) ' col 3 on 16'
            write(lunp,*) ' col 2 on 17'
            write(lunp,*) ' col 1 on 18'
            write(lunp,*) ' col 3 on 19'
            write(lunp,*) ' mark 1 on 2'
            write(lunp,*) ' mark 1 on 3'
            write(lunp,*) ' mark 1 on 4'
            write(lunp,*) ' mark 1 on 5'
            write(lunp,*) ' mark 1 on 6'
            write(lunp,*) ' mark 1 on 7'
            write(lunp,*) ' mark 1 on 8'
            write(lunp,*) ' mark 1 on 9'
            write(lunp,*) ' mark 1 on 10'
            write(lunp,*) ' mark 1 on 11'
            write(lunp,*) ' mark 1 on 12'
            write(lunp,*) ' mark 1 on 13'
            write(lunp,*) ' mark 1 on 14'
            write(lunp,*) ' mark 1 on 15'
            write(lunp,*) ' mark 1 on 16'
            write(lunp,*) ' mark 1 on 17'
            write(lunp,*) ' mark 1 on 18'
            write(lunp,*) ' mark 1 on 19'
            write(lunp,*) ' line off 2'
            write(lunp,*) ' line off 3'
            write(lunp,*) ' line off 4'
            write(lunp,*) ' line off 5'
            write(lunp,*) ' line off 6'
            write(lunp,*) ' line off 7'
            write(lunp,*) ' line off 8'
            write(lunp,*) ' line off 9'
            write(lunp,*) ' line off 10'
            write(lunp,*) ' line off 11'
            write(lunp,*) ' line off 12'
            write(lunp,*) ' line off 13'
            write(lunp,*) ' line off 14'
            write(lunp,*) ' line off 15'
            write(lunp,*) ' line off 16'
            write(lunp,*) ' line off 17'
            write(lunp,*) ' line off 18'
            write(lunp,*) ' line off 19'
            write(lunp,*) ' win 1'
            write(lunp,*) ' yplot 1 2 3 4'
            write(lunp,*) ' loc 0.05 0.48 0.39 1.0'
            write(lunp,*) ' la 11 p 0.06 9.0 just left "CCD #2"'
            write(lunp,*) ' r x1 0.0 0.1'
            write(lunp,*) ' r y1 0.0 10.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' win 2'
            write(lunp,*) ' yplot 1 5 6 7'
            write(lunp,*) ' loc 0.35 0.48 0.69 1.0'
            write(lunp,*) ' la 12 p 0.06 9.0 just left "CCD #3"'
            write(lunp,*) ' r x2 0.0 0.1'
            write(lunp,*) ' r y2 0.0 10.0'
            write(lunp,*) ' win 3'
            write(lunp,*) ' yplot 1 8 9 10'
            write(lunp,*) ' loc 0.65 0.48 0.99 1.0'
            write(lunp,*) ' la 13 p 0.06 9.0 just left "CCD #4"'
            write(lunp,*) ' r x3 0.0 0.1'
            write(lunp,*) ' r y3 0.0 10.0'
            write(lunp,*) ' win 4'
            write(lunp,*) ' yplot 1 11 12 13'
            write(lunp,*) ' loc 0.05 0.05 0.39 0.57'
            write(lunp,*) ' la 14 p 0.06 9.0 just left "CCD #5"'
            write(lunp,*) ' r x4 0.0 0.1'
            write(lunp,*) ' r y4 0.0 10.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 5'
            write(lunp,*) ' yplot 1 14 15 16'
            write(lunp,*) ' loc 0.35 0.05 0.69 0.57'
            write(lunp,*) ' la 15 p 0.06 9.0 just left "CCD #6"'
            write(lunp,*) ' r x5 0.0 0.1'
            write(lunp,*) ' r y5 0.0 10.0'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 6'
            write(lunp,*) ' yplot 1 17 18 19'
            write(lunp,*) ' loc 0.65 0.05 0.99 0.57'
            write(lunp,*) ' la 16 p 0.06 9.0 just left "CCD #7"'
            write(lunp,*) ' r x6 0.0 0.1'
            write(lunp,*) ' r y6 0.0 10.0'
            write(lunp,*) ' la x Rate'
        endif
c
c  Get an unused lun for the caldb file
c
        err = 0
        status = 0
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_EXP POSITION 02'
            print *, 'STATUS FTGIOU =', status
        endif
c
c  Open the FITS file
c
        call ftopen (lun,db_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_EXP POSITION 03'
            print *, 'STATUS FTOPEN =', status
            print *, db_file
        endif
c
c  Move to the correct extension
c
        if(err .eq. 0) then
            call ftmahd (lun,ichip,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_EXP POSITION 04'
                print *, 'STATUS FTMAHD =', status
                print *, db_file
                print *, 'Could not find the spectrum extension'
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,23,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                print *, 'Total number of Obs in QPB file: ', nchan
                write(test,3010) ichip
 3010           format('QUIES_CHIP',i1)
                if(extname .ne. test) then
                    print *, 'AUG_GET_EXP POSITION 05'
                    print *, extname
                    print *, 'This was supposed to be the QUIES_CHIP',
     +                      ichip,' extension'
                    stop
                endif
                if(status .ne. 0) then
                    err = 1
                    print *, 'AUG_GET_EXP POSITION 06'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'Failure on reading the header ',
     +                      'information'
                else
                    if(nchan .gt. 7500) then
                        print *, 'Too much data: ', ncols
                        print *, 'Limit selection to 7500'
                        ncols = 7500
                    endif
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,23,colnam,ncols,status)
c
c  Extract the channel numbers
c
                    iobid = 0
                    iname = 0
                    iobjt = 0
                    idate = 0
                    irevl = 0
                    ifilt = 0
                    itime = 0
                    iexpo = 0
                    irate = 0
                    iurate = 0
                    ihard = 0
                    iuhard = 0
                    ispec = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'OBSID') iobid = icol
                        if(colnam(icol) .eq. 'NAME') iname = icol
                        if(colnam(icol) .eq. 'OBJECT') iobjt = icol
                        if(colnam(icol) .eq. 'DATE') idate = icol
                        if(colnam(icol) .eq. 'REVOLUT') irevl = icol
                        if(colnam(icol) .eq. 'FILTER') ifilt = icol
                        if(colnam(icol) .eq. 'ONTIME') itime = icol
                        if(colnam(icol) .eq. 'EXPO') iexpo = icol
                        if(colnam(icol) .eq. 'RATE') irate = icol
                        if(colnam(icol) .eq. 'RATE_ERR') iurate = icol
                        if(colnam(icol) .eq. 'HARD') ihard = icol
                        if(colnam(icol) .eq. 'HARD_ERR') iuhard = icol
                        if(colnam(icol) .eq. 'SPECTRUM') ispec = icol
                    enddo

c        print *, colnam, iobid, iname, iobjt, idate, irevl, ifilt,
c     +       itime, iexpo, irate, iurate, ihard, iuhard, ispec

                    do i=1,nchan
                       if(iobid .ne. 0)
     +                      call ftgcvs(lun,iobid,i,1,1,' ',
     +                          obid(i),anyf,status)
                        if(iname .ne. 0)
     +                      call ftgcvs(lun,iname,i,1,1,' ',
     +                          name(i),anyf,status)
                        if(iobjt .ne. 0)
     +                      call ftgcvs(lun,iobjt,i,1,1,' ',
     +                          objt(i),anyf,status)
                        if(idate .ne. 0)
     +                      call ftgcvd(lun,idate,i,1,1,0,
     +                          date(i),anyf,status)
                        if(irevl .ne. 0)
     +                      call ftgcvj(lun,irevl,i,1,1,0,
     +                          revl(i),anyf,status)
                        if(ifilt .ne. 0)
     +                      call ftgcvs(lun,ifilt,i,1,1,' ',
     +                          filt(i),anyf,status)
                        if(itime .ne. 0)
     +                      call ftgcve(lun,itime,i,1,1,0,
     +                          time(i),anyf,status)
                        if(iexpo .ne. 0)
     +                      call ftgcve(lun,iexpo,i,1,1,0,
     +                          expo(i),anyf,status)
                        if(irate .ne. 0)
     +                      call ftgcve(lun,irate,i,1,1,0,
     +                          rate(i),anyf,status)
                        if(iurate .ne. 0)
     +                      call ftgcve(lun,iurate,i,1,1,0,
     +                          urate(i),anyf,status)
                        if(ihard .ne. 0)
     +                      call ftgcve(lun,ihard,i,1,1,0,
     +                          hard(i),anyf,status)
                        if(iuhard .ne. 0)
     +                      call ftgcve(lun,iuhard,i,1,1,0,
     +                          uhard(i),anyf,status)
                        if(ispec .ne. 0) then
                            call ftgcve(lun,ispec,i,1,2400,0,
     +                              tspec,anyf,status)
                            do ii=1,2400
                                spec(ii,i) = tspec(ii)
                            enddo
                            if((i .eq. nchan) .and. (status .ne. 0))
     +                              status = 0
                        endif
                    enddo
                    if(status .ne. 0) then
                        print *, 'Missing column in QPB info file'
                        print *, 'STATUS: ', status
                        return
                    endif
                endif
            endif
        endif
        call ftclos (lun,status)
c
c  first determine if the observation CCD is in an odd state
c
        iodd = 0
        if(idet .eq. 1) then
            if (ichip .eq. 4) then
                tr = 100.0*c_val(1)
                if(tr .lt. 5.2) then
                    temphard = 0.96
                elseif(tr .lt. 5.2) then
                    temphard = 0.423*tr - 1.24
                else
                    temphard = 0.423*tr - 1.24
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            elseif (ichip .eq. 5) then
                tr = 100.0*c_val(1)
                if(tr .lt. 7.0) then
                    temphard = 1.54
                elseif (tr .lt. 9.0) then
                    temphard = 0.53*tr - 2.17
                else
                    temphard = 2.60
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            else
                if(c_val(2) .lt. 1.5) iodd = 1
            endif
        else
            if (ichip .eq. 2) then
                if(c_val(2) .lt. 1.1) iodd = 1
            elseif (ichip .eq. 5) then
                tr = 100.0*c_val(1)
                if(tr .lt. 10.0) then
                    temphard = 0.15*tr
                elseif (tr .lt. 16.67) then
                    temphard = 0.15*tr
                else
                    temphard = 2.50
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            else
                if(c_val(2) .lt. 1.5) iodd = 1
            endif
        endif
c
c  Set the normal filtering limits
c
        d1 = -9.0
        d2 = 0.0
        hardmin = 0.2
        hardmax = 15.0
        ratemin = 0.01
        ratemax = 0.1
c
c  Sort through the archived observation data
c
        nobs = 0
        do i=1,nchan
            if((expo(i) .ge. 1000.0) .and. (rate(i) .gt. ratemin) .and.
     +              (rate(i) .lt. ratemax) .and.
     +              (hard(i) .gt. hardmin) .and.
     +              (hard(i) .lt. hardmax) .and.
     +              (filt(i) .ne. 'CalClo')) then
c
c  Write out all the data for plotting if selected
c
                if(p_flg .eq. 1) then
                    if(ichip .eq. 2) then
                        write(lunp,2120) rate(i), urate(i),
     +                      hard(i), uhard(i), d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2120                   format(1x,2f10.6,2f6.2,34f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunp,2130) rate(i), urate(i),
     +                      d1, d2, d1, d2, d1, d2, 
     +                      hard(i), uhard(i), d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2130                   format(1x,2f10.6,6f4.0,2f6.2,28f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunp,2140) rate(i), urate(i),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, 
     +                      hard(i), uhard(i), d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2140                   format(1x,2f10.6,12f4.0,2f6.2,22f4.0)
                    elseif(ichip .eq. 5) then
                        write(lunp,2150) rate(i), urate(i),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, hard(i), uhard(i), d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2
 2150                   format(1x,2f10.6,18f4.0,2f6.2,16f4.0)
                    elseif(ichip .eq. 6) then
                        write(lunp,2160) rate(i), urate(i),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      hard(i), uhard(i), d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2160                   format(1x,2f10.6,24f4.0,2f6.2,10f4.0)
                    elseif(ichip .eq. 7) then
                        write(lunp,2170) rate(i), urate(i),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2,
     +                      hard(i), uhard(i), d1, d2, d1, d2
 2170                   format(1x,2f10.6,30f4.0,2f6.2,4f4.0)
                    endif
                endif
c
c  Check to see whether the archived CCD is in an odd state
c  and select whether the data will be kept or not
c
                ikeep = 1
                if(idet .eq. 1) then
                    if (ichip .eq. 4) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 5.2) then
                            temphard = 0.96
                        elseif(tr .lt. 5.2) then
                            temphard = 0.423*tr - 1.24
                        else
                            temphard = 0.423*tr - 1.24
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i) .ge. temphard) ikeep = 0
                        else
                            if(hard(i) .lt. temphard) ikeep = 0
                        endif
                    elseif (ichip .eq. 5) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 7.0) then
                            temphard = 1.54
                        elseif (tr .lt. 9.0) then
                            temphard = 0.53*tr - 2.17
                        else
                            temphard = 2.60
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i) .ge. temphard) ikeep = 0
                        else
                            if(hard(i) .lt. temphard) ikeep = 0
                        endif
                    else
                        if(hard(i) .lt. 1.5) then
                            ikeep = 0
                        endif
                    endif
                endif
                if(idet .eq. 2) then
                    if(ichip .eq. 2) then
                        if(iodd .eq. 1) then
                            if(hard(i)  .ge. 1.1) ikeep = 0
                        else
                            if(hard(i)  .lt. 1.1) ikeep = 0
                        endif
                    elseif (ichip .eq. 5) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 10.0) then
                            temphard = 0.15*tr
                        elseif (tr .lt. 16.67) then
                            temphard = 0.15*tr
                        else
                            temphard = 2.50
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i)  .ge. temphard) ikeep = 0
                        else
                            if(hard(i)  .lt. temphard) ikeep = 0
                        endif
                    else
                        if (iodd .eq. 1) then
                            if(hard(i)  .ge. 1.5) ikeep = 0
                        else
                            if(hard(i)  .lt. 1.5) ikeep = 0
                        endif
                    endif
                endif
                if(filt(i) .eq. 'CalClo') ikeep = 0
c
c  Keep the data set so transfer the information
c
                if(ikeep .eq. 1) then
                    nobs = nobs + 1
                    obid(nobs) = obid(i)
                    name(nobs) = name(i)
                    objt(nobs) = objt(i)
                    date(nobs) = date(i)
                    revl(nobs) = revl(i)
                    filt(nobs) = filt(i)
                    time(nobs) = time(i)
                    expo(nobs) = expo(i)
                    rate(nobs) = rate(i)
                    urate(nobs) = urate(i)
                    hard(nobs) = hard(i)
                    uhard(nobs) = uhard(i)
                    do ii=1,2400
                        spec(ii,nobs) = spec(ii,i)
                    enddo
                endif
            endif
        enddo
c
        if(nobs .eq. 0) then
            print *, 'CCD in an anomalous state - no',
     +              'appropriate data'
            print *, 'Rerun analysis excluding instrument: MOS', 
     +              idet,'  CCD #', ichip
            stop
        endif
c
c  Set the energies of the bins
c
        do i=1,2400
            energ(i) = 0.005*(i - 1)
        enddo
c
c  Get some temporary values
c
        murate = 0.0
        muhard = 0.0
        do i=1,nobs
            murate = murate + urate(i)
            muhard = muhard + uhard(i)
        enddo
        murate = murate/nobs
        muhard = muhard/nobs
c
        print *,'Rate: ',c_val(1),' Rate error: ',murate
        print *,'Hard: ',c_val(2),' Hard error: ',muhard
c
c  Find the closest c_tim secs
c
        if(c_tim .eq. 0.0) then
            print *, 'Error: no exposure length provided'
            return
        else
            if((c_dst(1) .eq. 0.0) .and. (c_dst(2) .eq. 0.0) .and.
     +              (c_dst(3) .eq. 0.0)) then
                print *, 'Error: no distance scalings'
                return
            else
                do i=1,nobs
                    if(c_dst(1) .gt. 0.0) then
                        rdist(i) = (rate(i)-c_val(1))/c_dst(1)
                    else
                        rdist(i) = rate(i)*0.0
                    endif
                    if(c_dst(2) .gt. 0.0) then
                        hdist(i) = (hard(i)-c_val(2))/c_dst(2)
                    else
                        hdist(i) = hard(i)*0.0
                    endif
                    if(c_dst(3) .gt. 0.0) then
                        ddist(i) = (revl(i)-c_val(3))/c_dst(3)
                    else
                        ddist(i) = revl(i)*0.0
                    endif
                    dist(i) = sqrt(rdist(i)**2 + hdist(i)**2 +
     +                  ddist(i)**2)
                enddo
            endif
            do i=1,nobs
                ord(i) = i
                tdist(i) = dist(i)
            enddo
            do ii=nobs-1,1,-1
                do i=1,ii
                    if(tdist(i) .gt. tdist(i+1)) then
                        td = tdist(i)
                        tdist(i) = tdist(i+1)
                        tdist(i+1) = td
                        itd = ord(i)
                        ord(i) = ord(i+1)
                        ord(i+1) = itd
                    endif
                enddo
            enddo
c
            cum_exp = 0.0
            n = 0
            ave_rate = 0.0
            do while ((cum_exp .lt. c_tim) .and. (n .lt. nobs))
                n = n + 1
                cum_exp = cum_exp + expo(ord(n))
                ave_rate = ave_rate + rate(ord(n))*expo(ord(n))
                if(p_flg .eq. 1) then
                    if(ichip .eq. 2) then
                        write(lunp,2125) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, hard(ord(n)), uhard(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2125                   format(1x,2f10.6,2f4.0,2f6.2,32f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunp,2135) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2, 
     +                      hard(ord(n)), uhard(ord(n)), d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2, 
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2135                   format(1x,2f10.6,8f4.0,2f6.2,26f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunp,2145) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2,
     +                      hard(ord(n)), uhard(ord(n)), 
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2145                   format(1x,2f10.6,14f4.0,2f6.2,20f4.0)
                    elseif(ichip .eq. 5) then
                        write(lunp,2155) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      hard(ord(n)), uhard(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2, 
     +                      d1, d2, d1, d2, d1, d2
 2155                   format(1x,2f10.6,20f4.0,2f6.2,14f4.0)
                    elseif(ichip .eq. 6) then
                        write(lunp,2165) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, hard(ord(n)), uhard(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2
 2165                   format(1x,2f10.6,26f4.0,2f6.2,8f4.0)
                    elseif(ichip .eq. 7) then
                        write(lunp,2175) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2,
     +                      d1, d2, d1, d2, d1, d2, d1, d2, 
     +                      d1, d2, d1, d2, d1, d2, d1, d2, 
     +                      hard(ord(n)), uhard(ord(n)), d1, d2
 2175                   format(1x,2f10.6,32f4.0,2f6.2,2f4.0)
                    endif
                endif
            enddo
            ave_rate = ave_rate/cum_exp
            print *, 'Exposure: ', cum_exp
            print *, 'Exposure weighted count rate: ', ave_rate
            print *, 'Observation count rate: ', c_val(1), ' and ',
     +          'correction scale factor: ', ave_rate/c_val(1)
c
            if(p_flg .eq. 1) then
                if(ichip .eq. 2) then
                    write(lunp,2127) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, c_val(2), 0.2*c_val(2),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2
 2127               format(1x,2f10.6,4f4.0,2f6.2,30f4.0)
                elseif(ichip .eq. 3) then
                    write(lunp,2137) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, d1, d2, d1, d2, 
     +                  d1, d2, c_val(2), 0.2*c_val(2),
     +                  d1, d2, d1, d2, d1, d2, d1, d2, 
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2
 2137               format(1x,2f10.6,10f4.0,2f6.2,24f4.0)
                elseif(ichip .eq. 4) then
                    write(lunp,2147) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  c_val(2), 0.2*c_val(2), 
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2
 2147               format(1x,2f10.6,16f4.0,2f6.2,18f4.0)
                elseif(ichip .eq. 5) then
                    write(lunp,2157) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, 
     +                  c_val(2), 0.2*c_val(2),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2
 2157               format(1x,2f10.6,22f4.0,2f6.2,12f4.0)
                elseif(ichip .eq. 6) then
                    write(lunp,2167) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, c_val(2), 0.2*c_val(2),
     +                  d1, d2, d1, d2, d1, d2
 2167               format(1x,2f10.6,28f4.0,2f6.2,6f4.0)
                elseif(ichip .eq. 7) then
                    write(lunp,2177) c_val(1), 0.2*c_val(1),
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2,
     +                  d1, d2, d1, d2, d1, d2, d1, d2, 
     +                  d1, d2, d1, d2, d1, d2, d1, d2, 
     +                  d1, d2, c_val(2), 0.2*c_val(2)
 2177               format(1x,2f10.6,34f4.0,2f6.2)
                endif
            endif
        endif
c
        call ftfiou (lun,status)

c
c  Actually put together the spectrum
c
        do ii=1,2400
            aug(ii) = 0.0
        enddo
        aug_exp = 0.
        do i=1,n
            do ii=1,2400
                aug(ii) = aug(ii) + spec(ii,ord(i))
            enddo
            aug_exp = aug_exp + expo(ord(i))
        enddo
        aug_exp = aug_exp*ave_rate/c_val(1)
        call ftclos (lun,status)
        call ftfiou (lun,status)
c
        return
        end
c
c
c
        subroutine aug_get_pnexp (db_file,c_val,c_dst,c_tim,
     +      emin,emax,p_flg,aug,aug_exp,ichip,mchan,
     +      obsid,prefix,modeint,plc_lo,plc_hi)
c
        implicit none
c
        integer mchan
        integer block, err, hdutype, i, iback, ichip, icol, idate, 
     +      iexpo, iexpid, ifilt, ihard1, ihard2, ihard3, ii, ikeep, 
     +      ikey, imode, iobid, iobjt, iplot, 
     +      irate, irate1, irate2, irate3, 
     +      irate4, irevl, ispec, ispecoo, isubmode, itd, ictime, 
     +      ittime, iuhard1, iuhard2, iuhard3, iurate, iurate1, 
     +      iurate2, iurate3, iurate4, lun, lunp, lunpp, lunppp, 
     +      lunpppp, n, nb1, nb2, nb3, nchan, ncols, next, ng, nn, 
     +      nobs, nt, obsid, ord(7500), p_flg, plc_lo(2), plc_hi(2), 
     +      revl(7500), status, tfields, varidat
c
        real amscal, aug(4096), aug_exp, ave_rate, back(7500), cc, co, 
     +      c_dst(3), cum_exp, ctime(7500), c_tim, c_val(3), d1, d2, 
     +      ddist(7500), dist(7500), emax(mchan), emin(mchan), 
     +      energ(4096), expmax, expmin, expo(7500), hard1(7500), 
     +      hard2(7500), hard3(7500), hardmax, hardmin, hdist(7500), 
     +      muhard, murate, ootsca, rate(7500), rate1(7500), 
     +      rate2(7500), rate3(7500), rate4(7500), ratemax, ratemin, 
     +      rdist(7500), spec(4096,7500), specoo(4096,7500), td, 
     +      tdist(7500), 
c     +      te1, te2, teo1, teo2, 
     +      ttime(7500), tootsca, 
     +      tspec(4096), tspecoo(4096), uhard1(7500), uhard2(7500), 
     +      uhard3(7500), urate(7500), urate1(7500), urate2(7500), 
     +      urate3(7500), urate4(7500)
c
        double precision date(7500)
c
        character*1 key(7500)
        character*4 prefix
        character*6 filt(7500)
        character*8 tform(36), ttype(36), tunit(36)
        character*11 colnam(36), extna
        character*13 obid(7500)
        character*15 objt(7500)
        character*17 expid(7500)
        character*20 extname
        character*24 title
        character*25 mode(7500), modein, modeint, submode(7500) 
        character*80 augfile, db_file
        logical anyf
c
        save iplot, lunp, lunpp, lunppp, lunpppp
c
        data amscal /1440000.0/
        data iplot /0/
        status = 0
c
c  if(n_params(0) LT 7)then begin
c        print,"get_aug,db_file,r_lim,h_lim,d_lim,c_val,c_dst,c_tim,p_flg,aug,aug_exp"
c    print,"Purpose:to create a mean corner spectrum"
c    print,"  Given a rate, hardness, and date,"
c    print,"  finds the closest c_tim seconds of exposure"
c    print,"  in that 3-D space"
c    print,"Inputs:"
c    print,"  db_file:database file of corner spectra"
c    print,"  c_val:central [rate,hardness,date]"
c    print,"  c_dst:distance scaling in those directions"
c    print,"    as factor of mean uncertainty in those directions"
c    print,"    suggest:[0.00323,0.358,1.0]"
c    print,"  c_tim:how many seconds of exposure desired for spectrum"
c    print,"  p_flg:1 if informative plot to be made"
c    print,"Outputs:"
c    print,"  aug:the augmented spectrum"
c    print,"  aug_exp:the amount of exposure time in the augmented spectrum"
c    return
c  endif
c
        err = 0
        status = 0
c
        print 4001, c_val
 4001   format('Count Rate, Hardness, and Revolution: ',f8.6,f8.3,f8.0) 
        print 4002, c_dst
 4002   format('Scaling Factors for Obs. Selection: ',f8.6,f8.3,f8.0)
        if(modeint .eq. 'PrimeFullWindow') then
            print *, 'PN Mode: ', modeint
            modein = modeint
            ootsca = 0.063
        elseif(modeint .eq. 'PrimeFullWindowExtended3 ') then
            print *, 'PN Mode: ', modeint
            modein = 'PrimeFullWindowExten'
            ootsca = 0.0232
        elseif(modeint .eq. 'PrimeFullWindowExtended5 ') then
            print *, 'PN Mode: ', modeint
            modein = 'PrimeFullWindowExten'
            ootsca = 0.0163
        else
            print *, 'AUG_GET_PNEXP POSITION 01'
            print *, 'Bad submode: ', modeint
            stop
        endif
        print *, ootsca
 4005   format('OOT Scale Factor: ',f7.4)
c
c  If selected, make a plot, first set up the plot
c
        if((p_flg .eq. 1) .and. (iplot .eq. 0)) then
            iplot = 1
c
c  Get an unused lun
c
            status = 0
            call ftgiou(lunp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 02'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,2999) prefix
 2999       format('pn',a4,'-aug.qdp')
            open(lunp,form='formatted',status='unknown',
     +              file=augfile)
            write(lunp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
 3000       format('"',i10,' -- PN',a4,'"')
            write(lunp,*) ' la t ', title
            write(lunp,*) ' read serr 1'
            write(lunp,*) ' read serr 2'
            write(lunp,*) ' read serr 3'
            write(lunp,*) ' read serr 4'
            write(lunp,*) ' read serr 5'
            write(lunp,*) ' read serr 6'
            write(lunp,*) ' read serr 7'
            write(lunp,*) ' read serr 8'
            write(lunp,*) ' read serr 9'
            write(lunp,*) ' font roman'
            write(lunp,*) ' cs 1.5'
            write(lunp,*) ' la f'
            write(lunp,*) ' col 2 on 2'
            write(lunp,*) ' col 1 on 3'
            write(lunp,*) ' col 2 on 4'
            write(lunp,*) ' col 1 on 5'
            write(lunp,*) ' col 2 on 6'
            write(lunp,*) ' col 1 on 7'
            write(lunp,*) ' col 2 on 8'
            write(lunp,*) ' col 1 on 9'
            write(lunp,*) ' mark 1 on 2'
            write(lunp,*) ' mark 1 on 3'
            write(lunp,*) ' mark 1 on 4'
            write(lunp,*) ' mark 1 on 5'
            write(lunp,*) ' mark 1 on 6'
            write(lunp,*) ' mark 1 on 7'
            write(lunp,*) ' mark 1 on 8'
            write(lunp,*) ' mark 1 on 9'
            write(lunp,*) ' line off 2'
            write(lunp,*) ' line off 3'
            write(lunp,*) ' line off 4'
            write(lunp,*) ' line off 5'
            write(lunp,*) ' line off 6'
            write(lunp,*) ' line off 7'
            write(lunp,*) ' line off 8'
            write(lunp,*) ' line off 9'
            write(lunp,*) ' win 1'
            write(lunp,*) ' yplot 1 2 3'
            write(lunp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunp,*) ' la 11 p 0.14 7.0 just left "Quad #1"'
            write(lunp,*) ' r x1 0.0 0.005'
            write(lunp,*) ' r y1 0.0 8.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' win 2'
            write(lunp,*) ' yplot 1 4 5'
            write(lunp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunp,*) ' la 12 p 0.14 7.0 just left "Quad #2"'
            write(lunp,*) ' r x2 0.0 0.005'
            write(lunp,*) ' r y2 0.0 8.0'
            write(lunp,*) ' win 3'
            write(lunp,*) ' yplot 1 6 7'
            write(lunp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunp,*) ' la 13 p 0.14 7.0 just left "Quad #3"'
            write(lunp,*) ' r x3 0.0 0.005'
            write(lunp,*) ' r y3 0.0 8.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 4'
            write(lunp,*) ' yplot 1 8 9'
            write(lunp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunp,*) ' la 14 p 0.14 7.0 just left "Quad #4"'
            write(lunp,*) ' r x4 0.0 0.005'
            write(lunp,*) ' r y4 0.0 8.0'
            write(lunp,*) ' col off 1'
            write(lunp,*) ' la x Rate'
c
c  Get another unused lun for the augmented spectra
c
            status = 0
            call ftgiou(lunpp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3001) prefix
 3001       format('pn',a4,'-aug-spec.qdp')
            open(lunpp,form='formatted',status='unknown',
     +              file=augfile)
            write(lunpp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunpp,*) ' la t ', title
            write(lunpp,*) ' font roman'
            write(lunpp,*) ' cs 1.5'
            write(lunpp,*) ' la f'
            write(lunpp,*) ' col 1 on 2'
            write(lunpp,*) ' col 1 on 3'
            write(lunpp,*) ' col 1 on 4'
            write(lunpp,*) ' col 1 on 5'
            write(lunpp,*) ' mark 1 on 2'
            write(lunpp,*) ' mark 1 on 3'
            write(lunpp,*) ' mark 1 on 4'
            write(lunpp,*) ' mark 1 on 5'
            write(lunpp,*) ' line off 2'
            write(lunpp,*) ' line off 3'
            write(lunpp,*) ' line off 4'
            write(lunpp,*) ' line off 5'
            write(lunpp,*) ' win 1'
            write(lunpp,*) ' yplot 1 2'
            write(lunpp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunpp,*) ' la 11 p 4.0 200.0 just left "Quad #1"'
            write(lunpp,*) ' r x1 0.15 15.0'
            write(lunpp,*) ' r y1 0.1 500.0'
            write(lunpp,*) ' la y Counts'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 2'
            write(lunpp,*) ' yplot 1 3'
            write(lunpp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunpp,*) ' la 12 p 4.0 200.0 just left "Quad #2"'
            write(lunpp,*) ' r x2 0.15 15.0'
            write(lunpp,*) ' r y2 0.1 500.0'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 3'
            write(lunpp,*) ' yplot 1 4'
            write(lunpp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunpp,*) ' la 13 p 4.0 200.0 just left "Quad #3"'
            write(lunpp,*) ' r x3 0.15 15.0'
            write(lunpp,*) ' r y3 0.1 500.0'
            write(lunpp,*) ' la y Counts'
            write(lunpp,*) ' la x Energy (keV)'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 4'
            write(lunpp,*) ' yplot 1 5'
            write(lunpp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunpp,*) ' la 14 p 4.0 200.0 just left "Quad #4"'
            write(lunpp,*) ' r x4 0.15 15.0'
            write(lunpp,*) ' r y4 0.1 500.0'
            write(lunpp,*) ' col off 1'
            write(lunpp,*) ' la x Energy (keV)'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
c
c  Get another unused lun for the selected rate 
c  plotted with revolution
c
            status = 0
            call ftgiou(lunppp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03a'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3002) prefix
 3002       format('pn',a4,'-aug-rev-rate.qdp')
            open(lunppp,form='formatted',status='unknown',
     +              file=augfile)
            write(lunppp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunppp,*) ' la t ', title
            write(lunppp,*) ' read serr 2'
            write(lunppp,*) ' read serr 3'
            write(lunppp,*) ' read serr 4'
            write(lunppp,*) ' read serr 5'
            write(lunppp,*) ' read serr 6'
            write(lunppp,*) ' read serr 7'
            write(lunppp,*) ' read serr 8'
            write(lunppp,*) ' read serr 9'
            write(lunppp,*) ' font roman'
            write(lunppp,*) ' cs 1.5'
            write(lunppp,*) ' la f'
            write(lunppp,*) ' col 1 on 2'
            write(lunppp,*) ' col 2 on 3'
            write(lunppp,*) ' col 1 on 4'
            write(lunppp,*) ' col 3 on 5'
            write(lunppp,*) ' col 1 on 6'
            write(lunppp,*) ' col 4 on 7'
            write(lunppp,*) ' col 1 on 8'
            write(lunppp,*) ' col 5 on 9'
            write(lunppp,*) ' mark 1 on 2'
            write(lunppp,*) ' mark 1 on 3'
            write(lunppp,*) ' mark 1 on 4'
            write(lunppp,*) ' mark 1 on 5'
            write(lunppp,*) ' mark 1 on 6'
            write(lunppp,*) ' mark 1 on 7'
            write(lunppp,*) ' mark 1 on 8'
            write(lunppp,*) ' mark 1 on 9'
            write(lunppp,*) ' line off 2'
            write(lunppp,*) ' line off 3'
            write(lunppp,*) ' line off 4'
            write(lunppp,*) ' line off 5'
            write(lunppp,*) ' line off 6'
            write(lunppp,*) ' line off 7'
            write(lunppp,*) ' line off 8'
            write(lunppp,*) ' line off 9'
            write(lunppp,*) ' win 1'
            write(lunppp,*) ' yplot 1 2 3'
            write(lunppp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunppp,*) ' la 11 p 200.0 0.0026 just left "Quad #1"'
            write(lunppp,*) ' r x1 0.0 2000.0'
            write(lunppp,*) ' r y1 0.0 0.003'
            write(lunppp,*) ' la y Count Rate'
            write(lunppp,*) ' win 2'
            write(lunppp,*) ' yplot 1 4 5'
            write(lunppp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunppp,*) ' la 12 p 200.0 0.0026 just left "Quad #2"'
            write(lunppp,*) ' r x2 0.0 2000.0'
            write(lunppp,*) ' r y2 0.0 0.003'
            write(lunppp,*) ' win 3'
            write(lunppp,*) ' yplot 1 6 7'
            write(lunppp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunppp,*) ' la 13 p 200.0 0.0026 just left "Quad #3"'
            write(lunppp,*) ' r x3 0.0 2000.0'
            write(lunppp,*) ' r y3 0.0 0.003'
            write(lunppp,*) ' la y Count Rate'
            write(lunppp,*) ' la x Revolution'
            write(lunppp,*) ' win 4'
            write(lunppp,*) ' yplot 1 8 9'
            write(lunppp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunppp,*) ' la 14 p 200.0 0.0026 just left "Quad #4"'
            write(lunppp,*) ' r x4 0.0 2000.0'
            write(lunppp,*) ' r y4 0.0 0.003'
            write(lunppp,*) ' col off 1'
            write(lunppp,*) ' la x Revolution'
c
c  Get another unused lun for the selected rate 
c  plotted with revolution
c
            status = 0
            call ftgiou(lunpppp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03b'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3003) prefix
 3003       format('pn',a4,'-aug-rev-hard.qdp')
            open(lunpppp,form='formatted',status='unknown',
     +              file=augfile)
            write(lunpppp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunpppp,*) ' la t ', title
            write(lunpppp,*) ' read serr 2'
            write(lunpppp,*) ' read serr 3'
            write(lunpppp,*) ' read serr 4'
            write(lunpppp,*) ' read serr 5'
            write(lunpppp,*) ' read serr 6'
            write(lunpppp,*) ' read serr 7'
            write(lunpppp,*) ' read serr 8'
            write(lunpppp,*) ' read serr 9'
            write(lunpppp,*) ' font roman'
            write(lunpppp,*) ' cs 1.5'
            write(lunpppp,*) ' la f'
            write(lunpppp,*) ' col 1 on 2'
            write(lunpppp,*) ' col 2 on 3'
            write(lunpppp,*) ' col 1 on 4'
            write(lunpppp,*) ' col 3 on 5'
            write(lunpppp,*) ' col 1 on 6'
            write(lunpppp,*) ' col 4 on 7'
            write(lunpppp,*) ' col 1 on 8'
            write(lunpppp,*) ' col 5 on 9'
            write(lunpppp,*) ' mark 1 on 2'
            write(lunpppp,*) ' mark 1 on 3'
            write(lunpppp,*) ' mark 1 on 4'
            write(lunpppp,*) ' mark 1 on 5'
            write(lunpppp,*) ' mark 1 on 6'
            write(lunpppp,*) ' mark 1 on 7'
            write(lunpppp,*) ' mark 1 on 8'
            write(lunpppp,*) ' mark 1 on 9'
            write(lunpppp,*) ' line off 2'
            write(lunpppp,*) ' line off 3'
            write(lunpppp,*) ' line off 4'
            write(lunpppp,*) ' line off 5'
            write(lunpppp,*) ' line off 6'
            write(lunpppp,*) ' line off 7'
            write(lunpppp,*) ' line off 8'
            write(lunpppp,*) ' line off 9'
            write(lunpppp,*) ' win 1'
            write(lunpppp,*) ' yplot 1 2 3'
            write(lunpppp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunpppp,*) ' la 11 p 200.0 9.0 just left "Quad #1"'
            write(lunpppp,*) ' r x1 0.0 2000.0'
            write(lunpppp,*) ' r y1 0.0 10.0'
            write(lunpppp,*) ' la y Hardness'
            write(lunpppp,*) ' win 2'
            write(lunpppp,*) ' yplot 1 4 5'
            write(lunpppp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunpppp,*) ' la 12 p 200.0 9.0 just left "Quad #2"'
            write(lunpppp,*) ' r x2 0.0 2000.0'
            write(lunpppp,*) ' r y2 0.0 10.0'
            write(lunpppp,*) ' win 3'
            write(lunpppp,*) ' yplot 1 6 7'
            write(lunpppp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunpppp,*) ' la 13 p 200.0 9.0 just left "Quad #3"'
            write(lunpppp,*) ' r x3 0.0 2000.0'
            write(lunpppp,*) ' r y3 0.0 10.0'
            write(lunpppp,*) ' la y Hardness'
            write(lunpppp,*) ' la x Revolution'
            write(lunpppp,*) ' win 4'
            write(lunpppp,*) ' yplot 1 8 9'
            write(lunpppp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunpppp,*) ' la 14 p 200.0 9.0 just left "Quad #4"'
            write(lunpppp,*) ' r x4 0.0 2000.0'
            write(lunpppp,*) ' r y4 0.0 10.0'
            write(lunpppp,*) ' col off 1'
            write(lunpppp,*) ' la x Revolution'
        endif
c
c  Get an unused lun for the CalDB file
c
        err = 0
        status = 0
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_PNEXP POSITION 04'
            print *, 'STATUS FTGIOU =', status
        endif
c
c  Open the FITS file
c
        call ftopen (lun,db_file,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_PNEXP POSITION 05'
            print *, 'STATUS FTOPEN =', status
            print *, db_file
        endif
        print *, 'db_file: ', db_file
c
c  Move to the correct extension
c
        if(err .eq. 0) then
            next = ichip + 1
            call ftmahd (lun,next,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 06'
                print *, 'STATUS FTMAHD =', status
                print *, 'Could not find the spectrum extension'
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,36,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                write(extna,3011) ichip
 3011           format('QUIES_QUAD',i1)
                if(extname .ne. extna) then
                    print *, extname
                    print *, 'This was supposed to be a QUIES ext ', 
     +                      'for quadrant: ', ichip
                    stop
                else
                    print *, 'Quadrant and QPB Extension'
                    print *, ichip, extname
                endif
                if(status .ne. 0) then
                    err = 1
                    print *, 'AUG_GET_PNEXP POSITION 07'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'Failure on reading the header ',
     +                      'information'
                else
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,36,colnam,ncols,status)
                    if(ncols .gt. 7500) then
                        print *, 'Too much data: ', ncols
                        print *, 'Limit selection to 7500'
                        ncols = 7500
                    endif
c
c  Extract the channel numbers
c
                    nt = 0
                    ng = 0
                    nb1 = 0
                    nb2 = 0
                    nb3 = 0
                    iobid = 0
                    iexpid = 0
                    iobjt = 0
                    idate = 0
                    irevl = 0
                    imode = 0
                    isubmode = 0
                    ifilt = 0
                    ittime = 0
                    ictime = 0
                    ikey = 0
                    iexpo = 0
                    irate = 0
                    iurate = 0
                    irate1 = 0
                    iurate1 = 0
                    irate2 = 0
                    iurate2 = 0
                    irate3 = 0
                    iurate3 = 0
                    irate4 = 0
                    iurate4 = 0
                    ihard1 = 0
                    iuhard1 = 0
                    ihard2 = 0
                    iuhard2 = 0
                    ihard3 = 0
                    iuhard3 = 0
                    ispec = 0
                    ispecoo = 0
                    iback = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'OBSID') iobid = icol
                        if(colnam(icol) .eq. 'EXP_ID') iexpid = icol
                        if(colnam(icol) .eq. 'OBJECT') iobjt = icol
                        if(colnam(icol) .eq. 'DATE') idate = icol
                        if(colnam(icol) .eq. 'REVOLUT') irevl = icol
                        if(colnam(icol) .eq. 'OBS_MODE') imode = icol
                        if(colnam(icol) .eq. 'SUB_MODE') isubmode = icol
                        if(colnam(icol) .eq. 'FILTER') ifilt = icol
                        if(colnam(icol) .eq. 'TOTAL_TIME') ittime = icol
                        if(colnam(icol) .eq. 'CLEAN_TIME') ictime = icol
                        if(colnam(icol) .eq. 'KEY') ikey = icol
                        if(colnam(icol) .eq. 'EXPO') iexpo = icol
                        if(colnam(icol) .eq. 'RATE') irate = icol
                        if(colnam(icol) .eq. 'RATE_ERR') iurate = icol
                        if(colnam(icol) .eq. 'RATE1') irate1 = icol
                        if(colnam(icol) .eq. 'RATE1_ERR_1') 
     +                          iurate1 = icol
                        if(colnam(icol) .eq. 'RATE2') irate2 = icol
                        if(colnam(icol) .eq. 'RATE2_ERR_2') 
     +                          iurate2 = icol
                        if(colnam(icol) .eq. 'RATE3') irate3 = icol
                        if(colnam(icol) .eq. 'RATE3_ERR_3') 
     +                          iurate3 = icol
                        if(colnam(icol) .eq. 'RATE4') irate4 = icol
                        if(colnam(icol) .eq. 'RATE4_ERR_4') 
     +                          iurate4 = icol
                        if(colnam(icol) .eq. 'HARD_1') ihard1 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_1') 
     +                          iuhard1 = icol
                        if(colnam(icol) .eq. 'HARD_2') ihard2 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_2') 
     +                          iuhard2 = icol
                        if(colnam(icol) .eq. 'HARD_3') ihard3 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_3') 
     +                          iuhard3 = icol
                        if(colnam(icol) .eq. 'SPECTRUM') ispec = icol
                        if(colnam(icol) .eq. 'SPECTRUM_OO') 
     +                          ispecoo = icol
                        if(colnam(icol) .eq. 'BACKSCAL') iback = icol
                    enddo
c                    print *, colnam
c                    print *, ncols, iobid, iexpid, iobjt, idate, irevl,
c     +                   imode, isubmode, ifilt, ittime, ictime, ikey, 
c     +                   iexpo, irate, 
c     +                   iurate, ihard2, iuhard2, ispec, ispecoo, iback
c                    stop
                    nn = 1
                    do i=1,nchan
                        nt = nt + 1
                        if(ikey .ne. 0)
     +                      call ftgcvs(lun,ikey,i,1,1,' ',
     +                          key(nn),anyf,status)
                        if((key(nn) .eq. 'G') .or. 
     +                          (key(nn) .eq. ' ') .or. 
     +                          (key(nn) .eq. 'O')) then
                            if(iobid .ne. 0)
     +                          call ftgcvs(lun,iobid,i,1,1,' ',
     +                              obid(nn),anyf,status)
                            if(iexpid .ne. 0)
     +                          call ftgcvs(lun,iexpid,i,1,1,' ',
     +                              expid(nn),anyf,status)
                            if(iobjt .ne. 0)
     +                          call ftgcvs(lun,iobjt,i,1,1,' ',
     +                              objt(nn),anyf,status)
                            if(idate .ne. 0)
     +                          call ftgcvd(lun,idate,i,1,1,0,
     +                              date(nn),anyf,status)
                            if(irevl .ne. 0)
     +                          call ftgcvj(lun,irevl,i,1,1,0,
     +                              revl(nn),anyf,status)
                                if(anyf) revl(nn) = 0
                            if(ifilt .ne. 0)
     +                          call ftgcvs(lun,ifilt,i,1,1,' ',
     +                              filt(nn),anyf,status)
                            if(ictime .ne. 0)
     +                          call ftgcve(lun,ictime,i,1,1,0,
     +                              ctime(nn),anyf,status)
                                if(anyf) ctime(nn) = 0.0
                            if(ittime .ne. 0)
     +                          call ftgcve(lun,ittime,i,1,1,0,
     +                              ttime(nn),anyf,status)
                                if(anyf) ttime(nn) = 0.0
                            if(iexpo .ne. 0)
     +                          call ftgcve(lun,iexpo,i,1,1,0,
     +                              expo(nn),anyf,status)
                            if(iback .ne. 0) 
     +                          call ftgcve(lun,iback,i,1,1,0,
     +                              back(nn),anyf,status)
                            if(imode .ne. 0)
     +                          call ftgcvs(lun,imode,i,1,1,' ',
     +                              mode(nn),anyf,status)
                            if(isubmode .ne. 0)
     +                          call ftgcvs(lun,isubmode,i,1,1,' ',
     +                              submode(nn),anyf,status)
                            if(irate .ne. 0)
     +                          call ftgcve(lun,irate,i,1,1,0,
     +                              rate(nn),anyf,status)
                                if(anyf) rate(nn) = 0.0
                            if(iurate .ne. 0)
     +                          call ftgcve(lun,iurate,i,1,1,0,
     +                              urate(nn),anyf,status)
                                if(anyf) urate(nn) = 0.0
                            if(irate1 .ne. 0) 
     +                          call ftgcve(lun,irate1,i,1,1,0,
     +                              rate1(nn),anyf,status)
                                if(anyf) rate1(nn) = 0.0
                            if(iurate1 .ne. 0) 
     +                          call ftgcve(lun,iurate1,i,1,1,0,
     +                              urate1(nn),anyf,status)
                                if(anyf) urate1(nn) = 0.0
                            if(irate2 .ne. 0) 
     +                          call ftgcve(lun,irate2,i,1,1,0,
     +                              rate2(nn),anyf,status)
                                if(anyf) rate2(nn) = 0.0
                            if(iurate2 .ne. 0) 
     +                          call ftgcve(lun,iurate2,i,1,1,0,
     +                              urate2(nn),anyf,status)
                                if(anyf) urate2(nn) = 0.0
                            if(irate3 .ne. 0) 
     +                          call ftgcve(lun,irate3,i,1,1,0,
     +                              rate3(nn),anyf,status)
                                if(anyf) rate3(nn) = 0.0
                            if(iurate3 .ne. 0) 
     +                          call ftgcve(lun,iurate3,i,1,1,0,
     +                              urate3(nn),anyf,status)
                                if(anyf) urate3(nn) = 0.0
                            if(irate4 .ne. 0) 
     +                          call ftgcve(lun,irate4,i,1,1,0,
     +                              rate4(nn),anyf,status)
                                if(anyf) rate4(nn) = 0.0
                            if(iurate4 .ne. 0) 
     +                          call ftgcve(lun,iurate4,i,1,1,0,
     +                              urate4(nn),anyf,status)
                                if(anyf) urate4(nn) = 0.0
                            if(ihard1 .ne. 0)
     +                          call ftgcve(lun,ihard1,i,1,1,0,
     +                              hard1(nn),anyf,status)
                                if(anyf)  hard1(nn) = 0.0
                            if(iuhard1 .ne. 0)
     +                          call ftgcve(lun,iuhard1,i,1,1,0,
     +                              uhard1(nn),anyf,status)
                                if(anyf)  uhard1(nn) = 0.0
                            if(ihard2 .ne. 0)
     +                          call ftgcve(lun,ihard2,i,1,1,0,
     +                              hard2(nn),anyf,status)
                                if(anyf) hard2(nn) = 0.0
                            if(iuhard2 .ne. 0)
     +                          call ftgcve(lun,iuhard2,i,1,1,0,
     +                              uhard2(nn),anyf,status)
                                if(anyf) uhard2(nn) = 0.0
                            if(ihard3 .ne. 0)
     +                          call ftgcve(lun,ihard3,i,1,1,0,
     +                              hard3(nn),anyf,status)
                                if(anyf) hard3(nn) = 0.0
                            if(iuhard3 .ne. 0)
     +                          call ftgcve(lun,iuhard3,i,1,1,0,
     +                              uhard3(nn),anyf,status)
                                if(anyf) uhard3(nn) = 0.0
c
c  Extract the spectrum and normalize by exposure and area
c
                            if(ispec .ne. 0) then
                                call ftgcve(lun,ispec,i,1,4096,0,
     +                                  tspec,anyf,status)
                                do ii=1,4096
                                    spec(ii,nn) = amscal*tspec(ii)/
     +                                      (expo(nn)*back(nn))
                                enddo
                                if((i .eq. nchan) .and. 
     +                                  (status .ne. 0)) status = 0
                            endif
                            rate(nn) = 1.0e6*rate(nn)
                            urate(nn) = 1.0e6*urate(nn)
c
c                            write(94,*) ichip, i, nn, revl(nn), 
c     +                              expo(nn), ttime(nn), rate(nn), 
c     +                              hard2(nn) 
c
c  Extract the OOT spectrum and normalize by exposure and area
c
                            if(ispecoo .ne. 0) then
                                call ftgcve(lun,ispecoo,i,1,4096,0,
     +                                  tspecoo,anyf,status)
                                do ii=1,4096
                                    specoo(ii,nn) = amscal*tspecoo(ii)/
     +                                      (expo(nn)*back(nn))
                                enddo
                                if((i .eq. nchan) .and. 
     +                                  (status .ne. 0)) status = 0
                            endif
c
c  Set the OOT scale factor
c
                            if(submode(nn) .eq. 
     +                              '        PrimeFullWindow') then
                                tootsca = 0.063
                            elseif(submode(nn) .eq. 
     +                              'PrimeFullWindowExtended') then
                                tootsca = 0.0232
c
c  Full frame extended 5 is well hidden
c
                            elseif(submode(nn) .eq. 
     +                              'PrimeFullWindowExtended') then
                                tootsca = 0.0163
                            else
                                print *, 'AUG_GET_PNEXP POSITION 08'
                                print *, 'Bad submode: ', submode(nn)
                                stop
                            endif
cc
cc  Find the normal and OOT counts in the low and high spectral bands
cc
c                        te1 = 0.0
c                        teo1 = 0.0
c                        do ii=plc_lo(1),plc_lo(2)
c                            te1 = te1 + spec(ii,nn)
c                            teo1 = teo1 + specoo(ii,nn)
c                        enddo
c                        te2 = 0.0
c                        teo2 = 0.0
c                        do ii=plc_hi(1),plc_hi(2)
c                            te2 = te2 + spec(ii,nn)
c                            teo2 = teo2 + specoo(ii,nn)
c                        enddo
cc
cc  Find the OOT corrected count rates
cc
c                        rate1(nn) = (te1 - tootsca*teo1)
c                        urate1(nn) = sqrt(te1*expo(nn)*back(nn)/amscal + 
c     +                          tootsca*teo1*expo(nn)*back(nn)/amscal)*
c     +                          amscal/expo(nn)/back(nn)
c                        rate2(nn) = (te2 - tootsca*teo2)
c                        urate2(nn) = sqrt(te2*expo(nn)*back(nn)/amscal + 
c     +                          tootsca*teo2*expo(nn)*back(nn)/amscal)*
c     +                          amscal/expo(nn)/back(nn)
cc                        print *, rate1(nn),urate1(nn),rate2(nn),urate2(nn)
c                        if((rate1(nn) .gt. 0.0) .and. 
c     +                          (rate2(nn) .gt. 0.0)) then
c                            hard(nn) = rate2(nn)/rate1(nn)
c                            uhard(nn) = sqrt((urate2(nn)/rate1(nn))**2 + 
c     +                          (rate2(nn)*urate1(nn)/(rate1(nn)**2))**2)
c                            rate(nn) = rate1(nn) + rate2(nn)
c                            urate(nn) = sqrt(urate1(nn)**2 + urate2(nn)**2)
c                            ng = ng + 1
c                        else
c                            hard(nn) = 0.0
c                            uhard(nn) = 0.0
c                            rate(nn) = 0.0
c                            urate(nn) = 0.0
c                            nb = nb + 1
c                        endif
c
cc                        print *, rate1(nn),urate1(nn),rate2(nn),urate2(nn),
cc     +                        expo(nn), back(nn)
cc                        write(nlun,2345) date(nn), rate1(nn), urate1(nn),
cc     +                      rate2(nn),urate2(nn) 
cc 2345                   format(1x,f15.1,4f15.7)
cc
                            nn = nn + 1
                        endif
                    enddo
                    nn = nn - 1
                    if(status .ne. 0) then
                        print *, 'AUG_GET_PNEXP POSITION 09'
                        print *, 'Missing column in FWC info file'
                        print *, 'STATUS: ', status
                        return
                    endif
                endif
            endif
        endif
c        print *, 'ng, nb: ', ng, nb
        call ftclos (lun,status)
c
c  Set the normal filtering limits
c
        d1 = -9.0
        d2 = 0.0
        hardmin = 3.0
        hardmax = 7.0
        ratemin = 1.0e-3
        ratemax = 3.0e-3
        expmin = 5000.0
        expmax = 40000.0
c
c  Sort through the archived observation data collecting
c  the specifics on possible observations that can be used
c  for the augmentation.
c
        nobs = 0
        do i=1,nn
c            print 3999, p_flg, ichip, expo(i), rate(i), ratemin,
c     +          ratemax, hard(i), hardmin, hardmax, submode(i), modein
c 3999       format(2i4,f8.1,3f8.3,3f8.1,3x,2a25)
c            print *, submode(i), modein
c
            if((expo(i) .gt. 0.0) .and. (ttime(i) .gt. 0.0)) then
              if((expo(i) .ge. expmin) .and. 
     +              (expo(i) .lt. expmax) .and.
     +              (rate(i) .gt. ratemin) .and. 
     +              (rate(i) .lt. ratemax) .and. 
     +              (hard2(i) .gt. hardmin) .and.
     +              (hard2(i) .lt. hardmax) .and.
     +              (expo(i) .gt. 0.8*ttime(i))) then
c
c                write(94,*) ichip, i, revl(i), expo(i), ttime(i), 
c     +                  rate(i), hard2(i) 
c
c  Write out all the data for plotting if selected by the 
c  rate, exposure, and hardness criteria
c
                if(p_flg .eq. 1) then
                    if(ichip .eq. 1) then
                        write(lunpppp,2820) revl(i),
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2820                   format(1x,i6,2f6.2,14f4.0)
                        write(lunppp,2720) revl(i),
     +                      rate(i), urate(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2720                   format(1x,i6,2f10.6,14f4.0)
                        write(lunp,2120) rate(i), urate(i),
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2120                   format(1x,2f10.6,2f6.2,22f4.0)
                    elseif(ichip .eq. 2) then
                        write(lunpppp,2830) revl(i), 
     +                      d1, d2, d1, d2, 
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2830                   format(1x,i6,4f4.0,2f6.2,10f4.0)
                        write(lunppp,2730) revl(i), 
     +                      d1, d2, d1, d2, 
     +                      rate(i), urate(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2730                   format(1x,i6,4f4.0,2f10.6,10f4.0)
                        write(lunp,2130) rate(i), urate(i),
     +                      d1, d2, d1, d2,
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2130                   format(1x,2f10.6,4f4.0,2f6.2,18f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunpppp,2840) revl(i), 
     +                      d1, d2, d1, d2, 
     +                      d1, d2, d1, d2,
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2
 2840                   format(1x,i6,8f4.0,2f6.2,6f4.0)
                        write(lunppp,2740) revl(i), 
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2, 
     +                      rate(i), urate(i), d1, d2,
     +                      d1, d2, d1, d2
 2740                   format(1x,i6,8f4.0,2f10.6,6f4.0)
                        write(lunp,2140) rate(i), urate(i),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      hard2(i), uhard2(i), d1, d2,
     +                      d1, d2, d1, d2
 2140                   format(1x,2f10.6,8f4.0,2f6.2,14f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunpppp,2850) revl(i), 
     +                      d1, d2, d1, d2, 
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      hard2(i), uhard2(i), d1, d2
 2850                   format(1x,i6,12f4.0,2f6.2,2f4.0)
                        write(lunppp,2750) revl(i), 
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2, 
     +                      d1, d2, d1, d2,
     +                      rate(i), urate(i), d1, d2
 2750                   format(1x,i6,12f4.0,2f10.6,2f4.0)
                        write(lunp,2150) rate(i), urate(i),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      hard2(i), uhard2(i), d1, d2
 2150                   format(1x,2f10.6,12f4.0,2f6.2,10f4.0)
                    endif
                endif
c
c  Check to see whether the observation CCD is in an odd state
c  and select whether the data will be kept or not
c
                ikeep = 1
c
c  Keep the data set so transfer the information
c
                if(ikeep .eq. 1) then
                    nobs = nobs + 1
                    obid(nobs) = obid(i)
                    expid(nobs) = expid(i)
                    objt(nobs) = objt(i)
                    date(nobs) = date(i)
                    revl(nobs) = revl(i)
                    filt(nobs) = filt(i)
                    mode(nobs) = mode(i)
                    submode(nobs) = submode(i)
                    ttime(nobs) = ttime(i)
                    ctime(nobs) = ctime(i)
                    back(nobs) = back(i)
                    expo(nobs) = expo(i)
                    rate(nobs) = rate(i)
                    urate(nobs) = urate(i)
                    hard2(nobs) = hard2(i)
                    uhard2(nobs) = uhard2(i)
c            print *, (spec(ii,i),ii=40,50)
                    do ii=1,4096
                        spec(ii,nobs) = spec(ii,i)
                        specoo(ii,nobs) = specoo(ii,i)
                    enddo
                endif
              else
                  nb2 = nb2 + 1
              endif
            else
                nb3 = nb3 + 1
            endif
        enddo
c
c  Set the energies of the bins
c
        do i=1,4096
            energ(i) = 0.005*(i - 1)
        enddo
c
c  Get some temporary values for the average rate and hardness
c  errors for the corner data
c
        murate = 0.0
        muhard = 0.0
        do i=1,nobs
            murate = murate + urate(i)
            muhard = muhard + uhard2(i)
        enddo
        murate = murate/nobs
        muhard = muhard/nobs
c
c  Find the closest c_tim secs
c
        if(c_tim .eq. 0.0) then
            print *, 'AUG_GET_PNEXP POSITION 11'
            print *, 'No exposure length provided'
            stop
        else
            if((c_dst(1) .eq. 0.0) .and. (c_dst(2) .eq. 0.0) .and.
     +              (c_dst(3) .eq. 0.0)) then
                print *, 'AUG_GET_PNEXP POSITION 12'
                print *, 'No distance scalings'
                stop
            else
c
c  First, set the distance in parameter space
c
                do i=1,nobs
                    if(c_dst(1) .ne. 0.0) then
                        rdist(i) = (rate(i)-c_val(1))/c_dst(1)
                    else
                        rdist(i) = 0.0
                    endif
                    if(c_dst(2) .ne. 0.0) then
                        hdist(i) = (hard2(i)-c_val(2))/c_dst(2)
                    else
                        hdist(i) = 0.0
                    endif
                    if(c_dst(3) .ne. 0.0) then
                        ddist(i) = (revl(i)-c_val(3))/c_dst(3)
                    else
                        ddist(i) = 0.0
                    endif
                    dist(i) = sqrt(rdist(i)**2 + hdist(i)**2 +
     +                  ddist(i)**2)
                enddo
            endif
c
c  Sort through the CalDB data to order them by the distance
c  from the observation values.
c
            do i=1,nobs
                ord(i) = i
                tdist(i) = dist(i)
            enddo
            do ii=nobs-1,1,-1
                do i=1,ii
                    if(tdist(i) .gt. tdist(i+1)) then
                        td = tdist(i)
                        tdist(i) = tdist(i+1)
                        tdist(i+1) = td
                        itd = ord(i)
                        ord(i) = ord(i+1)
                        ord(i+1) = itd
                    endif
                enddo
            enddo
c
c  Now accumulate spectra until a limiting exposure is reached
c
            cum_exp = 0.0
            n = 0
            ave_rate = 0.0
            do while ((cum_exp .lt. c_tim) .and. (n .lt. nobs))
                ng = ng + 1
                n = n + 1
                cum_exp = cum_exp + expo(ord(n))
                ave_rate = ave_rate + rate(ord(n))*expo(ord(n))
                if(p_flg .eq. 1) then
                    if(ichip .eq. 1) then
                        write(lunpppp,2825) revl(ord(n)),
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2825                   format(1x,i6,2f4.0,2f6.2,12f4.0)
                        write(lunppp,2725) revl(ord(n)),
     +                      d1, d2, rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2725                   format(1x,i6,2f4.0,2f10.6,12f4.0)
                        write(lunp,2125) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2125                   format(1x,2f10.6,2f4.0,2f6.2,20f4.0)
                    elseif(ichip .eq. 2) then
                        write(lunpppp,2835) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2835                   format(1x,i6,6f4.0,2f6.2,8f4.0)
                        write(lunppp,2735) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2735                   format(1x,i6,6f4.0,2f10.6,8f4.0)
                        write(lunp,2135) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2
 2135                   format(1x,2f10.6,6f4.0,2f6.2,16f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunpppp,2845) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2
 2845                   format(1x,i6,10f4.0,2f6.2,4f4.0)
                        write(lunppp,2745) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2
 2745                   format(1x,i6,10f4.0,2f10.6,4f4.0)
                        write(lunp,2145) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n)),
     +                      d1, d2, d1, d2
 2145                   format(1x,2f10.6,10f4.0,2f6.2,12f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunpppp,2855) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n))
 2855                   format(1x,i6,14f4.0,2f6.2)
                        write(lunppp,2755) revl(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, rate(ord(n)), urate(ord(n))
 2755                   format(1x,i6,14f4.0,2f10.6)
                        write(lunp,2155) rate(ord(n)), urate(ord(n)),
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, d1, d2,
     +                      d1, d2, hard2(ord(n)), uhard2(ord(n))
 2155                   format(1x,2f10.6,14f4.0,2f6.2,8f4.0)
                    endif
                endif
            enddo
            ave_rate = ave_rate/cum_exp
            print 4010, cum_exp
 4010       format('Exposure: ',f10.1)
            print 4020, ave_rate
 4020       format('Exposure weighted count rate: ',f10.6)
            print 4030, c_val(1)
 4030       format('Observation count rate: ',f10.6)
            nb1 = nobs - ng
            print *, 'Total Obs, Good Observations, Selected Obs, ',
     +              'Not Selected Obs: ', nt, nn, ng, nb1
            print *, 'Out of Range Obs, Bad Exposure Obs: ', 
     +              nb2, nb3
        endif
c
        call ftfiou (lun,status)

c
c  Actually put together the spectrum
c
        do ii=1,4096
            aug(ii) = 0.0
        enddo
        aug_exp = 0.
        do i=1,n
            do ii=1,4096
                aug(ii) = aug(ii) + (spec(ii,ord(i)) - 
     +                  ootsca*specoo(ii,ord(i)))*expo(ord(i))
            enddo
            aug_exp = aug_exp + expo(ord(i))
        enddo
        do ii=1,4096
            aug(ii) = aug(ii)/aug_exp
        enddo
c
        cc = 0.0
        co = 0.0
        do i=plc_lo(1),plc_lo(2)
            cc = cc + aug(i) 
        enddo
        do i=plc_hi(1),plc_hi(2)
            co = co + aug(i) 
        enddo
c
        print 4040, cc+co
 4040   format('Augmented count rate: ',f10.6)
        print 4050, co/cc
 4050   format('Augmented hardness ratio: ', f8.3)
        print 4060, aug_exp
 4060   format('Augmented exposure: ',f10.1)
        if(p_flg .eq. 1) then
            do i=1,4096
                if(ichip .eq. 1) then
                    write(lunpp,2225) energ(i), 1.0e6*aug(i), 
     +                      d1, d1, d1 
 2225               format(1x,f10.3,f10.5,3f4.0)
                elseif(ichip .eq. 2) then
                    write(lunpp,2235) energ(i), d1, 1.0e6*aug(i), 
     +                      d1, d1
 2235               format(1x,f10.3,f4.0,f10.5,2f4.0)
                elseif(ichip .eq. 3) then
                    write(lunpp,2245) energ(i), d1, d1, 
     +                      1.0e6*aug(i), d1
 2245               format(1x,f10.3,2f4.0,f10.5,f4.0)
                elseif(ichip .eq. 4) then
                    write(lunpp,2255) energ(i), d1, d1, d1, 
     +                      1.0e6*aug(i)
 2255               format(1x,f10.3,3f4.0,f10.5)
                endif
            enddo
        endif
        call ftclos (lun,status)
        call ftfiou (lun,status)
c
        return
        end
c
c
c
        subroutine binit (time,rate,bin,bint,nchan,n,bin_rate,
     +      tstart,tend)
c
        integer n, nchan
        real bint, rate(nchan), bin_rate(nchan), t
        double precision tend(nchan), time(nchan), tstart(nchan)
c
        t = time(1) - 0.5*bint
        c = rate(1)
        i = 2
        n = 1
        tstart(1) = time(1) - 0.5*bint
        nn = 1
        do while (i .lt. nchan)
            do while ((time(i) - t .lt. bin) .and. (i .le. nchan))
                c = c + rate(i)
                nn = nn + 1
                i = i + 1
            enddo
            if(nn .gt. 0) then
                bin_rate(n) = c/nn
                tend(n) = time(i-1) + 0.5*bint
                n = n + 1
            endif
            tstart(n) = time(i) - 0.5*bint
            c = 0.0
            nn = 0
            t = time(i) - 0.5*bint
        enddo
        n = n - 1
c
        return
        end
c
c
c
        SUBROUTINE DCVSRT(COVAR,NCVM,MA,LISTA,MFIT)
C
CC  Repacks the covarience matrix for MQMLMN
C
C************************ FFORM VERSION 1.0 ************  1-MAR-88 14:40
C
CA  author : Press et al.      date:
CU  update : J. Bloch          date:
CU  update : SLS               date:  1-MAR-1988 14:39
C
CT  status: not tested
C
C   general description
CG  DCVSRT is the REAL*8 version of routine COVSRT found in "Numerical
CG  Recipes" by Press et al. See p 515 for details. The only difference
CG  is that array COVAR is REAL*8 instead of single precision.  Given
CG  the covarience matrix COVAR of a fit MFIT of MA total parameters,
CG  and there ordering LISTA, repack the covarience matrix to the true
CG  order of the parameters.  Elements associated with fixed parameters
CG  will be zero.  NCVM is the physical dimension of COVAR
C
C   call_var.          type I/O description
CP  MA                  I4  I   total number of parameters
CP  MFIT                I4  I   number of active parameters
CP  NCVM                I4      dimension of COVAR array
CP  COVAR               R8  I/O covarience matrix
CP  LISTA               I4  I   position vector
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        INTEGER I, J, MA, MFIT, LISTA(MFIT), NCVM
C
        DOUBLE PRECISION COVAR(NCVM,NCVM), SWAP
C
C  Zero all elements below the diagonal
C
        DO 12 J=1,MA-1
            DO 11 I=J+1,MA
                COVAR(I,J) = 0.
   11       ENDDO
   12   ENDDO
C
C  Repack off-diagonal elements of fit into correct locations
C  below the diagonal
C
        DO 14 I=1,MFIT-1
            DO 13 J=I+1,MFIT
                IF(LISTA(J) .GT. LISTA(I)) THEN
                    COVAR(LISTA(J),LISTA(I)) = COVAR(I,J)
                ELSE
                    COVAR(LISTA(I),LISTA(J)) = COVAR(I,J)
                ENDIF
   13       ENDDO
   14   ENDDO
C
C  Temporarily store origial elements in top row, and zero the diagonal
C
        SWAP = COVAR(1,1)
C
        DO 15 J=1,MA
            COVAR(1,J) = COVAR(J,J)
            COVAR(J,J) = 0.
   15   ENDDO
C
        COVAR(LISTA(1),LISTA(1)) = SWAP
C
C  Now sort the elements into proper order on diagonal
C
        DO 16 J=2,MFIT
            COVAR(LISTA(J),LISTA(J)) = COVAR(1,J)
   16   ENDDO
C
C  Finally, fill in above the diagonal by symmetry
C
        DO 18 J=2,MA
            DO 17 I=1,J-1
                COVAR(I,J) = COVAR(J,I)
   17       ENDDO
   18   ENDDO
C
        RETURN
        END
c
c
c
        SUBROUTINE DGAUSJ(A,N,NP,B,M,MP,IERR)
C
CC  Numerical Routines linear equation solution routine
C
C************************ FFORM VERSION 1.0 ************  1-MAR-88 15:03
C
CA  author : Press et al.      date:
CU  update : J. Bloch          date:  6-SEP-1987
CU  update : SLS               date:  1-MAR-1988 14:42
C
CT  status: not tested
C
C   general description
CG  Linear equation solution by Gauss-Jordan elimination.  A is an
CG  input matrix of N x N elements, stored in an array of physical
CG  dimensions NP x NP.  B is an input matrix of N x M containing the
CG  M right-hand side vectors, stored in an array of physical
CG  dimensions NP x MP.  On output, A is replaced by its matrix
CG  inverse, and B is replaced by the corresponding set of solution
CG  vectors.
CG  Modified by J. Bloch to be a double precision matrix version of the
CG  subroutine GAUSSJ in the book NUMERICAL RECIPES by Press et al.
C
C   call_var.          type I/O description
CP  A                   R8  I/O Updated REAL*8 Matrix which will be
CP                              replaced by it's inverse.
CP  N                   I4  I   actual order of matrix A
CP  NP                  I4      actual dimenstions of array A.
CP  B                   R8  I/O Updated array containing M right hand
CP                                  side vectors. Upon return, B
CP                                  contains the solution vectors.
CP  M                   I4  I   number of right hand side vectors.
CP  MP                  I4  I   actual second dimension size of B.
CP  IERR                I4  I/O = 0 no errors
CP                              = 3 singular matrix
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
C       PARAMETER NMAX=200
C
        INTEGER I, ICOL, IERR, IROW, J, K, L, LL, M, MP, N, NP
        INTEGER IPIV(200), INDXR(200), INDXC(200)
C
        DOUBLE PRECISION A(NP,NP), B(NP,MP), BIG, DUM, PIVINV
C
        IERR = 0
        DO 11 J=1,N
            IPIV(J) = 0
   11   ENDDO
C
C  Start the main loop over the columns to be reduced
C
        I = 0
        DO WHILE ((I .LT. N) .AND. (IERR .EQ. 0))
            I = I + 1
            BIG = 0.D0
C
C  Start the outer loop of the search for the pivot element
C
            J = 0
            DO WHILE ((J .LT. N) .AND. (IERR .EQ. 0))
                J = J + 1
                IF(IPIV(J) .NE. 1) THEN
                    K = 0
                    DO WHILE ((K .LT. N) .AND. (IERR .EQ. 0))
                        K = K + 1
                        IF (IPIV(K) .EQ. 0) THEN
                            IF (ABS(A(J,K)) .GE. BIG) THEN
                                BIG = DABS(A(J,K))
                                IROW = J
                                ICOL = K
                            ENDIF
                        ELSE IF (IPIV(K) .GT. 1) THEN
                            IERR = 3
                            PRINT *, 'DGAUSJ, singular matrix, pos 1'
                        ENDIF
                    ENDDO
                ENDIF
            ENDDO
            IF(IERR .EQ. 0) THEN
                IPIV(ICOL) = IPIV(ICOL) + 1
C
C  We now have the pivot element, so we interchange rows, if needed,
C  to put the pivot element on the diagonal.  The columns are not
C  physically interchanged, only relabeled: INDX(I), the column of the
C  Ith pivot element, is the Ith column that is reduced, while INDXR(I)
C  is the row in which that pivot element was origially located.  If
C  INDXR(I) =/ INDXC(I) there is an implied column interchange.  With
C  this form of bookkeeping, the solution B's will end up in the correct
C  order, and the inverse matrix will be scrambled by columns.
C
                IF (IROW .NE. ICOL) THEN
                    DO 14 L=1,N
                        DUM = A(IROW,L)
                        A(IROW,L) = A(ICOL,L)
                        A(ICOL,L) = DUM
   14               ENDDO
C
                    DO 15 L=1,M
                        DUM = B(IROW,L)
                        B(IROW,L) = B(ICOL,L)
                        B(ICOL,L) = DUM
   15               ENDDO
                ENDIF
C
C  We are now ready to divide the pivot row by the pivot element
C  located at IROW and ICOL
C
                INDXR(I) = IROW
                INDXC(I) = ICOL
                IF (A(ICOL,ICOL) .EQ. 0.) THEN
                    IERR = 3
                    PRINT *, 'DGAUSJ, singular matrix, pos 2'
                ELSE
                    PIVINV = 1.D0/A(ICOL,ICOL)
                    A(ICOL,ICOL) = 1.D0
                    DO 16 L=1,N
                        A(ICOL,L) = A(ICOL,L)*PIVINV
   16               ENDDO
C
                    DO 17 L=1,M
                        B(ICOL,L) = B(ICOL,L)*PIVINV
   17               ENDDO
C
C  Next we reduce the rows, except for the pivot row
C
                    DO 21 LL=1,N
                        IF(LL .NE. ICOL)THEN
                            DUM = A(LL,ICOL)
                            A(LL,ICOL) = 0.
                            DO 18 L=1,N
                                A(LL,L) = A(LL,L) - A(ICOL,L)*DUM
   18                       ENDDO
                            DO 19 L=1,M
                                B(LL,L) = B(LL,L) - B(ICOL,L)*DUM
   19                       ENDDO
                        ENDIF
   21               ENDDO
                ENDIF
            ENDIF
   22   ENDDO
C
C  This is the end of the main loop over the columns of the reduction.
C  Now unscramble the solution in view of the column interchanges.
C  We do this by interchanging pairs of columns in the order that the
C  permutation was built up.
C
        IF(IERR .EQ. 0) THEN
            DO 24 L=N,1,-1
                IF(INDXR(L).NE.INDXC(L))THEN
                    DO 23 K=1,N
                        DUM = A(K,INDXR(L))
                        A(K,INDXR(L)) = A(K,INDXC(L))
                        A(K,INDXC(L)) = DUM
   23               ENDDO
                ENDIF
   24       ENDDO
        ENDIF
C
        RETURN
        END
c
c
c
        double precision function DRADRS(DRAD)
C
CC    'Resets' radians to the interval  0 .. 2 * pi
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:58
C
CA  author : KOD        date: 19-JUL-1989 18:31
CU  update : KOD        date: 29-MAR-1990 15:08
C
CT  status: tested
C
C   general description
CG  'Resets' radians to the interval  0  .. 2 * pi.
C
C   call_var.          type I/O description
CP  DRAD                R8  I   radians (may be negative)
CP  DRADRS              R8      radians inside the interval  0 .. 2 * pi
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
CI  IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  RESVAL              SR      'resets' periodic variables
CR  ERRSTC              SR      continue error stack for one level
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
      INCLUDE 'igenl.inc'
C
      DOUBLE PRECISION  DRAD, DXMIN, DXMAX, DXOLD, DXNEW
      INTEGER     IERR, ISTAT
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'DRADRS'/
C
      IERR = 0
C
      DXMIN = 0.D0
      DXMAX = D2PI
      DXOLD = DRAD
C
      CALL RESVAL(DXMIN,DXMAX,DXOLD,DXNEW,ISTAT)
C
      if (ISTAT.gt.1) then
        IERR = max(IERR,ISTAT)
        call ERRSTC(RTNAME,10)
      endif
C
      if (IERR.lt.2) DRADRS = DXNEW
C
      RETURN
      END
c
c
c
        SUBROUTINE ELAZ8 (V, EL, AZ)
C
CC  ELAZ returns the elevation and azimuth of a given vector
C
C************************ FFORM VERSION 1.0 ************ 17-MAR-89 10:05
C
CA  author :                   date:
CU  update : SLS               date: 17-MAR-1989 10:06
C
CT  status: not tested
C
C   general description
CG  ELAZ returns the elevation and azimuth of a given vector
C
C   call_var.          type I/O description
CP  V                   R4  I   input direction vector (dim 3)
CP  EL                  R4    O elevation angle of vector v in decimal
CP                              degrees
CP  AZ                  R4    O azimuthal angle of vector v in decimal
CP                              degrees
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        DOUBLE PRECISION AZ, CCC, EL, RD, V(3)
C
        DATA RD /57.29577951D0/
C
        EL = RD*DASIN(V(3)/DSQRT(V(1)**2 + V(2)**2 + V(3)**2))
        CCC = V(1)**2 + V(2)**2
        IF (CCC .EQ. 0.) THEN
            AZ = 0.
        ELSE
            AZ = DMOD(DSIGN(DACOS(V(1)/DSQRT(V(1)**2+V(2)**2))*RD,
     +          V(2))+720.D0,360.D0)
        ENDIF
C
        RETURN
        END
c
c
c
      subroutine EQUECL(DJD,LMEAN,LDIR,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
C
CC    Converts coordinates between equatorial and ecliptical system
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
C
CA  author : KOD        date: 19-JUL-1989 18:31
CU  update : KOD        date: 29-MAR-1990 15:13
C
CT  status: tested
C
C   general description
CG  Subroutine to convert coordinates between the equatorial and the
CG  ecliptical system (LII,BII).
CG
CG  To convert the TRUE positions, use the MEAN obliquity of the
CG  ecliptic (normal case);
CG  for the APPARENT positions (affected by aberration and nutation)
CG  use the TRUE obliquity.
CG  If the equatorial coordinates refer to a standard equinox
CG  (1950, 2000, ...), the obliquity of the ecliptic for this
CG  epoch should be used.
CG
CG  (Jean Meeus, 'Astronomical Formulae for Calculators',
CG                third edition, 1985, p.43-44)
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Day for the conversion
CP
CP  LMEAN               L4  I   mean or true obliquity of the ecliptic:
CP                                LMEAN = .true.  : mean obliquity
CP                                LMEAN = .false. : true obliquity
CP
CP  LDIR                L4  I   transformation direction:
CP                                LDIR = .true.  : equ --> ecl
CP                                LDIR = .false. : equ <-- ecl
CP
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
CI  IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  EULER1              SR      general coordinate transformation
CR  JDEPS1              SR      calculates mean/true obl. of the ecl.
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
      INCLUDE 'igenl.inc'
C
      DOUBLE PRECISION DJD, DLOLD, DBOLD, DLNEW, DBNEW, D1, D2, D3,
     +              DEPS, DPHI1, DTHETA1, DPHI2, DTHETA2, DJ
      INTEGER     IERR, ISTAT
      LOGICAL     LMEAN, LDIR, LMN
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
      DATA          RTNAME /'EQUECL'/
C
      IERR = 0
C
C
      if (DLOLD.lt.0.D0 .or. DLOLD.GT.D2PI) then
        IERR = 1
        LOG = 'Warning: unusual value for DLOLD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (dabs(DBOLD).gt.DPID2) then
        IERR = max(IERR,2)
        LOG = 'Error in input parameter DBOLD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = max(IERR,2)
        LOG = 'Error in the input parameter DJD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
C
      if  (IERR.lt.2)  then
C
        DJ = DJD
        LMN = LMEAN
        call JDEPS1(DJ,LMN,DEPS,ISTAT)
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
C
      if  (IERR.lt.2)  then
C
        D1 = 0.D0
        D2 = DEPS
        D3 = 0.D0
C
        DPHI1   = DLOLD
        DTHETA1 = DPID2 - DBOLD
C
        if (LDIR) then
          call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        else
          call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        endif
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        else
          DLNEW = DPHI2
          DBNEW = DPID2 - DTHETA2
        endif
C
      endif
C
      RETURN
      END

      subroutine EQUGAL(DJD,LDIR,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
C
CC    Converts coordinates between equatorial and new galactic system
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
C
CA  author : KOD        date: 19-JUL-1989 19:45
CU  update : KOD        date: 20-MAR-1990 14:56
C
CT  status: tested
C
C   general description
CG  Subroutine to convert coordinates between the equatorial and the
CG  new galactic system (LII,BII).
CG  Parameters extracted from:
CG     F.C.Bertiau, 'Programmes for pocket calculators HP-67
CG                   and HP-97 in the field of theoretical and
CG                   observational astronomy',
CG                   1977 Leuven University Press, 'ASTR. COOR. 2'
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Day for the conversion
CP
CP  LDIR                L4  I   transformation direction:
CP                                LDIR = .true.  : equ --> gal II
CP                                LDIR = .false. : equ <-- gal II
CP
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
CI  IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  EULER1              SR      general coordinate transformation
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
      INCLUDE 'igenl.inc'
C
      DOUBLE PRECISION DJD, DLOLD, DBOLD, DLNEW, DBNEW, D11, D21, D31,
     &              D12, D22, D32, D1, D2, D3, DJD50, DPHI1, DTHETA1,
     &              DPHI2, DTHETA2, DT
      INTEGER     IERR, ISTAT
      LOGICAL     LDIR
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
C     julian day for 1950.01.01 / 00:00:00
      PARAMETER ( DJD50 = 2433282.5D0 )
C
      PARAMETER ( D11 = 4.92619181375D0 )
      PARAMETER ( D21 = 1.09257611174D0 )
      PARAMETER ( D31 = 5.70722665400D0 )
C
      PARAMETER ( D12 = 5.82362268000D-7 )
      PARAMETER ( D22 = 2.59857718339D-7 )
      PARAMETER ( D32 = 0.64223753852D-7 )
C
      DATA          RTNAME /'EQUGAL'/
C
      IERR = 0
C
      if (DLOLD.lt.0.D0 .or. DLOLD.GT.D2PI) then
        IERR = 1
        LOG = 'Warning: unusual value for DLOLD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (dabs(DBOLD).gt.DPID2) then
        IERR = max(IERR,2)
        LOG = 'Error in input parameter DBOLD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (IERR.lt.2) then
C
        DT = DJD - DJD50
C
        ISTAT = 0
        if (dabs(DT).gt.5.D4) ISTAT = 1
        if (dabs(DT).gt.5.D6) ISTAT = 2
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          LOG = 'Error in input parameter DJD'
          call WRERR(RTNAME,IERR,LOG)
        endif
C
      endif
C
      if  (IERR.lt.2)  then
C
        D1 = D11 + D12 * DT
        D2 = D21 + D22 * DT
        D3 = D31 + D32 * DT
C
        DPHI1   = DLOLD
        DTHETA1 = DPID2 - DBOLD
C
        if (LDIR) then
          call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        else
          call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        endif
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        else
          DLNEW = DPHI2
          DBNEW = DPID2 - DTHETA2
        endif
C
      endif
C
      RETURN
      END
c
c
c
        SUBROUTINE ERRSTC(RTNAME,NUM)
C
        INTEGER NUM
        CHARACTER*8 RTNAME
C
        Print *, 'Error in Routine: ', rtname
        Print *, 'Error Number: ', num
C
        RETURN
        END
c
c
c
      subroutine EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,IERR)
C
CC    General transformation of spherical coordinates
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:03
C
CA  author : KOD        date: 15-JUL-1989 18:31
CU  update : KOD        date: 20-MAR-1990 14:57
C
CT  status: tested
C
C   general description
CG
CG  Subroutine to convert spherical coordinates between coordinate
CG  systems which are rotated with respect to each other.
CG
CG  This subroutine converts the azimuthal and polar angles
CG  (DPH1,DTH1), given in mathematical notation and in radians,
CG  into the corresponding angles with respect to a coordinate
CG  system which orientation is given by the EULER angles (D1,D2,D3)
CG  in the following way:
CG
CG  The original cartesian coordinate system (X0,Y0,Z0) is rotated
CG  in three steps:
CG
CGT   1) rotation around the Z0-axis by the angle D1  --->  (X1,Y1,Z1)
CGT   2) rotation around the X1-axis by the angle D2  --->  (X2,Y2,Z2)
CGT   3) rotation around the Z2-axis by the angle D3  --->  (X3,Y3,Z3)
CG
CG  The calculation is accelerated by using differences of angles.
C
C   call_var.          type I/O description
CP  D1                  R8  I   EULER angle 1   (radians, -2 pi .. 2 pi)
CP  D2                  R8  I   EULER angle 2   (radians, -2 pi .. 2 pi)
CP  D3                  R8  I   EULER angle 3   (radians, -2 pi .. 2 pi)
CP
CP  DPH1                R8  I   original  azimuthal angle (rad, 0..2 pi)
CP  DTH1                R8  I   original    polar   angle (rad, 0..pi)
CP
CP  DPH2                R8    O resulting azimuthal angle (rad, 0..2 pi)
CP  DTH2                R8    O resulting   polar   angle (rad, 0..pi)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
CI  IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  REPOL1              SR      cartesian to polar coordinate conversion
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
      INCLUDE 'igenl.inc'
C
      DOUBLE PRECISION  D1, D2, D3, DPH1, DPH2, DTH1, DTH2,
     &              DST, DCT, DH, DS1, DC1, DS2, DC2, DX, DY, DZ,
     &              DR, DPHI, DTHETA, DRADRS
      INTEGER     IERR
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
      DATA          RTNAME /'EULER1'/
C
      IERR = 0
C
      if (max(dabs(D1),dabs(D2),dabs(D3)).gt.D2PI .or.
     &    DPH1.lt.0.D0) then
        IERR = max(IERR,1)
        LOG = 'Warning: Unusual input parameters'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (DTH1.lt.0.D0 .or. DTH1.gt.DPI) then
        IERR = max(IERR,2)
        LOG = 'Error in input parameters'
        call WRERR(RTNAME,IERR,LOG)
      else
C
        DST = sin(DTH1)
        DCT = cos(DTH1)
C
        DH  = DPH1 - D1
        DS1 = sin(DH)
        DC1 = cos(DH)
C
        DS2 = sin(D2)
        DC2 = cos(D2)
C
        DX = DST*DC1
        DY = DST*DS1*DC2 + DCT*DS2
        DZ =-DST*DS1*DS2 + DCT*DC2
C
        call REPOL1(DX,DY,DZ,DR,DPHI,DTHETA)
C
        if (DABS(DR-1.D0).gt.1.E-6) then
          IERR = max(IERR,2)
        else
          DPH2 = DRADRS(DPHI - D3)
          DTH2 = DTHETA
        endif
C
      endif
C
      RETURN
      END
c
c
c
        SUBROUTINE EULROT8(R,R1,E1,E2,E3,MTEST)
C
CC  Temp
C
C************************ FFORM VERSION 1.0 ************ 17-MAR-89 11:17
C
CU  update : Steve Holz        date:  1-APR-1982
CU  update : SLS               date: 17-MAR-1989 11:16
CU  update : SLS               date: 17-MAR-1989 11:17
C
CT  status: not tested
C
C   general description
CG  source: Goldstein, p. 107.
CG  EULROT uses the euler angles E1, E2, and E3 to rotate the vector R
CG  into the vector R1 or vica versa, depending on the value of MTEST.
CG  <extended_routine_description>
C
C   call_var.          type I/O description
CP  R1                  R4    O vector in primed frame, R1 = A R
CP  R                   R4  I/O vector in unprimed frame, R = A1 R1
CP  E1                  R4  I   Euler angle PHI, R' frame wrt R frame
CP  E2                  R4  I   Euler angle THETA, R' frame wrt R frame
CP  E3                  R4  I   Euler angle PSI, R' frame wrt R frame
CP  MTEST               I4      > 0 vector R is rotated into the primed
CP                              frame and the result stored in R1
CP                              <= 0 vector R1 is rotated from the
CP                              primed frame into the unprimed frame and
CP                              the result stored in R.
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        INTEGER I, J, MTEST
C
        DOUBLE PRECISION A(3,3), A1(3,3), E1, E2, E3, R(3), R1(3)
C
        A(1,1) = DCOS(E3)*DCOS(E1) - DCOS(E2)*DSIN(E1)*DSIN(E3)
        A(1,2) = DCOS(E3)*DSIN(E1) + DCOS(E2)*DCOS(E1)*DSIN(E3)
        A(1,3) = DSIN(E3)*DSIN(E2)
C
        A(2,1) = -DSIN(E3)*DCOS(E1) - DCOS(E2)*DSIN(E1)*DCOS(E3)
        A(2,2) = -DSIN(E3)*DSIN(E1) + DCOS(E2)*DCOS(E1)*DCOS(E3)
        A(2,3) = DCOS(E3)*DSIN(E2)
C
        A(3,1) = DSIN(E2)*DSIN(E1)
        A(3,2) = -DSIN(E2)*DCOS(E1)
        A(3,3) = DCOS(E2)
C
        IF(MTEST .GT. 0) THEN
C
C  Rotate into the primed system
C
            DO I=1,3
                R1(I) = 0.0
                DO J=1,3
                    R1(I) = R1(I) + A(I,J)*R(J)
                ENDDO
            ENDDO
        ELSE
C
C  Rotate into the unprimed system
C
            DO I=1,3
                DO J=1,3
                    A1(I,J) = A(J,I)
                ENDDO
            ENDDO
C
            DO I=1,3
                R(I) = 0.0
                DO J=1,3
                    R(I) = A1(I,J)*R1(J) + R(I)
                ENDDO
            ENDDO
        ENDIF
C
        RETURN
        END
c
c
c
        subroutine gauss_fit(tempx,tempy,tempu,n,np,ap,iflag)
c
        implicit none
c
        integer ftype, i, ierr, iflag, iup, iuplim, 
     +      lista(10), maxitr, np, nppo, n
        real a(10), alamda, almlim, ap(10), chisq, deltch,
     +      i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, 
     +      ochisq, olamda, tempx(n), tempy(n), tempu(n)
        double precision alpha(10,10), covar(10,10)
        character*1 fix(10)
        character*15 pshort(10)
c
        common /modpar/ i0, i1, i2, i3, i4, i5, i6, i7, i8, i9
c
        equivalence(a(1),i0)
c
        data fix/10*' '/
        data ftype /2/
        data pshort /'NORM', 'WIDTH', 'OFFSET', ' ', ' ', ' ', ' ',
     +      ' ', ' ', ' '/
c
        ierr = 0
c
        if(np .ne. 3) then
            print *, 'Error detected in: GAUSS_FIT'
            print *, 'Wrong number of parameters for a Gaussian fit'
            stop
        endif
c
        do i=1,np
            lista(i) = i
            fix(i) = '*'
            a(i) = ap(i)
        enddo
        nppo = np
c
c  *********************************************************************
c  *                                                                   *
c  *                       Main Fit Begins Here                        *
c  *                                                                   *
c  *********************************************************************
c
        maxitr = 100
        almlim = 1.E3
        iuplim = 10
c
c        call report(-1,' Initial Model Value Settings:',a,fix)
c
        alamda = -1.0
        ochisq = 0.0
        iup = 0
        i = 1
        iflag = 0
c
        do while ((iflag .eq. 0) .and. (i .lt. maxitr))
            i = i + 1
            olamda = alamda
            call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo,
     +          covar,alpha,10,chisq,alamda,ierr)
            if(ierr .eq. 3) then
                print *, 'Bad return from MQM'
                print *, 'Program stopped in GAUSS_FIT'
                print *, 'Position 01'
                stop
            endif
c
            if(ierr .eq. 3) then
                iflag = 3
            else
                deltch = abs(ochisq - chisq)
c
c  Check for various termination conditions
c
                if((alamda .lt. olamda) .and. (deltch .lt. 0.1))
     +                  iflag = 1
                if(iflag .eq. 0) then
                    if(alamda .gt. olamda) iup = iup + 1
                    if(alamda .lt. olamda) iup = 0
                    if(iup .gt. iuplim) iflag = 1
                    if(alamda .gt. almlim) iup = iup + 1
c
c                    write(*,*)
c                    write(*,*) 'ITER =',i,'  ALAMDA =',alamda,
c     +                  '  M.L.E. =',chisq
c                    call report(-1,'Intermediate step..',a,fix)
                endif
            endif
        enddo
c
c
c
        if(iflag .ne. 3) then
c
c            write(*,*)
c            call report(-1,' Final Model Values: (*=Floating)',a,fix)
c
            write(*,1010) ' Np =',np,' N =',n,' M.L.E. =',chisq
 1010       format(/,1x,a,i2,a,i4,a,g12.4,a,g14.7)
c
c  Finish off the fit to get the covarience matrix
c
            alamda = 0.0
            call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo,
     +          covar,alpha,10,chisq,alamda,ierr)
            if(ierr .eq. 3) then
                print *, 'Bad return from MQM'
                print *, 'Program stopped in GAUSS_FIT'
                print *, 'Position 02'
                stop
            endif
c
c  Write out final values and one sigma uncertainties
c
            print *, 'Final parameter values and the formal ',
     +                      'standard errors'
            print *, ' '
c
            do i=1,nppo
                ap(i) = a(i)
                write(6,2100) pshort(lista(i)),a(lista(i)),
     +                  sngl(dsqrt(dabs(covar(lista(i),lista(i)))))
 2100           format(1x,a15,f13.5,' +/-',f11.5)
            enddo
            iflag = 0
        endif
c
        return
        end
c
c
c
        SUBROUTINE GFUNCS (TEMPX,ALOC,YMODEL,DYDA,DY2DA2,
     +      LISTA,MFIT,MA)
C
CC  Calculates Gaussian function distribution and partial derivatives
C
C************************ FFORM VERSION 1.0 ************ 21-APR-88 08:24
C
CA  author : SLS               date: 17-August-2005 15:00
C
CT  status: not tested
C
C   general description
CG  PFUNCS is the model value subroutine for gauss_fit for use with the
CG  fitting subroutine MQM.FOR.
C
C   call_var.          type I/O description
CP  TEMPX               R4  I/? X value.
CP  ALOC                R4  I/? array of model values.
CP                              A(1) = normalization
CP                              A(2) = width
CP                              A(3) = peak point
CP  YMODEL              R4      value of the function.
CP  DYDA                R4  I/O array of first partial derivatives
CP                              w.r.t. each parameter. (Only the
CP                              parameters actually being fit are
CP                              computed.)
CP  DY2DA2              R4  I/O array of second partial derivatives
CP                              w.r.t. the parameters.  (Only those
CP                              parameters being fit will be computed.)
CP  LISTA               I4  I   list of parameters being fitted.
CP  MFIT                I4  I   number of parameters being fit.
CP  MA                  I4  I   number of parameters in model.
CP                              (Should be <= 10)
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
C
C***********************************************************************
C
        IMPLICIT NONE
C
        SAVE
C
        INTEGER I, MFIT, LISTA(MFIT), MA
C
        REAL ALOC(MA), DY2DA2(10,10), DYDA(10), TEMPX, YMODEL
C
        DOUBLE PRECISION A(10), CHAN, G, YMOD
C
        DO I=1,10
            A(I) = DBLE(ALOC(I))
        ENDDO
        CHAN = DBLE(TEMPX)
C
C  Compute the model value
C
        G   = exp(-A(2)*(CHAN - A(3))*(CHAN - A(3)))
        YMOD = A(1)*G
        YMODEL = SNGL(YMOD)
C
C  Compute values on intervals to prepare for computation
C  of numerical derivatives.
C
        DYDA(1) = G
        DYDA(2) = -A(1)*(CHAN - A(3))*(CHAN - A(3))*G
        DYDA(3) = 2.0*A(1)*A(2)*(CHAN - A(3))*G
C
        DY2DA2(1,1) = 0.0
        DY2DA2(1,2) = -(CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(1,3) = 2.0*A(2)*(CHAN - A(3))*G
        DY2DA2(2,1) = -(CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(2,2) = A(1)*(CHAN - A(3))*(CHAN - A(3))*
     +          (CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(2,3) = 2.0*A(1)*(CHAN - A(3))*
     +          (1.0 -  A(2)*(CHAN - A(3))*(CHAN - A(3)))*G
        DY2DA2(3,1) = 2.0*A(2)*(CHAN - A(3))*G
        DY2DA2(3,2) = 2.0*A(1)*(CHAN - A(3))*
     +          (1.0 -  A(2)*(CHAN - A(3))*(CHAN - A(3)))*G
        DY2DA2(3,3) = 2.0*A(1)*A(2)*
     +          (2.0*A(2)*(CHAN - A(3))*(CHAN - A(3)) - 1.0)*G
C
        RETURN
        END
c
c
c
        subroutine hard_scale (xmmdir,alpha,idet,
     +          filt,islow,ishigh,fscal)
c
        implicit none
c
        integer bitpix, block, err, gcount, 
     +      idet, iflag, i, ii, iii, islow, ishigh, 
     +      lun, naxes(3), naxis, pcount, status
        real alpha, fscal, 
     +      fscarr(240,240,9), fscarrh(240,240,9), fscarrm(240,240,9), 
     +      fscarrs(240,240,9), ts, th
        character*1 db_fi(80), sp_fi(25)
        character*4 filt
        character*25 sp_file, sp_files, sp_filem, sp_fileh
        character*80 db_file, xmmdir
        logical anyf, extend, simple
c
        equivalence (db_file,db_fi)
        equivalence (sp_file,sp_fi)
c
        data err /0/
        data status /0/
        data iflag /0/
c
        if(iflag .eq. 0) then
            iflag = 1
c
c  Get an unused lun
c
            call ftgiou(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 1'
                print *, 'STATUS FTGIOU =', status
                stop
            endif
c
c  Set the name
c
            db_file = xmmdir
            sp_files = '/scale-factors-s.fits.gz'
            sp_filem = '/scale-factors-m.fits.gz'
            sp_fileh = '/scale-factors-h.fits.gz'
c
c  Read in the soft data
c
            sp_file = sp_files
            ii = 1
            do while (db_fi(ii) .ne. ' ')
                ii = ii + 1
            enddo
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo
c
c  Open the FITS file for the scale factors
c
            call ftopen(lun,db_file,0,block,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 2'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif
c
c  Read the FITS file header information
c
            call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +                      pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 3'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif
c
c  Read in the file
c
            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1),
     +              naxes(2),naxes(3),fscarrs,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 4'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif
c
c  Close the FITS file
c
            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 5'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif
c
c  Read in the medium spectrum scale factor array
c
            sp_file = sp_filem
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo
c
c  Open the FITS file for the scale factors
c
            call ftopen(lun,db_file,0,block,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 6'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif
c
c  Read the FITS file header information
c
            call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +                      pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 7'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif
c
c  Read in the file
c
            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1),naxes(2),
     +              naxes(3),fscarrm,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 8'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif
c
c  Close the FITS file
c
            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 9'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif
c
c  Read in the medium spectrum scale factor array
c
            sp_file = sp_fileh
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo
c
c  Open the FITS file for the scale factors
c
            call ftopen(lun,db_file,0,block,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 10'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif
c
c  Read the FITS file header information
c
            call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +                      pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 11'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif
c
c  Read in the file
c
            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1),naxes(2),
     +              naxes(3),fscarrh,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 12'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif
c
c  Close the FITS file
c
            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 13'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif
c
c  Clean up
c
            call ftfiou (lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 14'
                print *, 'STATUS FTFIOU =', status
                print *, db_file
            endif
c
            if((alpha .lt. 1.0) .or. (alpha .gt. 2.4)) then
                print *, 'HARDSCALE Error'
                print *, 'Selected value for alpha out of range'
                stop
            elseif(alpha .eq. 1.0) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrh(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .eq. 1.7) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrm(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .eq. 2.4) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrs(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .lt. 1.7) then
                ts = (1.7 - alpha)/0.7
                th = (alpha - 1.0)/0.7
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = 
     +                              ts*fscarrm(i,ii,iii) + 
     +                              th*fscarrh(i,ii,iii)
                        enddo
                    enddo
                enddo
            else
                ts = 2.4 - alpha
                th = alpha - 1.7
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = 
     +                              ts*fscarrs(i,ii,iii) + 
     +                              th*fscarrm(i,ii,iii)
                        enddo
                    enddo
                enddo
            endif
        endif
c
c  Now evaluate the scale factor
c
        if(filt .eq. 'Thin') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,1)
            elseif(idet .eq. 2) then
                fscal = fscarr(islow,ishigh,4)
            else
                fscal = fscarr(islow,ishigh,7)
            endif
        elseif(filt .eq. 'Medi') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,2)
            elseif(idet .eq. 2) then
                fscal = 1.0
            else
                fscal = fscarr(islow,ishigh,8)
            endif
        elseif(filt .eq. 'Thic') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,3)
            elseif(idet .eq. 2) then
                fscal = fscarr(islow,ishigh,6)
            else
                fscal = fscarr(islow,ishigh,9)
            endif
        endif
c
        return
        end
c
c
c
      subroutine JDEPS1(DJD,LMEAN,DEPS,IERR)
C
CC    Calculates the MEAN/TRUE obliquity of the ecl. for the Jul. Date
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:05
C
CA  author : KOD        date: 29-AUG-1989 17:22
CU  update : KOD        date: 29-MAR-1990 15:53
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the MEAN/TRUE obliquity of the ecliptic
CG  for a given Julian Date, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.44,69,70,81.
CG
CG  The parameters for calculating the mean obliquity have been
CG  taken from 'THE ASTRONOMICAL ALMANACH', 1984, S21.
CG
CG  For APPARENT positions (affected by aberration and nutation)
CG  use the TRUE obliquity, otherwise the MEAN obliquity.
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date
CP
CP  LMEAN               L4  I   mean or true obliquity of the ecliptic:
CP                                LMEAN = .true.  : mean obliquity
CP                                LMEAN = .false. : true obliquity
CP
CP  DEPS                R8    O obliquity of the ecliptic  (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  JDNUT1              SR      calculates the nutation parameters
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
C
      DOUBLE PRECISION DJD, DEPS, DJD0, DYEAR, D1, D2, D3, D4, DT, DP,
     &              DE, DJ
      INTEGER     IERR, ISTAT
      LOGICAL     LMEAN
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
      PARAMETER ( DJD0 = 2451545.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      PARAMETER ( D1 =  0.40909280422D0 )
      PARAMETER ( D2 = -2.26965525D-4 )
      PARAMETER ( D3 = -2.86040D-9 )
      PARAMETER ( D4 =  8.78967D-9 )
C
      DATA          RTNAME /'JDEPS1'/
C
      IERR = 0
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = 2
        LOG = 'Error in the input parameter DJD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (IERR.lt.2) then
C
        DT = (DJD - DJD0) / DYEAR
C
        DEPS = D1 + ( D2 + ( D3 + D4 * DT ) * DT ) * DT
C
        if (.not.LMEAN) then
C
          DJ = DJD
          call JDNUT1(DJ,DP,DE,ISTAT)
C
          if  (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,10)
          else
            DEPS = DEPS + DE
          endif
C
        endif
C
      endif
C
      RETURN
      END

      subroutine JDNUT1(DJD,DP,DE,IERR)
C
CC    Calculates the nutation in longitude and in obliquity from the JD
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:06
C
CA  author : KOD        date: 29.08L-1989 14:45
CU  update : KOD        date: 20-MAR-1990 15:04
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the nutation in longitude and the
CG  nutation in obliquity which are needed for the calculation
CG  of the APPARENT place of a star and for that of the APPARENT
CG  sidereal time, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.69-70 .
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date
CP  DP                  R8    O nutation in longitude (radians)
CP  DE                  R8    O nutation in obliquity (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
C
      DOUBLE PRECISION DJD, DP, DE, DJD0, DYEAR, DT, DLS, DLM,
     &              DMS, DMM, DOM, DH, DLS2, DLM2, DOM2, DF
      INTEGER     IERR
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
      PARAMETER ( DJD0 = 2415020.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      PARAMETER ( DF = 4.848136811D-6 )
C
      DATA          RTNAME /'JDNUT1'/
C
      IERR = 0
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = 2
        LOG = 'Error in the input parameter DJD'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (IERR.lt.2) then
C
        DT = (DJD - DJD0) / DYEAR
C
        DLS = 4.8816283D0 + (628.3319506D0 + 5.2883D-6 * DT) * DT
        DLM = 4.7199672D0 + (8399.709144D0 - 1.9775D-5 * DT) * DT
        DMS = 6.2565830D0 + (628.3019466D0 - 2.6180D-6 * DT) * DT
        DMM = 5.1680002D0 + (8328.691104D0 + 1.6043D-4 * DT) * DT
        DOM = 4.5236020D0 - (33.75714610D0 + 3.6268D-5 * DT) * DT
C
        DLS2 = 2.D0 * DLS
        DLM2 = 2.D0 * DLM
        DOM2 = 2.D0 * DOM
C
        DH = - (17.2327D0 + 0.01737D0 * DT ) * sin(DOM)
     &       - ( 1.2729D0 + 0.00013D0 * DT ) * sin(DLS2)
     &       + 0.2088D0 * sin(DOM2) - 0.2037D0 * sin(DLM2)
     &       + ( 0.1261D0 - 0.00031D0 * DT) * sin(DMS)
     &       + 0.0675 * sin(DMM)
     &       - (0.0497D0 - 0.00012D0 * DT) * sin(DLS2+DMS)
     &       - 0.0342D0 * sin(DLM2-DOM)
     &       - 0.0261D0 * sin(DLM2+DMM)
     &       + 0.0214D0 * sin(DLS2-DMS)
     &       - 0.0149D0 * sin(DLS2-DLM2+DMM)
     &       + 0.0124D0 * sin(DLS2-DOM)
     &       + 0.0114D0 * sin(DLM2-DMM)
C
        DP = DH * DF
C
        DH =   ( 9.2100D0 + 0.00091D0 * DT ) * cos(DOM)
     &       + ( 0.5522D0 - 0.00029D0 * DT ) * cos(DLS2)
     &       - 0.0904D0 * cos(DOM2)
     &       + 0.0884D0 * cos(DLM2)
     &       + 0.0216D0 * cos(DLS2+DMS)
     &       + 0.0183D0 * cos(DLM2-DOM)
     &       + 0.0113D0 * cos(DLM2+DMM)
     &       - 0.0093D0 * cos(DLS2-DMS)
     &       - 0.0066D0 * cos(DLS2-DOM)
C
        DE = DH * DF
C
      endif
C
      RETURN
      END
c
        SUBROUTINE MQM (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA,
     +      LISTA,MFIT,COVAR,ALPHA,NCA,CHISQ,ALAMDA,IERR)
C
CC  Maximum likelyhood fitting routine
C
C************************ FFORM VERSION 1.0 ************  2-MAR-88 09:16
C
CA  author : J. Bloch          date:  4-SEP-1987
CU  update : J. Bloch          date:  6-SEP-1987
CU  update : SLS               date:  2-MAR-1988 09:25
C
CT  status: not tested
C
C   general description
CG  MQMLMN was derived from MRQMIN in the book "Numerical Recipes" by
CG  Press et al. It finds the best parameter fit using the marquardt
CG  method applied to the Poisson Maximum Likelihood Estimator, as
CG  described by Cash in the 1979 March 15 Astrophysical Journal. The
CG  SIG array has been removed because it is not needed.
C
C   call_var.          type I/O description
CP  FTYPE               I4      = 1 Polynomial function (PFUNCS)
CP                              = 2 Gaussian function (GFUNCS)
CP  IERR                I4  I/O = 0 no error
CP                              = 2 Bad return from DGAUSJ
CP                              = 3
CP  TEMPX               R4  ?/? independent variable data values.
CP  TEMPY               R4  ?/? dependent variable data values.
CP  TEMPU               R4  ?/? dependent variable uncertainty values.
CP  NDATA               I4      number of data points to fit.
CP  A                   R4  I/O model parameter values.
CP  MA                  I4  I/? number of model parameter values.
CP  LISTA               I4  I/O list of parameters
CP                                  in array A actually being varied.
CP  MFIT                I4  I/? number of parameters actually being fit.
CP  COVAR               R8  I/O covariance matrix and work
CP                                  array used durint the fit.
CP  ALPHA               R8  I/O matrix ALPHA used in the Marquard method
CP                                  (See Press et al.,pp. 521-528)
CP  NCA                 I4  I/? Actual dimensions of ALPHA and COVAR.
CP  CHISQ               R4  I/O Output Maximum Likelihood Estimator.
CP  ALAMDA              R4  I/O Updated REAL iteration control variable.
CP                              Must be set to  < 0.0 on first call
CP                              of MQMLMN. If a step fails, ALMADA grows
CP                              by a factor of 10. On final call, set
CP                              ALMADA = 0.0 so curvature matrix is
CP                              returned.
CP  PFUNCS              R4  ?/? Input name of model function, with call
CP                                  sequence:
CP                                  CALL PFUNCS(X1,A,YMOD,DYDA,DY2DA2,MA)
CP                                      With:
CP                                      X1 = Independent variable at
CP                                          which to evaluate the model.
CP                                      A(MA) = Array of model params.
CP                                      YMOD = Output model value at X1.
CP                                      DYDA(MA) = Vector of first
CP                                           derivatives of the model
CP                                           w.r.t. the parameters.
CP                                      DY2DA2(20,20)= Matrix of partial
CP                                           second derivatives of the
CP                                           model w.r.t the parameters.
CP                                      LISTA = list of parameters being
CP                                           fitted.
CP                                      MFIT  = number of parameters being
CP                                           fit.
CP                                      MA = Input INTEGER number of
CP                                           parameters in the model.
C
C   include_block name          description
C
C   routines_called    type     description
CR  DCVSRT              SR      Repacks the covarience matrix for MQM
CR  DGAUSJ              SR      Numerical Routines linear equation
CR                              solution routine
CR  MRQ_CB              SR      Computes matrix elements for MQM
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        SAVE
C
        INTEGER MA
        INTEGER FTYPE, IERR, IERR1, IHIT, J, K, KK, LISTA(MA), MFIT,
     +      NCA, NDATA
C
        REAL A(MA), ALAMDA, ATRY(10), CHISQ, OCHISQ,
     +      TEMPX(NDATA), TEMPY(NDATA), TEMPU(NDATA)
C
        DOUBLE PRECISION ALPHA(NCA,NCA), COVAR(NCA,NCA), 
     +      BETA(10), DA(10)
C
        IERR = 0
C
C  Do some initializations
C
        IF(ALAMDA .LT. 0.) THEN
            KK = MFIT + 1
C
C  Check to see if LISTA contains a proper permutation of the coeffs.
C
            J = 0
            DO WHILE ((J .LT. MA) .AND. (IERR .EQ. 0))
                J = J + 1
                IHIT = 0
                DO 11 K=1,MFIT
                    IF(LISTA(K) .EQ. J) IHIT = IHIT + 1
   11           ENDDO
                IF(IHIT .EQ. 0) THEN
                    LISTA(KK) = J
                    KK = KK + 1
                ELSE IF (IHIT .GT. 1) THEN
                    IERR = 3
                    PRINT *, 'MQM: Improper permutation ',
     +                  'in LISTA, 1'
                ENDIF
            ENDDO
C
            IF(KK .NE. (MA+1)) THEN
                 IERR = 3
                 PRINT *, 'MQM: Improper permutation in LISTA, 2'
            ENDIF
C
            IF(IERR .EQ. 0) THEN
                ALAMDA = 0.001
                CALL MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA,
     +              LISTA,MFIT,ALPHA,BETA,NCA,CHISQ,IERR1)
                IF(IERR1 .EQ. 3) THEN
                    IERR = 3
                    PRINT *, 'MQM ==> Bad return from MRQ #1'
                ELSE
                    OCHISQ = CHISQ
                    DO 13 J=1,MA
                        ATRY(J) = A(J)
   13               ENDDO
                ENDIF
            ENDIF
        ENDIF
C
C  Alter linearized fitting matrix by augmenting diagonal elements
C
        IF(IERR .EQ. 0) THEN
            DO 15 J=1,MFIT
                DO 14 K=1,MFIT
                    COVAR(J,K) = ALPHA(J,K)
   14           ENDDO
                COVAR(J,J) = ALPHA(J,J)*(1.0D0 + DBLE(ALAMDA))
                DA(J) = BETA(J)
   15       ENDDO
C
C  Get the matrix solution
C
            CALL DGAUSJ(COVAR,MFIT,NCA,DA,1,1,IERR1)
            IF(IERR1 .EQ. 3) THEN
                IERR = 3
                PRINT *, 'MQM ==> Bad return from DGAUSJ'
                RETURN
            ELSE
C
C  If converged evaluate covarience matrix with ALAMDA = 0
C
                IF(ALAMDA .EQ. 0.) THEN
                    CALL DCVSRT(COVAR,NCA,MA,LISTA,MFIT)
                ELSE
C
C  Check to see if the trial succeeded
C
                    DO 16 J=1,MFIT
                        ATRY(LISTA(J)) = A(LISTA(J)) + SNGL(DA(J))
   16               ENDDO
C
                    CALL MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,
     +                  ATRY,MA,LISTA,MFIT,COVAR,DA,NCA,CHISQ,
     +                  IERR1)
                    IF(IERR1 .EQ. 3) THEN
                        IERR = 3
                        PRINT *,'MQM ==> Bad return ',
     +                          'from MRQ #2'
                    ELSE
                        IF(CHISQ .LT. OCHISQ) THEN
C
C  Success, accept the new solution
C
                            ALAMDA = 0.1*ALAMDA
                            OCHISQ = CHISQ
                            DO 18 J=1,MFIT
                                DO 17 K=1,MFIT
                                    ALPHA(J,K) = COVAR(J,K)
   17                           ENDDO
                                BETA(J) = DA(J)
                                A(LISTA(J)) = ATRY(LISTA(J))
   18                       ENDDO
                        ELSE
C
C  Failure, increase ALAMDA and return
C
                            ALAMDA = 10.*ALAMDA
                            CHISQ = OCHISQ
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF
C
        RETURN
        END
c
c
c
         SUBROUTINE MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA,
     +       LISTA,MFIT,ALPHA,BETA,NALP,CHISQ,IERR)
C
CC  Computes matrix elements for MQM
C
C************************ FFORM VERSION 1.0 ************ DD-MMM-YY HH:MM
C
CA  author : J. Bloch          date:  4-SEP-1987
CU  update : J. Bloch          date:  6-SEP-1987
CU  update : SLS               date:  2-MAR-1988 09:23
C
CT  status: not tested
C
C   general description:
CG  MRQ was derived from MRQCOF in the book "Numerical Recipes"
CG  by Press et al. It computes the matrix elements needed for
CG  the marquardt method applied to the Poisson Maximum Likelihood
CG  Estimator, as described by Cash in the 1979 March 15
CG  Astrophysical Journal. The SIG array has been removed because
CG  it is not needed.
C
C   call_var.          type I/O description
CP  FTYPE               I4      = 1 Polynomial function (PFUNCS)
CP                              = 2 Gaussian function (GFUNCS)
CP  IERR                I4    O = 0 no error
CP                              = 3 illegal model value (model < 0.)
CP  TEMPX               R4  ?/? array of independent var. data values.
CP  TEMPY               R4  ?/? array of dependent variable data values.
CP  TEMPU               R4  ?/? array of dependent variable uncertainty values.
CP  NDATA               I4      number of data points to fit.
CP  A                   R4  I/O array of model parameter values.
CP  MA                  I4  I/? number of model parameter values.
CP  LISTA               I4  I/O array list of parameters
CP                                  in array A actually being varied.
CP  MFIT                I4  I/? number of parameters actually being fit.
CP  COVAR               R8  I/O covariance matrix and work
CP                                  array used durint the fit.
CP  ALPHA               R8  I/O matrix ALPHA used in the Marquard
CP                              method.  (See Press et al., pp. 521-528)
CP  BETA                        vector beta used in the Marquardt
CP                              method.  (See Press et al. pp. 521-528)
CP  NALP                        actual dimensions of ALPHA and BETA.
CP  CHISQ               R4  I/O Output Maximum Likelihood Estimator.
C
C   include_block_name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  HFLAG               R       output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
CR  PFUNCS              SR      input name of model function, with call
CR                                  sequence:
CR                              CALL PFUNCS(X1,A,YMOD,DYDA,DY2DA2,LISTA,
CR                                          MFIT,MA)
CR                                  With:
CR                                    X1 = Independent variable at
CR                                        which to evaluate the model.
CR                                    A(MA) = Array of model parameters.
CR                                    YMOD = Output model value at X1.
CR                                    DYDA(MA) = Vector of first
CR                                         derivatives of the model
CR                                         w.r.t. the parameters.
CR                                    DY2DA2(20,20) = Matrix of partial
CR                                         second derivatives of the
CR                                         model w.r.t the parameters.
CR                                    LISTA(MFIT) = Same as above.
CR                                    MFIT = Same as above.
CR                                    MA = Input INTEGER number of
CR                                         parameters in the model.
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
C       PARAMETER MMAX=10
C
        INTEGER MFIT
        INTEGER FTYPE, I, IERR, J, K, KK, LISTA(MFIT), 
     +      MA, NALP, NDATA
C
        REAL A(MA), CHISQ, DY, DYDA(10), D2YDA2(10,10),
     +          TEMPX(NDATA), TEMPY(NDATA), SIG2I, TEMPU(NDATA),
     +          WT, YMOD
C
        DOUBLE PRECISION ALPHA(NALP,NALP), BETA(MA)
C
        IERR = 0
C
C  Initialize (symmetric) ALPHA and BETA
C
        DO 12 J=1,MFIT
            DO 11 K=1,J
                ALPHA(J,K) = 0.
   11       ENDDO
            BETA(J) = 0.
   12   ENDDO
C
        CHISQ = 0.0
C
C  Summation loop over all data
C
        I = 0
        DO WHILE ((I .LT. NDATA) .AND. (IERR .EQ. 0))
            I = I + 1
            IF(FTYPE .EQ. 1) THEN
                CALL PFUNCS(TEMPX(I),A,YMOD,DYDA,
     +                  D2YDA2,LISTA,MFIT,MA)
            ELSEIF(FTYPE .EQ. 2) THEN
                CALL GFUNCS(TEMPX(I),A,YMOD,DYDA,
     +                  D2YDA2,LISTA,MFIT,MA)
            ELSE
                PRINT *, 'Illegal model type'
            ENDIF
            IF(YMOD .LT. -1.0D200) THEN
                PRINT *,'MRQ: Illegal model value =', YMOD
                PRINT *,'               TEMPX(I) =', TEMPX(I)
                PRINT *,'               TEMPY(I) =', TEMPY(I)
                PRINT *,'               TEMPU(I) =', TEMPU(I)
                PRINT *,'                      A =',
     +                  (A(KK),KK=1,MA)
                IERR = 3
            ELSE
                SIG2I = 1./(TEMPU(I)*TEMPU(I))
                DY = TEMPY(I) - YMOD
C
                DO 14 J=1,MFIT
                    WT = SIG2I*DYDA(LISTA(J))
                    DO 13 K=1,J
                        ALPHA(J,K) = ALPHA(J,K) + WT*DYDA(LISTA(K))
   13               ENDDO
                    BETA(J) = BETA(J) + DY*WT
   14           ENDDO
                CHISQ = CHISQ + DY*DY*SIG2I
c                print *, tempx(i), tempy(i), ymod, chisq
            ENDIF
        ENDDO
C
        if(ierr .eq. 0) then
C
C  Fill in the symmetric side
C
            do j=2,mfit
                do k=1,j-1
                    alpha(k,j) = alpha(j,k)
                enddo
            enddo
        endif
c
        return
        end
c
c
c
        SUBROUTINE PFUNCS (TEMPX,ALOC,YMODEL,DYDA,DY2DA2,
     +      LISTA,MFIT,MA)
C
CC  Calculates polynomial function distribution and partial derivatives
C
C************************ FFORM VERSION 1.0 ************ 21-APR-88 08:24
C
CA  author : SLS               date: 17-August-2005 15:00
C
CT  status: not tested
C
C   general description
CG  PFUNCS is the model value subroutine for poly_fit for use with the
CG  fitting subroutine MQMLMN.FOR.
C
C   call_var.          type I/O description
CP  TEMPX               R4  I/? X value.
CP  ALOC                R4  I/? array of model values.
CP                              A(1) = constant
CP                              A(2) = linear
CP                              A(3) = quadratic
CP                              A(4) - A(10) third to ninth order
CP  YMODEL              R4      value of the polynomial.
CP  DYDA                R4  I/O array of first partial derivatives
CP                              w.r.t. each parameter. (Only the
CP                              parameters actually being fit are
CP                              computed.)
CP  DY2DA2              R4  I/O array of second partial derivatives
CP                              w.r.t. the parameters.  (Only those
CP                              parameters being fit will be computed.)
CP  LISTA               I4  I   list of parameters being fitted.
CP  MFIT                I4  I   number of parameters being fit.
CP  MA                  I4  I   number of parameters in model.
CP                              (Should be <= 10)
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
C
C***********************************************************************
C
        IMPLICIT NONE
C
        SAVE
C
        INTEGER I, II, MFIT, LISTA(MFIT), MA
C
        REAL ALOC(MA), DY2DA2(10,10), DYDA(10), TEMPX, YMODEL
C
        DOUBLE PRECISION A(10), DTIME, YMOD
C
        DO I=1,10
            A(I) = DBLE(ALOC(I))
        ENDDO
        DTIME = DBLE(TEMPX)
C
C  Compute the model value
C
        YMOD = A(1) + A(2)*DTIME + A(3)*DTIME*DTIME +
     +      A(4)*DTIME*DTIME*DTIME +
     +      A(5)*DTIME*DTIME*DTIME*DTIME +
     +      A(6)*DTIME*DTIME*DTIME*DTIME*DTIME +
     +      A(7)*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME +
     +      A(8)*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        YMODEL = SNGL(YMOD)
C
C  Compute values on intervals to prepare for computation
C  of numerical derivatives.
C
        DYDA(1) = 1
        DYDA(2) = DTIME
        DYDA(3) = DTIME*DTIME
        DYDA(4) = DTIME*DTIME*DTIME
        DYDA(5) = DTIME*DTIME*DTIME*DTIME
        DYDA(6) = DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(7) = DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(8) = DTIME*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(9) = 0.0
        DYDA(10) = 0.0
C
        DO I=1,10
            DO II=1,10
                DY2DA2(I,II) = 0
            ENDDO
        ENDDO
C
        RETURN
        END
c
c
c
        subroutine poly_fit(tempx,tempy,tempu,n,np,ap)
c
        implicit none
c
        integer ftype, i, ierr, iflag, iup, iuplim, 
     +      lista(10), maxitr, n, np, nppo
        real a(10), ainit(10), alamda, almlim, ap(10), chisq, deltch,
     +      i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, it,
     +      ochisq, olamda, t, tempx(n), tempy(n), tempu(n)
        double precision alpha(10,10), covar(10,10)
        character*1 fix(10)
        character*15 pshort(10)
        character*80 pname(10)
c
        common /modpar/ i0, i1, i2, i3, i4, i5, i6, i7, i8, i9
c
        equivalence(a(1),i0)
c
        data ainit /1.0, -0.1, 0.03, -0.01, 0.003,
     +         -0.0001, 0.00003, -0.00001, 0.000003, -0.000001/
        data fix/10*' '/
        data ftype /1/
        data pshort /'I0', 'I1', 'I2', 'I3', 'I4', 'I5', 'I6',
     +      'I7', 'I8', 'I9'/
c
        ierr = 0
c
        do i=1,10
            a(i) = ainit(i)
        enddo
c
        it = 0.0
        t = 0.0
        do i=1,n
            if(tempy(i) .ne. 0.0) then
                t = t + tempy(i)
                it = it + 1
            endif
        enddo
        a(1) = t/it
c
        nppo = np + 1
        do i=0,np
            lista(i+1) = i + 1
            fix(i+1) = '*'
            write(pname(i+1),3000) i
 3000       format('I',i1)
        enddo
        if(np+2 .lt. 10) then
            do i=np+2,9
                lista(i+1) = 0
                fix(i+1) = ' '
                pname(i+1) = ' '
                a(i) = 0.0
            enddo
        endif
c
c  *********************************************************************
c  *                                                                   *
c  *                       Main Fit Begins Here                        *
c  *                                                                   *
c  *********************************************************************
c
        maxitr = 100
        almlim = 1.E3
        iuplim = 10
c
c        call report(-1,' Initial Model Value Settings:',a,fix)
c
        alamda = -1.0
        ochisq = 0.0
        iup = 0
        i = 1
        iflag = 0
c
        do while ((iflag .eq. 0) .and. (i .lt. maxitr))
            i = i + 1
            olamda = alamda
            call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo,
     +          covar,alpha,10,chisq,alamda,ierr)
            if(ierr .eq. 3) then
                print *, 'Bad return from MQM'
                print *, 'Program stopped in POLY_FIT'
                print *, 'Position 01'
                stop
            endif
c
            deltch = abs(ochisq - chisq)
c
c  Check for various termination conditions
c
            if((alamda .lt. olamda) .and. (deltch .lt. 0.1))
     +              iflag = 1
            if(iflag .eq. 0) then
                if(alamda .gt. olamda) iup = iup + 1
                if(alamda .lt. olamda) iup = 0
                if(iup .gt. iuplim) iflag = 1
                if(alamda .gt. almlim) iup = iup + 1
c
c                write(*,*)
c                write(*,*) 'ITER =',i,'  ALAMDA =',alamda,
c     +                  '  M.L.E. =',chisq
c                call report(-1,'Intermediate step..',a,fix)
            endif
        enddo
c
c        write(*,*)
c        call report(-1,' Final Model Values: (*=Floating)',a,fix)
        write(*,1010) ' Np =',np,' N =',n,' M.L.E. =',chisq
 1010   format(/,1x,a,i2,a,i4,a,g12.4,a,g14.7)
c
c  Finish off the fit to get the covarience matrix
c
        alamda = 0.0
        call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo,
     +          covar,alpha,10,chisq,alamda,ierr)
        if(ierr .eq. 3) then
            print *, 'Bad return from MQM'
            print *, 'Program stopped in POLY_FIT'
            print *, 'Position 02'
            stop
        endif
c
c  Write out final values and one sigma uncertainties
c
        print *, 'Final parameter values and the formal standard errors'
        print *
c
        do i=1,nppo
            ap(i) = a(i)
            write(6,2100) pshort(lista(i)),a(lista(i)),
     +              sngl(dsqrt(dabs(covar(lista(i),lista(i)))))
 2100       format(1x,a15,f13.5,' +/-',f11.5)
        enddo
c
        return
        end
c
c
c
        subroutine putfit(infile,rarray,i2array,i4array,iflag,
     +      units,comm,isize1,isize2,bzero,bscale,ctype,crval,cdelt,
     +      crpix,clobber,status)
c
c  INFILE    file name of output file
c  RARRAY    REAL*4 array for output
c  I2ARRAY   INTEGER*2 array for output
c  I4ARRAY   INTEGER*4 array for output
c  IFLAG     flag for input file type selection
c            1 => R*4 array
c            2 => I*2 array
c            3 => I*4 array
c  UNITS     character string showing the units of the array
c  COMM      comment character string
c  ISIZE     array (square) dimension
c  BZERO     offset value for array
c  BSCALE    slope value for array
c  CRVAL1    reference coordinate for first axis (RA)
c  CDELT1    pixel size for first axis
c  CRVAL2    reference coordinate for second axis (Dec)
c  CDELT2    pixel size for second axis
C  CLOBBER   1 = overwrite an existing file
C            0 = don't
c  STATUS    FITSIO status flag
c
        implicit none
c
        integer isize1, isize2
        real bzero, bscale, cdelt1, cdelt2,
     +      crpix1, crpix2, crval1, crval2, rarray(isize1,isize2)
        double precision cdelt(2), crpix(2), crval(2)
        integer*2 i2array(isize1,isize2)
        integer i4array(isize1,isize2)
        character*5  com
        character*8  ctype(2), units
        character*60 comm, comment
        character*132 infile
        integer bitpix, block, clobber, err, fpixel, iflag,
     +      gcount, group, naxes(10), naxis, lun, pcount, status
        logical extend, simple
c
        equivalence (comment,com)
c
        err = 0
c
c  Get an unused lun
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 31'
        endif
        comment = comm
c
c  Write out the array
c
        print *, 'Writing output file'
        print *, infile
c
        simple = .TRUE.
        naxis = 2
        naxes(1) = isize1
        naxes(2) = isize2
        pcount = 0
        gcount = 1
        extend = .FALSE.
        if(iflag .eq. 1) then
            bitpix = -32
        elseif(iflag .eq. 2) then
            bitpix = 16
        elseif(iflag .eq. 3) then
            bitpix = 32
        endif
        group = 1
        fpixel = 1
c
c  Check to see if there is an existing file
c
        call ftopen (lun,infile,0,block,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber .eq.  1) then
                print *, 'Clobber the file and continue'
                call ftdelt(lun,status)
            else
                print *, 'Will not clobber file'
                print *, 'XMM-BACK-IMAGE POSITION 0'
                stop
            endif
        endif
        status = 0
c
c  Open the output file
c
        call ftinit(lun,infile,2880,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'POSITION 1'
            print *, infile
        endif
c
c  Set the output file header
c
        call ftphpr(lun,simple,bitpix,naxis,naxes,pcount,gcount,
     +              extend,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTPHPR =', status
            print *, 'POSITION 2'
            print *, infile
        endif
c
c  Write some useful header keywords
c
        call ftpcom(lun,comment,status)
        call ftpkys(lun,'CONTENT','IMAGE','  ',status)
        call ftpkys(lun,'TELESCOP','XMM','mission name',status)
        call ftpkys(lun,'INSTRUME','EPIC','instrument name',status)
        call ftpkys(lun,'OBS_MODE','POINTING',
     +          'obs mode: POINTING,SLEW, OR SCAN',status)
        call ftpkys(lun,'RADECSYS','FK5',
     +          'Equatorial system reference',status)
        call ftpkyj(lun,'EQUINOX',2000,'Equinox',status)
        call ftpkyf(lun,'BZERO',bzero,4,' ',status)
        call ftpkyf(lun,'BSCALE',bscale,8,' ',status)
        call ftpkys(lun,'BUNIT',units,'Units of data',status)
        crpix1 = crpix(1)
        call ftpkyf(lun,'CRPIX1',crpix1,4,'Reference pixel',status)
        call ftpkys(lun,'CTYPE1',ctype(1),'Projection',status)
        crval1 = crval(1)
        call ftpkyf(lun,'CRVAL1',crval1,4,'Central longitude',status)
        cdelt1 = cdelt(1)
        call ftpkyf(lun,'CDELT1',cdelt1,8,'Pixel size',status)
        call ftpkys(lun,'CUNIT1','deg','Units of coordinate',status)
c
        crpix2 = crpix(2)
        call ftpkyf(lun,'CRPIX2',crpix2,4,'Reference pixel',status)
        call ftpkys(lun,'CTYPE2',ctype(2),'Projection',status)
        crval2 = crval(2)
        call ftpkyf(lun,'CRVAL2',crval2,4,'Central latitude',status)
        cdelt2 = cdelt(2)
        call ftpkyf(lun,'CDELT2',cdelt2,8,'Pixel size',status)
        call ftpkys(lun,'CUNIT2','deg','Units of coordinate',status)
c
        call ftpcom(lun,
     +  'This file was produced using the XMM-ESAS software written by',
     +  status)
        call ftpcom(lun,
     +  'S. L. Snowden, and written using the FITSIO package of ',
     +  status)
        call ftpcom(lun,'W. D. Pence.',status)
c
c  Define the file format
c
        call ftpdef(lun,bitpix,naxis,naxes,pcount,gcount,extend,
     +              status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTPDEF =', status
            print *, 'POSITION 3'
            print *, infile
        endif
c
c  Write the file
c
        if(iflag .eq. 1) then
            call ftp2de(lun,group,isize1,isize1,isize2,
     +          rarray,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTP2DE =', status
                print *, 'POSITION 4'
                print *, infile
            endif
        elseif(iflag .eq. 2) then
            call ftp2di(lun,group,isize1,isize1,isize2,
     +          i2array,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTP2DI =', status
                print *, 'POSITION 5'
                print *, infile
            endif
        elseif(iflag .eq. 3) then
            call ftp2dj(lun,group,isize1,isize1,isize2,
     +          i4array,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTP2DJ =', status
                print *, 'POSITION 6'
                print *, infile
            endif
        endif
c
c  Close the file
c
        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 7'
            print *, infile
        endif
c
c  Clean up
c
        call ftfiou (lun,status)
c

        return
        end
c
c
c
        subroutine read_arf (obj_arf,mchan,nchan,arf,emin,emax,err)
c
        implicit none
c
        integer mchan
        integer block, err, hdutype, i, iarf, icol, imin,  
     +      imax, irow, lun, nchan, ncols, status, tfields, varidat
        real arf(mchan), emax(mchan), emin(mchan)
        character*8 colnam(10), tform(10), ttype(10), tunit(10)
        character*20 extname
        character*80 obj_arf
        logical anyf
c
        status = 0
        err = 0
c
c  Get an unused lun
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            print *, 'READ_ARF'
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 01'
            err = 3
            return
        ENDIF
c
c  Open the FITS file
c
        call ftopen (lun,obj_arf,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'READ_ARF'
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 02'
            print *, obj_arf
            err = 3
            return
        endif
c
c  Read the FITS file header information
c
        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'READ_ARF'
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 03'
                print *, 'Could not find the SPECRESP extension'
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,10,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'READ_ARF'
                    print *, 'STATUS FTGHBN =', status
                    print *, 'POSITION 04'
                    err = 3
                    return
                endif
                if(extname .ne. 'SPECRESP') then
                    print *, extname
                    print *, 'READ_ARF'
                    print *,' POSITION 05'
                    print *, 'This was supposed to be the SPECRESP ext'
                    err = 3
                    return
                endif
                if(mchan .lt. nchan) then
                    print *, 'READ_ARF'
                    print *,' POSITION 06'
                    print *, 'Number of array elements smaller than',
     +                      ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, 'READ_ARF'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'POSITION 07'
                    print *, 'Failure on reading the header ',
     +                      'information'
                    err = 3
                    return
                else
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, 'READ_ARF'
                        print *, 'STATUS FTGKNS =', status
                        print *, 'POSITION 08'
                        print *, 'Failure on reading the column names'
                    endif
c
c  Extract the channel numbers
c
                    iarf = 0
                    imin = 0
                    imax = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'ENERG_LO') imin = icol
                        if(colnam(icol) .eq. 'ENERG_HI') imax = icol
                        if(colnam(icol) .eq. 'SPECRESP') iarf = icol
                    enddo
                    do irow=1,nchan
                        arf(irow) = 0
                        emin(irow) = 0
                        emax(irow) = 0
                    enddo
                    do i=1,nchan
                        if(iarf .ne. 0)
     +                      call ftgcve(lun,iarf,i,1,1,0,
     +                          arf(i),anyf,status)
                        if(imin .ne. 0)
     +                      call ftgcve(lun,imin,i,1,1,0,
     +                          emin(i),anyf,status)
                        if(imax .ne. 0)
     +                      call ftgcve(lun,imax,i,1,1,0,
     +                          emax(i),anyf,status)
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'READ_ARF'
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 09'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
            if(status .ne. 0) then
                print *, 'READ_ARF'
                print *, 'STATUS = ', status
                print *, 'POSITION 09'
                print *, 'Failure in READ_RMF getting energy bounds'
            endif
            call ftclos (lun,status)
        endif
c
c  Clean up
c
        call ftfiou (lun,status)
c
        return
        end
c
c
c
        subroutine read_pha (spec_name,mchan,nchan,channel,counts,stat,
     +          quality,group,exposure,backscal,areascal,revl,obsid,
     +          modet,err)
c
        implicit none
c
        integer mchan
        integer block, channel(mchan), err, frmtime, group(mchan), 
     +      hdutype, i, ich, 
     +      icol, icou, igrp, iqual, irow, lun, nchan, ncols, 
     +      obsid, quality(mchan), revl, status, tfields, varidat
        real areascal, backscal, counts(mchan), exposure, stat(mchan)
        character*1 mod(25)
        character*8 colnam(10), tform(10), ttype(10), tunit(10)
        character*20 extname
        character*25 mode, modet
        character*80 com, spec_name
        logical anyf
c
        equivalence (mode,mod)
c
        status = 0
        err = 0
c
c  Get an unused lun
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'READ_PHA POSITION 01'
        endif
c
c  Open the FITS file
c
        call ftopen (lun,spec_name,0,block,status)
        if(status .ne. 0) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'READ_PHA POSITION 02'
            print *, spec_name
        endif
c
c  Read in the header information
c
        call ftgkyj (lun,'OBS_ID',obsid,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 03'
            print *, 'Missing OBS_ID keyword'
            print *, spec_name
            status = 0
            err = 2
        endif
c
c  Read in the header information
c
        call ftgkyj (lun,'REVOLUT',revl,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 04'
            print *, 'Missing REVOLUT keyword'
            status = 0
            err = 2
        endif
c
c  Read in the instrument information
c
        call ftgkys (lun,'INSTRUME',mode,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 05'
            print *, 'Missing INSTRUME keyword'
            status = 0
            err = 2
        elseif (mode .eq. 'EPN') then
c
c  Read in the mode information if a PN observation
c
            call ftgkys (lun,'SUBMODE',mode,com,status)
            if(status .ne. 0) then
                print *, 'READ_PHA POSITION 06'
                print *, 'Missing SUBMODE keyword'
                status = 0
                err = 2
            endif
            call ftgkyj (lun,'FRMTIME',frmtime,com,status)
            if(status .ne. 0) then
                print *, 'READ_PHA POSITION 07'
                print *, 'Missing FRMTIME keyword'
                status = 0
                err = 2
            else
                if(frmtime .lt. 100) then
                    continue
                elseif(frmtime .lt. 210) then
                    mod(24) = '3'
                else
                    mod(24) = '5'
                endif
                mod(25) = ' '
                modet = mode
            endif
        endif
c
c  Move to the spectrum extension
c
        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMRHD =', status
                print *, 'READ_PHA POSITION 08'
                print *, 'Could not find the spectrum extension'
                print *, spec_name
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,10,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                if(extname .ne. 'SPECTRUM') then
                    print *, 'READ_PHA POSITION 09'
                    print *, extname
                    print *, 'This was supposed to be a spectrum ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, spec_name
                    print *,' 2', mchan, nchan
                    print *, 'Number of array elements smaller than',
     +                      ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_PHA POSITION 10'
                    print *, 'Failure on reading the header ',
     +                      'information'
                else
c
c  Read in the header information
c
                    call ftgkye (lun,'BACKSCAL',backscal,com,status)
                    if(status .ne. 0) then
                        print *, 'READ_PHA POSITION 11'
                        print *, 'Missing BACKSCAL keyword'
                        status = 0
                        err = 2
                    endif
                    call ftgkye (lun,'EXPOSURE',exposure,com,status)
                    if(status .ne. 0) then
                        if(backscal .gt. 0) then
                            print *, 'READ_PHA POSITION 12a'
                            print *, 'Missing EXPOSURE keyword'
                        else
                            print *, 'READ_PHA POSITION 12b'
                            print *, 'Missing EXPOSURE keyword'
                            print *, 'Probably due to exclustion of ',
     +                        'of the CCD in the selection expression'
                            print *, 'or no selected data on the CCD'
                        endif
                        status = 0
                        err = 2
                    endif
                    call ftgkye (lun,'AREASCAL',areascal,com,status)
                    if(status .ne. 0) then
                        print *, 'READ_PHA POSITION 13'
                        print *, 'Missing AREASCAL keyword'
                        status = 0
                        err = 2
                    endif
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
c
c  Extract the channel numbers
c
                    ich = 0
                    icou = 0
                    igrp = 0
                    iqual = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'CHANNEL') ich = icol
                        if(colnam(icol) .eq. 'COUNTS') icou = icol
                        if(colnam(icol) .eq. 'GROUPING') igrp = icol
                        if(colnam(icol) .eq. 'QUALITY') iqual = icol
                    enddo
                    do irow=1,nchan
                        channel(irow) = 0
                        counts(irow) = 0
                        group(irow) = 0
                        quality(irow) = 0
                    enddo
                    do i=1,nchan
                        if(ich .ne. 0)
     +                      call ftgcvj(lun,ich,i,1,1,0,
     +                          channel(i),anyf,status)
                        if(icou .ne. 0)
     +                      call ftgcve(lun,icou,i,1,1,0,
     +                          counts(i),anyf,status)
                        if(igrp .ne. 0)
     +                      call ftgcvj(lun,igrp,i,1,1,0,
     +                          group(i),anyf,status)
                        if(iqual .ne. 0)
     +                      call ftgcvj(lun,iqual,i,1,1,0,
     +                          quality(i),anyf,status)
                    enddo
c
c  Calculate the uncertainties
c
                    do irow=1,nchan
                        stat(irow) = sqrt(abs(counts(irow)))
                    enddo
                endif
            endif
            call ftclos (lun,status)
        endif
c
c  Clean up
c
        call ftfiou (lun,status)
c
        return
        end
c
c
c
        subroutine read_rmf (obj_rmf,mchan,detm,chan,
     +      eminin,emaxin,nchanrin,eminout,emaxout,nchanrout)
c
        implicit none
c
        integer mchan
        integer block, chan(mchan), err, 
     +      fchan(mchan,20), fchant(mchan), 
     +      fnum, hdutype, i, ich, icol, idetm, iehi, ielo, 
     +      iifchan, igrp, ii, imin, imax, inchan, 
     +      ioff, irow, k, kk, lun, mnum, nchan, nchanr(mchan,20), 
     +      nchanrin, nchanrout, nchant(mchan), ncols, ngrp(mchan), 
     +      nnum, status, tfields, tnull, varidat
        real detm(mchan,mchan), detmt(mchan), emaxin(mchan), 
     +      emaxout(mchan), eminin(mchan), eminout(mchan)
        double precision tscal, tzero
        character*1 dtyp
        character*8 colnam(10), tdisp, tform(10), ttyp, ttype(10), 
     +      tuni, tunit(10)
        character*20 extname
        character*80 obj_rmf
        logical anyf
c
        status = 0
        err = 0
c
c  Get an unused lun
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, '  '
            print *, 'STATUS FTGIOU =', status
            print *, 'READ_RMF POSITION 31'
        endif
c
c  Open the FITS file
c
        call ftopen (lun,obj_rmf,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, '  '
            print *, 'STATUS FTOPEN =', status
            print *, 'READ_RMF POSITION 32'
            print *, obj_rmf
        endif
c
c  Move to the matrix extension
c
        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, '  '
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 33'
                print *, 'Could not find the matrix extension'
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,10,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                nchanrin = nchan
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHBN =', status
                    print *, 'READ_RMF POSITION 34'
                    print *, 'Could not find the matrix extension'
                endif
                if(extname .ne. 'MATRIX') then
                    print *, extname
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'This was supposed to be a matrix ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, obj_rmf
                    print *,' 3', mchan, nchan
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'Number of array elements smaller than',
     +                      ' number of channels in matrix'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_RMF POSITION 35'
                    print *, 'Failure on reading the header ',
     +                      'information'
                else
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, '  '
                        print *, 'STATUS FTGKNS =', status
                        print *, 'READ_RMF POSITION 36'
                        print *, 'Failure on reading the column names'
                    endif
c
c  Extract the channel numbers
c
                    ielo = 0
                    iehi = 0
                    igrp = 0
                    iifchan = 0
                    inchan = 0
                    idetm = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'ENERG_LO') ielo = icol
                        if(colnam(icol) .eq. 'ENERG_HI') iehi = icol
                        if(colnam(icol) .eq. 'N_GRP') igrp = icol
                        if(colnam(icol) .eq. 'F_CHAN') iifchan = icol
                        if(colnam(icol) .eq. 'N_CHAN') inchan = icol
                        if(colnam(icol) .eq. 'MATRIX') idetm = icol
                    enddo
                    do irow=1,nchan
                        eminin(irow) = 0
                        emaxin(irow) = 0
                        ngrp(irow) = 0
                        do ii=1,20
                            nchanr(irow,ii) = 0
                            fchan(irow,ii) = 0
                        enddo
                        do ii=1,mchan
                            detm(irow,ii) = 0
                        enddo
                    enddo
                    call ftgbcl(lun,iifchan,ttyp,tuni,dtyp,fnum,
     +                  tscal,tzero,tnull,tdisp,status)
                    call ftgbcl(lun,inchan,ttyp,tuni,dtyp,nnum,
     +                  tscal,tzero,tnull,tdisp,status)
                    do i=1,nchan
                        if(ielo .ne. 0)
     +                      call ftgcve(lun,ielo,i,1,1,0,
     +                          eminin(i),anyf,status)
                        if(iehi .ne. 0)
     +                      call ftgcve(lun,iehi,i,1,1,0,
     +                          emaxin(i),anyf,status)
                        if(igrp .ne. 0)
     +                      call ftgcvj(lun,igrp,i,1,1,0,
     +                          ngrp(i),anyf,status)
                        if(iifchan .ne. 0) then
                            call ftgcvj(lun,iifchan,i,1,fnum,0,
     +                              fchant,anyf,status)
                            do ii=1,fnum
                                fchan(i,ii) = fchant(ii)
                            enddo
                        endif
                        if(inchan .ne. 0) then
                            call ftgcvj(lun,inchan,i,1,nnum,0,
     +                              nchant,anyf,status)
                            do ii=1,nnum
                                nchanr(i,ii) = nchant(ii)
                            enddo
                        endif
                        if(idetm .ne. 0) then
                            call ftgdes(lun,idetm,i,mnum,ioff,status)
                            call ftgcve(lun,idetm,i,1,mnum,0,
     +                              detmt,anyf,status)
                            kk = 0
                            do k=1,fnum
                                do ii=1,nchant(k)
                                    kk = kk + 1
                                    ioff = fchant(k) + ii
                                    detm(i,ioff) = detmt(kk)
                                enddo
                            enddo
                        endif
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 37'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
        endif
c
c  Move to the EBOUNDS extension
c
        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, '  '
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 33'
                print *, 'Could not find the EBOUNDS extension'
            else
c
c  Read the FITS file header information
c
                call ftghbn (lun,10,nchan,tfields,ttype,tform,
     +              tunit,extname,varidat,status)
                nchanrout = nchan
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHBN =', status
                    print *, 'READ_RMF POSITION 34'
                    print *, 'Could not find the spectrum extension'
                endif
                if(extname .ne. 'EBOUNDS') then
                    print *, extname
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'This was supposed to be a energy ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, obj_rmf
                    print *,' 3', mchan, nchan
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'Number of array elements smaller than',
     +                      ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_RMF POSITION 35'
                    print *, 'Failure on reading the header ',
     +                      'information'
                else
c
c  Read in the column names
c
                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, '  '
                        print *, 'STATUS FTGKNS =', status
                        print *, 'READ_RMF POSITION 36'
                        print *, 'Failure on reading the column names'
                    endif
c
c  Extract the channel numbers
c
                    ich = 0
                    imin = 0
                    imax = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'CHANNEL') ich = icol
                        if(colnam(icol) .eq. 'E_MIN') imin = icol
                        if(colnam(icol) .eq. 'E_MAX') imax = icol
                    enddo
                    do irow=1,nchan
                        chan(irow) = 0
                        eminout(irow) = 0
                        emaxout(irow) = 0
                    enddo
                    do i=1,nchan
                        if(ich .ne. 0)
     +                      call ftgcvj(lun,ich,i,1,1,0,
     +                          chan(i),anyf,status)
                        if(imin .ne. 0)
     +                      call ftgcve(lun,imin,i,1,1,0,
     +                          eminout(i),anyf,status)
                        if(imax .ne. 0)
     +                      call ftgcve(lun,imax,i,1,1,0,
     +                          emaxout(i),anyf,status)
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 37'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
            if(status .ne. 0) then
                print *, 'Failure in READ_RMF getting energy bounds'
                print *, 'STATUS = ', status
            endif
            call ftclos (lun,status)
        endif
c
c  Clean up
c
        call ftfiou (lun,status)
c
        return
        end
c
c
c
      subroutine REPOL1(DX,DY,DZ,DR,DPHI,DTHETA)
C
CC    Converts rectangular coordinates into polar coordinates
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
C
CA  author : KOD        date: 15-JUN-1989 18:31
CU  update : KOD        date: 20-MAR-1990 15:09
C
CT  status: tested
C
C   general description
CG  Subroutine to convert the rectangular coordinates (DX,DY,DZ)
CG  into the polar coordinates (DR,DPHI,DTHETA) - mathematical
CG  notation.
C
C   call_var.          type I/O description
CP  DX                  R8  I   x-value
CP  DY                  R8  I   y-value
CP  DZ                  R8  I   z-value
CP
CP  DR                  R8    O radius   (same unit as DX,DY,DZ)
CP  DPHI                R8    O azimuthal angle  (radians)
CP  DTHETA              R8    O  polar    angle  (radians)
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
CI  IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
      INCLUDE 'igenl.inc'
C
      DOUBLE PRECISION DX, DY, DZ, DR, DPHI, DTHETA, DH
C
      DR = dsqrt(DX*DX+DY*DY+DZ*DZ)
C
      if (DR.eq.0.0D0) then
        DPHI   = 0.0D0
        DTHETA = 0.0D0
      else
        DTHETA = dacos(DZ/DR)
        DH = dsqrt(DX*DX+DY*DY)
        if (DH.eq.0.0D0) then
          DPHI = 0.0D0
        else
          DPHI = dasin(DY/DH)
          if (DX  .lt. 0.0D0) DPHI = DPI  - DPHI
          if (DPHI.lt. 0.0D0) DPHI = DPHI + D2PI
        endif
      endif
C
      RETURN
      END
c
c
c
        SUBROUTINE REPORT(IO,MESS,A,FIX)
C
CC  Writes out parameter values from MQM to a unit
C
C************************ FFORM VERSION 1.0 ************ 22-MAR-88 08:22
C
CA  author : J. Bloch          date: ??-SEP-1987
CU  update : SLS               date: 17-August-2005
C
CT  status: not tested
C
C   general description
CG  Writes out parameter values to a unit
C
C   call_var.          type I/O description
CP  IO                  I4  I
CP  MESS                C
CP  A                   R4  I
CP  FIX                 C   I
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
c        IMPLICIT NONE
C
        INTEGER IO
C
        REAL A(10), I0, I1, I2, I3, I4, I5, I6, I7, I8, I9, TEMP(10)
C
        COMMON /DUMMY/ I0, I1, I2, I3, I4, I5, I6, I7, I8, I9
C
        CHARACTER*1 FIX(10)
        CHARACTER*(*) MESS
C
        EQUIVALENCE(TEMP(1),I0)
C
        DO 5 I=1,10
            TEMP(I) = A(I)
    5   ENDDO
C
        IF(IO .GT. 0) THEN
            WRITE(IO,*) MESS
            WRITE(IO,10) '   ',FIX(1),'I0=',I0,
     +          '   ',FIX(2),'I1=',I1,
     +          '   ',FIX(3),'I2=',I2,
     +          '   ',FIX(4),'I3=',I3
            WRITE(IO,10) '   ',FIX(5),'I4=',I4,
     +          '   ',FIX(6),'I5=',I5,
     +          '   ',FIX(7),'I6=',I6,
     +          '   ',FIX(8),'I7=',I7
            WRITE(IO,10) '   ',FIX(9),'I8=',I8,
     +          '   ',FIX(10),'I9=',I9
        ELSE
            WRITE(*,*) MESS
            WRITE(*,10) '   ',FIX(1),'I0=',I0,
     +          '   ',FIX(2),'I1=',I1,
     +          '   ',FIX(3),'I2=',I2,
     +          '   ',FIX(4),'I3=',I3
            WRITE(*,10) '   ',FIX(5),'I4=',I4,
     +          '   ',FIX(6),'I5=',I5,
     +          '   ',FIX(7),'I6=',I6,
     +          '   ',FIX(8),'I7=',I7
            WRITE(*,10) '   ',FIX(9),'I8=',I8,
     +          '   ',FIX(10),'I9=',I9
        ENDIF
   10   FORMAT(1X,3A,F15.8,3A,F15.8,3A,F15.8,3A,F15.8)
C
        RETURN
        END
c
c
c
      subroutine RESVAL(DXMIN,DXMAX,DXOLD,DXNEW,IERR)
C
CC    'Resets' periodic variables to their basic interval
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
C
CA  author : KOD        date: 19-JUL-1989 18:31
CU  update : KOD        date: 20-MAR-1990 15:10
C
CT  status: tested
C
C   general description
CG  Subroutine to 'reset' the values of periodically defined
CG  variables to their basic interval.
C
C   call_var.          type I/O description
CP  DXMIN               R8  I   left  boundary of basic interval
CP  DXMAX               R8  I   right boundary of basic interval
CP
CP  DXOLD               R8  I   value before resetting
CP  DXNEW               R8    O value after  resetting
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
      IMPLICIT NONE
      INCLUDE 'cgenl.cmn'
C
      DOUBLE PRECISION DXMIN, DXMAX, DXOLD, DXNEW, DDIFF, DH
      INTEGER     IERR
      CHARACTER*8   RTNAME
      CHARACTER*80  LOG
C
      DATA          RTNAME /'RESVAL'/
C
      IERR = 0
C
      if (DXMIN.ge.DXMAX) then
        IERR = 2
        LOG = 'Error in input parameters DXMIN,DXMAX'
        call WRERR(RTNAME,IERR,LOG)
      endif
C
      if (IERR.lt.2) then
        DDIFF = DXMAX - DXMIN
        DH = (DXOLD-DXMIN) / DDIFF
        DH = DH - DINT(DH)
        if (DH.lt.0.D0) DH = DH + 1.D0
        DXNEW = DXMIN + DH * DDIFF
      endif
C
      RETURN
      END
c
c
c
        subroutine retfit(infile,rarray,i2array,i4array,
     +      iflag,isize1,isize2,ctype,crval,cdelt,crpix,status)
c
c  INFILE    file name of input file
c  RARRAY    REAL*4 array for input
c  I2ARRAY   INTEGER*2 array for input
c  I4ARRAY   INTEGER*4 array for input
c  IFLAG     flag for input file type selection
c            1 => R*4 array
c            2 => I*2 array
c            3 => I*4 array
c  ISIZE1    array dimension 1
c  ISIZE2    array dimension 2
c  CRVAL1    reference coordinate for first axis (RA)
c  CDELT1    pixel size for first axis
c  CRVAL2    reference coordinate for second axis (Dec)
c  CDELT2    pixel size for second axis
c  STATUS    FITSIO status flag
c
        implicit none
c
        integer isize1, isize2
c
        integer*2 i2array(isize1,isize2)
        integer bitpix, block, err, hdutype, 
     +      i4array(isize1,isize2), iflag, 
     +      gcount, naxes(10), naxis, lun, pcount, status
        real    rarray(isize1,isize2)
        character*8 ctype(2)
        character*80 com
        character*132 infile
        double precision cdelt(2), crpix(2), crval(2)
        logical anyf, extend, simple
c
        status = 0
        err = 0
c
c  Get an unused lun
c
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 1'
            print *, 'STATUS FTGIOU =', status
        endif
c
c  Read the array
c
        print *, 'Reading input file'
        print *, trim(infile)
c
c  Open the FITS file for the first observation exposure
c
        call ftopen(lun,infile,0,block,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 2'
            print *, 'STATUS FTOPEN =', status
            print *, infile
            stop
        endif
c
c  Read the FITS file header information
c
        call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +                      pcount,gcount,extend,status)
c
c  Check to see if the primary has the image
c
        if(naxis .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            call ftghpr (lun,10,simple,bitpix,naxis,naxes,
     +                      pcount,gcount,extend,status)
        endif
c
        if((isize1 .ne. naxes(1)) .or. (isize2 .ne. naxes(2))) then
            print *, 'RETFIT POSITION 3a, STATUS = ', STATUS
            print *, infile
            print *, 'Input array size not equal to image'
            print *, isize1, isize2, ' --  ', naxes(1), naxes(2)
            stop
        endif
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 3b'
            print *, 'STATUS FTGHPR =', status
            print *, infile
            stop
        endif
c
c  Get the useful keywords
c
        call ftgkys(lun,'CTYPE1',ctype(1),com,status)
        call ftgkyd(lun,'CRVAL1',crval(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CTYPE1'
            stop
        endif
        call ftgkyd(lun,'cdelt1',cdelt(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CDELT1'
            stop
        endif
        call ftgkyd(lun,'crpix1',crpix(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRPIX1'
            stop
        endif
        call ftgkys(lun,'CTYPE2',ctype(2),com,status)
        call ftgkyd(lun,'CRVAL2',crval(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRVAL2'
            stop
        endif
        call ftgkyd(lun,'CDELT2',cdelt(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CDELT2'
            stop
        endif
        call ftgkyd(lun,'CRPIX2',crpix(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRPIX2'
            stop
        endif
c
c  Read in an R*4 file
c
        if(iflag .eq. 1) then
            call ftg2de (lun,0,0,naxes(1),naxes(1),naxes(2),rarray,
     +              anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'RETFIT POSITION 4'
                print *, 'STATUS FTG2DE =', status
                print *, infile
            endif
c
c  Read in an I*2 file
c
        elseif(iflag .eq. 2) then
            call ftg2di (lun,0,0,naxes(1),naxes(1),naxes(2),i2array,
     +              anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'RETFIT POSITION 5'
                print *, 'STATUS FTG2DI =', status
                print *, infile
            endif
c
c  Read in an I*4 file
c
        elseif(iflag .eq. 3) then
            call ftg2dj (lun,0,0,naxes(1),naxes(1),naxes(2),i4array,
     +              anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'RETFIT POSITION 6'
                print *, 'STATUS FTG2DJ =', status
                print *, infile
            endif
        endif
c
c  Close the FITS file
c
        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 7'
            print *, 'STATUS FTCLOS =', status
            print *, infile
        endif
c
c  Clean up
c
        call ftfiou (lun,status)
c
        return
        end
c
c
c
        SUBROUTINE ROTPOL (CENLON,CENLAT,INLO,INLA,OUTLO,OUTLA,MODE)
C
CC  Temp
C
C************************ FFORM VERSION 1.0 ************ 17-MAR-89 09:53
C
CA  author : SLS               date:
CU  update : SLS               date: 17-MAR-1989 09:53
C
CT  status: not tested
C
C   general description
CG  ROTPOL returns coordinates of the direction given transformed
CG  to a polar system where up is at the old CENLON,CENLAT.
C
C   call_var.          type I/O description
CP  CENLON              R4  I   longitude in degrees for new north pole
CP  CENLAT              R4  I   latitude in degrees for new north pole
CP  INLO                R4  ?/? longitude of position to be rotated
CP  INLA                R4  ?/? latitude of position to be rotated
CP  OUTLO               R4  ?/? longitude of position in rotated coords
CP  OUTLA               R4  ?/? latitude of position in rotated coords
CP  MODE                I4  I/? = 1 rotate to prime (new) system
CP                              = -1 rotate from prime (new) system
C
C   include_block name          description
C
C   routines_called    type     description
CR  ELAZ8               SR
CR  EULROT8             SR
CR  VECT8               SR
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        INTEGER MODE
C
        REAL CENLAT, CENLON, INLA, INLO, OUTLA, OUTLO
        DOUBLE PRECISION DEGRAD, DINLA, DINLO, DOUTLA, DOUTLO, 
     +      E1, E2, E3, V(3), V1(3)
C
        DATA DEGRAD /0.01745329252D0/
C
        E1 = (90. + CENLON)
        IF(E1 .GT. 360.) E1 = E1 - 360.
        E1 = E1*DEGRAD
        E2 = (90. - CENLAT)*DEGRAD
        E3 = 270.*DEGRAD
        DINLA = INLA
        DINLO = INLO
C
        IF(MODE .NE. 1) THEN
            CALL VECT8 (DINLA, DINLO, V)
            CALL EULROT8 (V1,V,E1,E2,E3,MODE)
            CALL ELAZ8 (V1,DOUTLA,DOUTLO)
        ELSE
            CALL VECT8 (DINLA, DINLO, V1)
            CALL EULROT8 (V1,V,E1,E2,E3,MODE)
            CALL ELAZ8 (V,DOUTLA,DOUTLO)
        ENDIF
C
        OUTLA = DOUTLA
        OUTLO = DOUTLO
C
        RETURN
        END
c
c
c
        subroutine smooth (rate,smoo,nchan,smoo_rate)
c
        implicit none
c
        integer i, ih, j, j1, j2, nchan, smoo
        real n, rate(nchan), smoo_rate(nchan), t
c
        ih = smoo/2
        do i=1,nchan
            j1 = i - ih
            if(j1 .lt. 1) j1 = 1
            j2 = i + ih
            if(j2 .gt. nchan) j2 = nchan
            t = 0.0
            n = 0.0
            do j=j1,j2
                 if(rate(j) .gt. 0) then
                    t = t + rate(j)
                    n = n + 1.0
                endif
            enddo
            if(n .gt. 0) then
                smoo_rate(i) = t/n
            else
                smoo_rate(i) = 0.0
            endif
        enddo
c
        return
        end
c
c
c
        SUBROUTINE SOLVE (MP,NP,UU,YY,SSIG,MA,NA,AA,CCHISQ,CHICON,CVM)
C
        IMPLICIT NONE
C
        INTEGER I, II, J, MA, MP, NA, NP
        REAL AA(NP), CCHISQ, CHICON(MP), UU(MP,NP), YY(MP),
     +          SSIG(MP), TOL
        DOUBLE PRECISION A(1500), B(25000), CHISQ, CHITMP, CVM(NP,NP),
     +          SIG(25000), SUM, THRESH, TMP, U(25000,1500),
     +          V(1500,1500), W(1500), WMAX, Y(25000)
C
        TOL = 1.E-5
C
        PRINT *, 'SOLVE: Position 1'
        DO I=1,MA
            Y(I) = YY(I)
            SIG(I) = SSIG(I)
            DO II=1,NA
               U(I,II) = UU(I,II)
            ENDDO
        ENDDO
C
        DO I=1,MA
            TMP = 1./SIG(I)
            DO J=1,NA
                U(I,J) = U(I,J)*TMP
            ENDDO
            B(I) = Y(I)*TMP
        ENDDO
C
        CALL SVDCMP(U,MA,NA,MP,NP,W,V)
C
        WMAX = 0.
C
        DO J=1,NA
            IF(W(J) .GT. WMAX) WMAX = W(J)
        ENDDO
C
        THRESH = TOL*WMAX
C
        DO J=1,NA
            IF(W(J) .LT. THRESH) W(J) = 0.
        ENDDO
C
        CALL SVBKSB(U,W,V,MA,NA,MP,NP,B,A)
C
        CALL SVDVAR(V,NA,NP,W,CVM)
C
C Calculate (UU*AA-YY)**2 for the fit as well as
C the contribution from each equation
C
        CHISQ = 0.
C
        DO I=1,MA
            SUM = 0.
            DO J=1,NA
                SUM = SUM + A(J)*UU(I,J)
            ENDDO
            CHITMP = ((Y(I)-SUM)/SIG(I))**2
            IF(CHITMP .GT. 10.) THEN
                PRINT 2000, I, CHITMP, Y(I), SIG(I), SUM
 2000           FORMAT(1X,I5,4F10.2)
            ENDIF
            CHISQ = CHISQ + CHITMP
            CHICON(I) = CHITMP
        ENDDO
C
        DO I=1,NA
            AA(I) = A(I)
        ENDDO
        CCHISQ = CHISQ
C
        RETURN
        END
c
c
c
        subroutine spscale(xmmdir,xmmd,mosnum,
     +          maskfile,specfile,back,c)
c
c  Date: 15 November 2006
c
c  Calculates the relative contribution of SP flares by region
c
c  spscale uses SP flare detector maps and region images to
c  find the average SP flare contribution over the region.  It
c  also returns the BACKSCALE value from the spectrum for the
c  region.
c  Input:
c    QPB data directory
c    MOS number
c    Mask image file name
c    Spectrum file name
c  Output:
c    BACKSCAL value for the region
c    Average SP intensity for the region
c
        implicit none
c
        integer*2 i2array(780,780)
c
        integer block, err, hdutype, i4array(780,780), i, ii, iii, 
     +      lunspec, mosnum, n, nchan, status, tfields, varidat
c
        real back, c, mask(780,780), sp(780,780)
c
        double precision cdelt(2), crpix(2), crval(2)
c
        character*1 db_fi(80), sp_fi(25), xmmd(80)
        character*8 ctype(2), tform(10), ttype(10), tunit(10)
        character*25 sp_file
        character*20 extname
        character*60 com, maskfile, specfile
        character*80 db_file, xmmdir
        character*132 tempfile
c
        equivalence (db_file,db_fi)
        equivalence (sp_file,sp_fi)
c
        data err /0/
        data db_file /'     '/ 
        data status /0/
c
c  Set the SP detector maps
c
        if(mosnum .eq. 1) then
            sp_file = '/mos1-sp-tot.fits.gz'
        elseif(mosnum .eq. 2) then
            sp_file = '/mos1-sp-tot.fits.gz'
        else
            sp_file = '/pn-sp-tot.fits.gz'
        endif
c
c  Create the SP detector map FITS file name including the
c  directory string
c
        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,24
            db_fi(ii+iii) = sp_fi(iii+1)
        enddo
        print *,'db_fi: ',db_fi
c
c  Read in the SP image
c
        tempfile = db_file
        call retfit(tempfile,sp,i2array,i4array,
     +              1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            print *, 'Cant find SP image'
            print *, db_file
            stop
        endif
c
c  Read in the mask image
c
        tempfile = maskfile
        call retfit(tempfile,mask,i2array,i4array,
     +              1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            print *, 'Cant find mask image'
            print *, maskfile
            stop
        endif
c
c  Get an unused lun for the spectrum
c
        call ftgiou(lunspec,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 01'
        endif
c
c  Open the mask spectrum FITS file
c
        print *, specfile
        call ftopen (lunspec,specfile,0,block,status)
        if(status .ne. 0) then
            err = 1
            print *, 'ROUTINE SPSCALE'
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 02'
            print *, specfile
        endif
c
c  Move to the spectrum extension
c
        if(err .eq. 0) then
            call ftmrhd (lunspec,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 03'
                print *, 'Could not find the spectrum extension'
                print *, specfile
            else
c
c  Read the FITS file header information
c
                call ftghbn (lunspec,10,nchan,tfields,
     +                  ttype,tform,tunit,extname,varidat,status)
                if(extname .ne. 'SPECTRUM') then
                    print *, extname
                    print *, 'This was supposed to be a spectrum ext'
                    stop
                else
c
c  Read in the header information
c
                    call ftgkye (lunspec,'BACKSCAL',back,com,status)
                    print *, back
                    if(status .ne. 0) then
                        print *, 'Missing BACKSCAL keyword'
                        status = 0
                        err = 2
                    endif
                endif
            endif
        endif
        call ftclos (lunspec,status)
        back = back/1440000.0
c
c  Loop over the image
c
        n = 0
        c = 0
        do i=1,780
            do ii=1,780
                if (mask(i,ii) .gt. 0) then
                    n = n + 1
                    c = c + sp(i,ii)
                endif
            enddo
        enddo
        c = c/n
c
        return
        end
c
c
c
        SUBROUTINE SVDCMP(A,M,N,MP,NP,W,V)
C
C        IMPLICIT NONE
C
        INTEGER I, ITS, J, K, L, M, MP, N, NM, NP
        PARAMETER (NMAX=1500)
        DOUBLE PRECISION A(MP,NP), ANORM, C, F, G, H, S, SCALE,
     +      W(NP), V(NP,NP), RV1(NMAX), X, Y, Z
C
        G = 0.0
        SCALE = 0.0
        ANORM = 0.0
        DO I=1,N
            L = I + 1
            RV1(I) = SCALE*G
            G = 0.0
            S = 0.0
            SCALE = 0.0
            IF (I.LE.M) THEN
                DO K=I,M
                    SCALE = SCALE + DABS(A(K,I))
                ENDDO
                IF (SCALE .NE. 0.0) THEN
                    DO K=I,M
                        A(K,I) = A(K,I)/SCALE
                        S = S + A(K,I)*A(K,I)
                    ENDDO
                    F = A(I,I)
                    G = -DSIGN(DSQRT(S),F)
                    H = F*G - S
                    A(I,I) = F - G
                    IF (I .NE. N) THEN
                        DO J=L,N
                            S = 0.0
                            DO K=I,M
                                S = S + A(K,I)*A(K,J)
                            ENDDO
                            F = S/H
                            DO K=I,M
                                A(K,J) = A(K,J) + F*A(K,I)
                            ENDDO
                        ENDDO
                    ENDIF
                    DO K= I,M
                        A(K,I) = SCALE*A(K,I)
                    ENDDO
                ENDIF
            ENDIF
            W(I) = SCALE*G
            G = 0.0
            S = 0.0
            SCALE = 0.0
            IF ((I .LE. M) .AND. (I .NE. N)) THEN
                DO K=L,N
                    SCALE = SCALE + DABS(A(I,K))
                ENDDO
                IF (SCALE .NE. 0.0) THEN
                    DO K=L,N
                        A(I,K) = A(I,K)/SCALE
                        S = S + A(I,K)*A(I,K)
                    ENDDO
                    F = A(I,L)
                    G = -DSIGN(DSQRT(S),F)
                    H = F*G - S
                    A(I,L) = F - G
                    DO K=L,N
                        RV1(K) = A(I,K)/H
                    ENDDO
C
                    IF (I .NE. M) THEN
                        DO J=L,M
                            S = 0.0
                            DO K=L,N
                                S = S + A(J,K)*A(I,K)
                            ENDDO
                            DO K=L,N
                                A(J,K) = A(J,K) + S*RV1(K)
                            ENDDO
                        ENDDO
                    ENDIF
                    DO K=L,N
                        A(I,K) = SCALE*A(I,K)
                    ENDDO
                ENDIF
            ENDIF
            ANORM = MAX(ANORM,(DABS(W(I))+DABS(RV1(I))))
        ENDDO
C
        DO I=N,1,-1
            IF (I .LT. N) THEN
                IF (G .NE. 0.0) THEN
                    DO J=L,N
                        V(J,I) = (A(I,J)/A(I,L))/G
                    ENDDO
                    DO J=L,N
                        S = 0.0
                        DO K=L,N
                            S = S + A(I,K)*V(K,J)
                        ENDDO
                        DO K=L,N
                            V(K,J) = V(K,J) + S*V(K,I)
                        ENDDO
                    ENDDO
                ENDIF
                DO J=L,N
                    V(I,J) = 0.0
                    V(J,I) = 0.0
                ENDDO
            ENDIF
            V(I,I) = 1.0
            G = RV1(I)
            L = I
        ENDDO
C
        DO I=N,1,-1
            L = I + 1
            G = W(I)
            IF (I.LT.N) THEN
                DO J=L,N
                    A(I,J) = 0.0
                ENDDO
            ENDIF
            IF (G .NE. 0.0) THEN
                G = 1.0/G
                IF (I .NE. N) THEN
                    DO J=L,N
                        S = 0.0
                        DO K=L,M
                            S = S + A(K,I)*A(K,J)
                        ENDDO
                        F = (S/A(I,I))*G
                        DO K=I,M
                            A(K,J) = A(K,J) + F*A(K,I)
                        ENDDO
                    ENDDO
                ENDIF
                DO J=I,M
                    A(J,I) = A(J,I)*G
                ENDDO
            ELSE
                DO J= I,M
                    A(J,I) = 0.0
                ENDDO
            ENDIF
            A(I,I) = A(I,I) + 1.0
        ENDDO
C
        DO K=N,1,-1
            DO ITS=1,30
                DO L=K,1,-1
                    NM = L - 1
                    IF ((ABS(RV1(L))+ANORM).EQ.ANORM)  GO TO 2
                    IF ((ABS(W(NM))+ANORM).EQ.ANORM)  GO TO 1
                ENDDO
    1           C = 0.0
                S = 1.0
                DO I=L,K
                    F = S*RV1(I)
                    IF ((ABS(F)+ANORM) .NE. ANORM) THEN
                        G = W(I)
                        H = SQRT(F*F+G*G)
                        W(I) = H
                        H = 1.0/H
                        C = (G*H)
                        S = -(F*H)
                        DO J=1,M
                            Y = A(J,NM)
                            Z = A(J,I)
                            A(J,NM) = (Y*C)+(Z*S)
                            A(J,I) = -(Y*S)+(Z*C)
                        ENDDO
                    ENDIF
                ENDDO
    2           Z = W(K)
                IF (L .EQ. K) THEN
                    IF (Z .LT. 0.0) THEN
                        W(K) = -Z
                        DO J=1,N
                            V(J,K) = -V(J,K)
                        ENDDO
                    ENDIF
                    GO TO 3
                ENDIF
                IF (ITS .EQ. 3000) PRINT *, 
     +              'No convergence in 3000 iterations'
                X = W(L)
                NM = K-1
                Y = W(NM)
                G = RV1(NM)
                H = RV1(K)
                F = ((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
                G = SQRT(F*F+1.0)
                F = ((X-Z)*(X+Z)+H*((Y/(F+SIGN(G,F)))-H))/X
                C = 1.0
                S = 1.0
                DO J=L,NM
                    I = J + 1
                    G = RV1(I)
                    Y = W(I)
                    H=S*G
                    G=C*G
                    Z=SQRT(F*F+H*H)
                    RV1(J)=Z
                    C=F/Z
                    S=H/Z
                    F= (X*C)+(G*S)
                    G=-(X*S)+(G*C)
                    H=Y*S
                    Y=Y*C
                    DO NM=1,N
                        X=V(NM,J)
                        Z=V(NM,I)
                        V(NM,J)= (X*C)+(Z*S)
                        V(NM,I)=-(X*S)+(Z*C)
                    ENDDO
                    Z=SQRT(F*F+H*H)
                    W(J)=Z
                    IF (Z.NE.0.0) THEN
                        Z=1.0/Z
                        C=F*Z
                        S=H*Z
                    ENDIF
                    F= (C*G)+(S*Y)
                    X=-(S*G)+(C*Y)
                    DO NM=1,M
                        Y=A(NM,J)
                        Z=A(NM,I)
                        A(NM,J)= (Y*C)+(Z*S)
                        A(NM,I)=-(Y*S)+(Z*C)
                    ENDDO
                ENDDO
                RV1(L)=0.0
                RV1(K)=F
                W(K)=X
            ENDDO
    3       CONTINUE
        ENDDO
C
        RETURN
        END
C
C ***
C
        SUBROUTINE SVBKSB(U,W,V,M,N,MP,NP,B,X)
C
C        IMPLICIT NONE
C
        PARAMETER (NMAX=1500)
        INTEGER I, J, JJ, M, MP, N, NP
        DOUBLE PRECISION S, U(MP,NP), W(NP), V(NP,NP), B(MP), 
     +      X(NP), TMP(NMAX)
C
        DO J=1,N
            S = 0.
            IF(W(J) .NE. 0.)THEN
                DO I=1,M
                    S = S + U(I,J)*B(I)
                ENDDO
                S = S/W(J)
            ENDIF
            TMP(J) = S
        ENDDO
C
        DO J=1,N
            S = 0.
            DO JJ=1,N
                S = S + V(J,JJ)*TMP(JJ)
            ENDDO
            X(J) = S
        ENDDO
C
        RETURN
        END
C
C
C ***
C
        SUBROUTINE SVDVAR(V,MA,NP,W,CVM)

        INTEGER MA, NP, MMAX
        DOUBLE PRECISION CVM(NP,NP), V(NP,NP), W(NP)
        PARAMETER (MMAX=1500)
        INTEGER I, J, K
        DOUBLE PRECISION SUM, WTI(MMAX)

        DO I=1,MA
          WTI(I) = 0.
          IF(W(I) .NE. 0.) WTI(I)=1./(W(I)*W(I))
        ENDDO

        DO I=1,MA
          DO J=1,I
            SUM = 0.
            DO K=1,MA
              SUM = SUM + V(I,K)*V(J,K)*WTI(K)
            ENDDO
            CVM(I,J) = SUM
            CVM(J,I) = SUM
          ENDDO
        ENDDO

        RETURN
        END
c
c
c
        subroutine swcxscale(xmmdir,xmmd,mosnum,line,
     +          maskfile,specfile,back,c)
c
c  Date: 23 June 2010
c
c  Calculates the relative contribution of SP flares by region
c
c  spscale uses SP flare detector maps and region images to
c  find the average SP flare contribution over the region.  It
c  also returns the BACKSCALE value from the spectrum for the
c  region.
c  Input:
c    QPB data directory
c    MOS number
c    Mask image file name
c    Spectrum file name
c  Output:
c    BACKSCAL value for the region
c    Average SP intensity for the region
c
        implicit none
c
        integer*2 i2array(780,780)
c
        integer block, err, hdutype, i4array(780,780), i, ii, iii, 
     +      line, lunspec, mosnum, n, nchan, status, tfields, varidat
c
        real back, c, mask(780,780), swcx(780,780)
c
        double precision cdelt(2), crpix(2), crval(2)
c
        character*1 db_fi(80), swcx_fi(25), xmmd(80)
        character*8 ctype(2), tform(10), ttype(10), tunit(10)
        character*25 swcx_file
        character*20 extname
        character*60 com, maskfile, specfile
        character*80 db_file, xmmdir
        character*132 tempfile
c
        equivalence (db_file,db_fi)
        equivalence (swcx_file,swcx_fi)
c
        data err /0/
        data db_file /'     '/ 
        data status /0/
c
c  Set the SP detector maps
c
        if(mosnum .eq. 1) then
            if(line .eq. 1) then
                swcx_file = '/mos1-ovii-exp.fits.gz'
            elseif(line .eq. 2) then
                swcx_file = '/mos1-oviii-exp.fits.gz'
            elseif(line .eq. 3) then
                swcx_file = '/mos1-neix-exp.fits.gz'
            elseif(line .eq. 4) then
                swcx_file = '/mos1-mgxi-exp.fits.gz'
            else
                print *, 'SWCXSCALE Position 1'
                print *, 'MOS1  ', line
                print *, 'No exposure map for this SWCX line'
                stop
            endif
        elseif(mosnum .eq. 2) then
            if(line .eq. 1) then
                swcx_file = '/mos2-ovii-exp.fits.gz'
            elseif(line .eq. 2) then
                swcx_file = '/mos2-oviii-exp.fits.gz'
            elseif(line .eq. 3) then
                swcx_file = '/mos2-neix-exp.fits.gz'
            elseif(line .eq. 4) then
                swcx_file = '/mos2-mgxi-exp.fits.gz'
            else
                print *, 'SWCXSCALE Position 2'
                print *, 'MOS2  ', line
                print *, 'No exposure map for this SWCX line'
                stop
            endif
        else
            if(line .eq. 1) then
                swcx_file = '/pn-ovii-exp.fits.gz'
            elseif(line .eq. 2) then
                swcx_file = '/pn-oviii-exp.fits.gz'
            elseif(line .eq. 3) then
                swcx_file = '/pn-neix-exp.fits.gz'
            elseif(line .eq. 4) then
                swcx_file = '/pn-mgxi-exp.fits.gz'
            else
                print *, 'SWCXSCALE Position 3'
                print *, 'PN  ', line
                print *, 'No exposure map for this SWCX line'
                stop
            endif
        endif
c
c  Create the SP detector map FITS file name including the
c  directory string
c
        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,24
            db_fi(ii+iii) = swcx_fi(iii+1)
        enddo
        print *,'db_fi: ',db_fi
c
c  Read in the SP image
c
        tempfile = db_file
        call retfit(tempfile,swcx,i2array,i4array,
     +              1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            print *, 'Cant find exposure image'
            print *, db_file
            stop
        endif
c
c  Read in the mask image
c
        tempfile = maskfile
        call retfit(tempfile,mask,i2array,i4array,
     +              1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            print *, 'Cant find mask image'
            print *, maskfile
            stop
        endif
c
c  Get an unused lun for the spectrum
c
        call ftgiou(lunspec,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 01'
        endif
c
c  Open the mask spectrum FITS file
c
        print *, specfile
        call ftopen (lunspec,specfile,0,block,status)
        if(status .ne. 0) then
            err = 1
            print *, 'ROUTINE SPSCALE'
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 02'
            print *, specfile
        endif
c
c  Move to the spectrum extension
c
        if(err .eq. 0) then
            call ftmrhd (lunspec,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 03'
                print *, 'Could not find the spectrum extension'
                print *, specfile
            else
c
c  Read the FITS file header information
c
                call ftghbn (lunspec,10,nchan,tfields,
     +                  ttype,tform,tunit,extname,varidat,status)
                if(extname .ne. 'SPECTRUM') then
                    print *, extname
                    print *, 'This was supposed to be a spectrum ext'
                    stop
                else
c
c  Read in the header information
c
                    call ftgkye (lunspec,'BACKSCAL',back,com,status)
                    print *, back
                    if(status .ne. 0) then
                        print *, 'Missing BACKSCAL keyword'
                        status = 0
                        err = 2
                    endif
                endif
            endif
        endif
        call ftclos (lunspec,status)
        back = back/1440000.0
c
c  Loop over the image
c
        n = 0
        c = 0
        do i=1,780
            do ii=1,780
                if (mask(i,ii) .gt. 0) then
                    n = n + 1
                    c = c + swcx(i,ii)
                endif
            enddo
        enddo
        c = c/n
c
        return
        end
c
c
c
        SUBROUTINE VECT8 (EL, AZ, V)
C
CC Transforms elavation and azimuth to a unit vector
C
C************************ FFORM VERSION 1.0 ************ 17-MAR-89 10:24
C
CA  author : Rick Borken       date: Nov 1972
CU  update : Dave Burrows      date: Fall 1981
CU  update : SLS               date: 17-MAR-1989 10:23
C
CT  status: not tested
C
C   general description
CG  VECT returns the components of a unit vector in the direction
CG  with given elevation and azimuth
C
C   call_var.          type I/O description
CP  EL                  R4      elevation angle in degrees
CP  AZ                  R4  I   azimuth angle in degrees
CP  V                   R4    O unit vector in direction of EL, AZ
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        DOUBLE PRECISION A, AZ, B, DR, EL, V(3)
C
        DATA DR /0.01745329252D0/
C
        A = EL*DR
        B = AZ*DR
        V(1) = DCOS(A)*DCOS(B)
        V(2) = DCOS(A)*DSIN(B)
        V(3) = DSIN(A)
C
        RETURN
        END
c
c
c
        SUBROUTINE WRERR (RTNAME,IERR,TEXT)
C
        INTEGER IERR
        CHARACTER*8 RTNAME
        CHARACTER*80 TEXT
C
        print *, 'Error in Routine: ', RTNAME
        print *, 'Error Number: ', IERR
        print *, text
C
        RETURN
        END
c
c
c
        SUBROUTINE XYPOLE(ALAT,ALONG,X,Y,IPOS)
C
CC  XYPOLE calculates a polar equal area-solid angle projection.
C
C************************ FFORM VERSION 1.0 ************ 17-MAR-89 12:56
C
CA  author : Rick Borken       date:    DEC-1975
CU  update : Steve Holz        date:  1-APR-82
CU  update : SLS               date: 29-MAR-1989 10:08
C
CT  status: not tested
C
C   general description
CG  For each input latitude, longitude point (in degrees), a value of
CG  X and Y are returned (in inches).  The X,Y grid has its origin at
CG  the pole (latitude = +-90.) and is scaled so that the diameter of
CG  the equator is 81. units and one degree is one unit.
CG  The input integer IPOS is used to specify which hemisphere is to be
CG  plotted.  If IPOS=1,0 the positive, negative pole is centered
CG  respectively.  Longitude increases clockwise for a north polar plot
CG  and counterclockwise for a south polar plot.  Zero longitude is at
CG  the top of the page.
C
C   call_var.          type I/O description
CP  ALAT                R4  I   latitude to be cast in degrees
CP  ALONG               R4  I   longitude to be cast in degrees
CP  IPOS                I4  I   casting control = 1 positive pole
CP                                              = 0 negative pole
CP  X                   R4    O X coord of cast point
CP  Y                   R4    O Y coord of cast point
C
C   include_block name          description
C
C   routines_called    type     description
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
        IMPLICIT NONE
C
        INTEGER IPOS
        REAL ALAT, ALONG, X, Y
        DOUBLE PRECISION DR, R, RLAT, RLONG, SQ
C
        DATA DR /0.01745329252D0/
C
        RLAT = ALAT*DR
        RLONG = ALONG*DR
        IF(IPOS .EQ. 0) THEN
            RLAT = -1.*RLAT
            RLONG = -1.*RLONG
        ENDIF
        SQ = 2.D0*(1.D0-DSIN(RLAT))
        IF(SQ .LT. 0.D0) SQ = 0.D0
C
C  The following line makes one degree at 1 and the equator at 81.0295
C
        R = 57.29650742D0*DSQRT(SQ)
C
C  The following line makes one degree at 0.061706 and the equator at 5
C       R = 3.53553391*SQRT(SQ)
C
        Y = R*DCOS(RLONG)
        X = R*DSIN(RLONG)
C
        RETURN
        END
